Package: date4ts
Title: Wrangle and Modify Ts Object with Classic Frequencies and Exact
        Dates
Version: 0.1.2
Authors@R: 
    person("Tanguy", "Barthelemy", email = "tangbarth@hotmail.fr", role = c("aut", "cre"))
Description: The ts objects in R are managed using a very specific date format (in the form c(2022, 9) for September 2022 or c(2021, 2) for the second quarter of 2021, depending on the frequency, for example). We focus solely on monthly and quarterly series to manage the dates of ts objects. The general idea is to offer a set of functions to manage this date format without it being too restrictive or too imprecise depending on the rounding. This is a compromise between simplicity, precision and use of the basic 'stats' functions for creating and managing time series (ts(), window()).
    Les objets ts en R sont gérés par un format de date très particulier (sous la forme c(2022, 9) pour septembre 2022 ou c(2021, 2) pour le deuxième trimestre 2021 selon la fréquence par exemple). On se concentre uniquement sur les séries mensuelles et trimestrielles pour gérer les dates des objets ts. Lidée générale est de proposer un ensemble de fonctions pour gérer ce format de date sans que ce soit trop contraignant ou trop imprécis selon les arrondis. Cest un compromis entre simplicité, précision et utilisation des fonctions du package 'stats' de création et de gestion des séries temporelles (ts(), window()).
License: GPL (>= 3)
URL: https://github.com/TractorTom/date4ts,
        https://tractortom.github.io/date4ts/
BugReports: https://github.com/TractorTom/date4ts/issues
Encoding: UTF-8
Language: fr-FR
RoxygenNote: 7.3.3
Imports: stats, checkmate
Depends: R (>= 4.1)
LazyData: true
Suggests: testthat (>= 3.0.0), renv, fuzzr, pkgdown, devtools, usethis,
        covr, withr, altdoc
Config/testthat/edition: 3
NeedsCompilation: no
Packaged: 2026-04-29 14:07:09 UTC; onyxia
Author: Tanguy Barthelemy [aut, cre]
Maintainer: Tanguy Barthelemy <tangbarth@hotmail.fr>
Repository: CRAN
Date/Publication: 2026-04-29 15:20:02 UTC
Built: R 4.6.0; ; 2026-04-29 15:57:17 UTC; unix

# Generated by roxygen2: do not edit by hand

export(as_yyyymm)
export(as_yyyytt)
export(assert_date_ts)
export(assert_expression)
export(assert_frequency)
export(assert_scalar_date)
export(assert_scalar_integer)
export(assert_scalar_natural)
export(assert_timeunits)
export(assert_ts)
export(check_date_ts)
export(check_expression)
export(check_frequency)
export(check_scalar_date)
export(check_scalar_integer)
export(check_scalar_natural)
export(check_timeunits)
export(check_ts)
export(combine2ts)
export(date2date_ts)
export(date_ts2date)
export(date_ts2timeunits)
export(diff_periode)
export(extend_ts)
export(first_date)
export(get_value_ts)
export(is_before)
export(last_date)
export(libelles)
export(mens2trim)
export(na_trim)
export(next_date_ts)
export(normalize_date_ts)
export(previous_date_ts)
export(set_value_ts)
export(substr_year)
export(trim2mens)
export(ts2df)
importFrom(stats,frequency)

#' @title Obtenir la date précédente
#'
#' @inheritParams date_ts2timeunits
#' @param lag un entier
#'
#' @returns En sortie, la fonction retourne un vecteur d'entier qui représente
#' la date à la période passée au format `date_ts`.
#'
#' @details Lorsqu'on parle de date précédente, on parle de date passée.
#' L'argument `lag` est entier et désigne le nombre de décalage que l'on affecte
#' à notre date.
#' Par exemple pour des lag positif (1L, 2L, 10L) on désigne le décalage de la
#' période précédente, celle d'avant et celle d'il y a 10 périodes.
#' Cependant, lorsque l'argument `lag` vaut zéro, la fonction retourne la
#' `date` inchangée. Aussi lorsque l'argument `lag` est négatif, la fonction se
#' comporte comme la fonction `next_date_ts` et retourne les périodes futures et
#' non passées.
#' @export
#'
#' @seealso `next_date_ts`
#'
#' @examples
#'
#' previous_date_ts(c(2020L, 4L), frequency_ts = 4L, lag = 2L)
#' previous_date_ts(c(2021L, 1L), frequency_ts = 4L, lag = -2L)
#'
#' previous_date_ts(c(2020L, 4L), frequency_ts = 12L, lag = 2L)
#' previous_date_ts(c(2022L, 6L), frequency_ts = 12L, lag = 12L)
previous_date_ts <- function(date_ts, frequency_ts, lag = 1L) {
    coll <- checkmate::makeAssertCollection()

    # Check de la fréquence
    frequency_ts <- assert_frequency(
        frequency_ts,
        add = coll,
        .var.name = "frequency_ts"
    )

    # Check du format date_ts
    if (isTRUE(check_frequency(frequency_ts, warn = FALSE))) {
        date_ts <- assert_date_ts(
            x = date_ts,
            frequency_ts,
            add = coll,
            .var.name = "date_ts"
        )
    }

    # Check l'argument lag
    lag <- assert_scalar_integer(lag, add = coll, .var.name = "lag")

    checkmate::reportAssertions(coll)

    year <- date_ts[1L]
    period <- date_ts[2L] - lag

    return(normalize_date_ts(
        date_ts = c(year, period),
        frequency_ts = frequency_ts,
        test = FALSE
    ))
}

#' @title Obtenir la date suivante
#'
#' @inheritParams previous_date_ts
#'
#' @returns En sortie, la fonction retourne un vecteur d'entier qui représente
#' la date à la période future au format `date_ts`.
#'
#' @details Lorsqu'on parle de date suivante, on parle de date future.
#' L'argument `lag` est entier et désigne le nombre de décalage que l'on affecte
#' à notre date.
#' Par exemple pour des lag positif (1L, 2L, 10L) on désigne le décalage de la
#' période suivante, celle d'après et celle dans 10 périodes.
#' Cependant, lorsque l'argument `lag` vaut zéro, la fonction retourne la
#' `date` inchangée. Aussi lorsque l'argument `lag` est négatif, la fonction
#' se comporte comme la fonction `previous_date_ts` et retourne les périodes
#' passées et non futures.
#' @export
#'
#' @seealso `previous_date_ts`
#'
#' @examples
#'
#' next_date_ts(c(2020L, 4L), frequency_ts = 4L, lag = 2L)
#' next_date_ts(c(2021L, 1L), frequency_ts = 4L, lag = -2L)
#'
#' next_date_ts(c(2020L, 4L), frequency_ts = 12L, lag = 2L)
#' next_date_ts(c(2022L, 6L), frequency_ts = 12L, lag = 12L)
next_date_ts <- function(date_ts, frequency_ts, lag = 1L) {
    coll <- checkmate::makeAssertCollection()

    # Check de la fréquence
    frequency_ts <- assert_frequency(
        frequency_ts,
        add = coll,
        .var.name = "frequency_ts"
    )

    # Check du format date_ts
    if (isTRUE(check_frequency(frequency_ts, warn = FALSE))) {
        date_ts <- assert_date_ts(
            x = date_ts,
            frequency_ts,
            add = coll,
            .var.name = "date_ts"
        )
    }

    # Check l'argument lag
    lag <- assert_scalar_integer(x = lag, add = coll, .var.name = "lag")

    checkmate::reportAssertions(coll)

    year <- date_ts[1L]
    period <- date_ts[2L] + lag

    return(normalize_date_ts(
        date_ts = c(year, period),
        frequency_ts = frequency_ts,
        test = FALSE
    ))
}

#' @title Première date non NA
#'
#' @description Cette fonction calcule la première date pour laquelle l'objet
#' `series` ne vaut pas NA.
#'
#' @param series un objet ts unidimensionnel conforme aux règles de assert_ts
#'
#' @returns En sortie, la fonction retourne un objet au format `date_ts`
#' (`AAAA`, `c(AAAA, MM)` ou `c(AAAA, TT)`)
#'
#' @details La date retournée en output est au format `date_ts`. Si l'objet
#' `series` ne contient que des NAs, la fonction retourne une erreur.
#'
#' @export
#'
#' @seealso `last_date`
#'
#' @examples
#'
#' ts1 <- ts(c(NA, NA, NA, 1:10, NA), start = 2000, frequency = 12L)
#' ts2 <- ts(c(1:10, NA), start = 2020, frequency = 4L)
#'
#' stats::start(ts1)
#' first_date(ts1)
#'
#' stats::start(ts1)
#' first_date(ts2)
#'
first_date <- function(series) {
    # Check de l'objet series
    assert_ts(series, .var.name = "series")

    time_ts <- stats::time(na_trim(series))
    first_time <- time_ts[1L]
    frequency_ts <- as.integer(stats::frequency(series))
    return(c(first_time %/% 1L, (first_time %% 1L) * frequency_ts + 1L))
}

#' @title Dernière date non NA
#'
#' @description Cette fonction calcule la dernière date pour laquelle l'objet
#' `series` ne vaut pas NA.
#'
#' @inheritParams first_date
#'
#' @returns En sortie, la fonction retourne un objet au format `date_ts`
#' (`AAAA`, `c(AAAA, MM)` ou `c(AAAA, TT)`)
#'
#' @details La date retournée en output est au format `date_ts`. Si l'objet
#' `series` ne contient que des NAs, la fonction retourne une erreur.
#'
#' @export
#'
#' @seealso `first_date`
#'
#' @examples
#'
#' ts1 <- ts(c(NA, NA, NA, 1:10, NA), start = 2000, frequency = 12L)
#' ts2 <- ts(c(1:10), start = 2020, frequency = 4L)
#'
#' stats::end(ts1)
#' last_date(ts1)
#'
#' stats::end(ts2)
#' last_date(ts2)
#'
last_date <- function(series) {
    # Check de l'objet series
    assert_ts(series, .var.name = "series")

    time_ts <- stats::time(na_trim(series))
    last_time <- time_ts[length(time_ts)]
    frequency_ts <- as.integer(stats::frequency(series))

    if (frequency_ts == 12L) {
        return(as_yyyymm(last_time))
    } else if (frequency_ts == 4L) {
        return(as_yyyytt(last_time))
    }
}

#' @title Comparaison de 2 date_ts
#'
#' @inheritParams diff_periode
#' @param strict un booleen (default `FALSE`)
#'
#' @returns En sortie, la fonction retourne un booleen (de longueur 1) qui
#' indique si la date `a` est antérieure à la date `b`.
#'
#' @details Les dates `a` et `b` sont au format date_ts. L'argument frequency_ts
#' est nécessaire pour interpréter les dates.
#' Ainsi, si je souhaite comparer la date `a = c(2023L, 4L)` et la date `b
#' = c(2023L, -2L)`. Dans le cas d'une fréquence mensuelle, la date `a` est
#' antérieure à la date `b`. Dans le cas d'une fréquence mensuelle, c'est
#' l'inverse. Si `strict` vaut `TRUE`, la fonction compare strictement les dates
#' `a` et `b` (`<`).
#' @export
#'
#' @examples
#'
#' is_before(a = c(2020L, 3L), b = c(2022L, 4L), frequency_ts = 12L)
#' is_before(a = c(2022L, 3L), b = c(2010L, 1L), frequency_ts = 4L)
#'
#' is_before(a = c(2022L, 4L), b = c(2022L, 4L), frequency_ts = 12L)
#' is_before(a = c(2022L, 4L), b = c(2022L, 4L),
#'     frequency_ts = 12L, strict = TRUE)
#'
#' # Importance de la fréquence
#' is_before(a = c(2022L, -3L), b = c(2021L, 8L), frequency_ts = 12L)
#' is_before(a = c(2022L, -3L), b = c(2021L, 8L), frequency_ts = 4L)
#'
is_before <- function(a, b, frequency_ts, strict = FALSE) {
    coll <- checkmate::makeAssertCollection()

    # Check de strict
    checkmate::assert_flag(strict, add = coll, .var.name = "replace_na")

    # Check de la fréquence
    frequency_ts <- assert_frequency(
        frequency_ts,
        add = coll,
        .var.name = "frequency_ts"
    )

    if (isTRUE(check_frequency(frequency_ts, warn = FALSE))) {
        # Check du format date_ts a
        a <- assert_date_ts(x = a, frequency_ts, add = coll, .var.name = "a")

        # Check du format date_ts b
        b <- assert_date_ts(x = b, frequency_ts, add = coll, .var.name = "b")
    }

    checkmate::reportAssertions(coll)

    tu_a <- date_ts2timeunits(a, frequency_ts = frequency_ts)
    tu_b <- date_ts2timeunits(b, frequency_ts = frequency_ts)

    if (strict) {
        return(tu_a < tu_b)
    } else {
        return(tu_a <= tu_b)
    }
}


#' @title Intervalle entre 2 dates
#'
#' @param a un objet date_ts, c'est-à-dire un vecteur numérique, de préférence
#' `integer` au format `AAAA`, `c(AAAA, MM)` ou `c(AAAA, TT)`
#' @param b un objet date_ts, c'est-à-dire un vecteur numérique, de préférence
#' `integer` au format `AAAA`, `c(AAAA, MM)` ou `c(AAAA, TT)`
#' @param frequency_ts un entier qui vaut `4L` (ou `4.0`) pour les séries
#' trimestrielles et `12L` (ou `12.0`) pour les séries mensuelles.
#'
#' @returns En sortie, la fonction retourne un entier qui désigne le nombre de
#' période (mois ou trimestres) qui sépare les 2 dates `a` et `b`.
#'
#' @details On travaille ici avec des dates au format date_ts, c'est-à-dire qui
#' passe le test de la fonction `assert_date_ts`. Lorsqu'on parle d'intervalle
#' et de nombre de période entre `a` et `b`, les bornes sont incluses. Ainsi
#' `diff_periode(2020L, 2020L, 12L)` retourne bien 1L et non 2L ou 0L.
#'
#' @export
#'
#' @examples
#' # Une seule période
#' diff_periode(a = 2020L, b = 2020L, frequency_ts = 4L)
#'
#' diff_periode(a = c(2000L, 1L), b = c(2020L, 4L), frequency_ts = 4L)
#'
#' # Ordre chronologique respecté
#' diff_periode(a = c(2021L, 5L), b = c(2023L, 8L), frequency_ts = 12L)
#'
#' # Date inversées
#' diff_periode(a = c(2023L, 8L), b = c(2021L, 5L), frequency_ts = 12L)
#'
diff_periode <- function(a, b, frequency_ts) {
    coll <- NULL

    # Check de la fréquence
    frequency_ts <- assert_frequency(
        frequency_ts,
        add = coll,
        .var.name = "frequency_ts"
    )
    # Check du format date_ts a
    a <- assert_date_ts(x = a, frequency_ts, add = coll, .var.name = "a")
    # Check du format date_ts b
    b <- assert_date_ts(x = b, frequency_ts, add = coll, .var.name = "b")

    if (!is_before(a, b, frequency_ts)) {
        return(diff_periode(b, a, frequency_ts))
    }
    if (length(a) == 1L) {
        a <- c(a, 1L)
    }
    if (length(b) == 1L) {
        b <- c(b, 1L)
    }
    return((b[1L] - a[1L]) * frequency_ts + b[2L] - a[2L] + 1L)
}

#' @title Ajuste un objet date_ts dans un format conforme.
#'
#' @inheritParams date_ts2timeunits
#' @param test un booléen (Default is TRUE)
#'
#' @returns En sortie, la fonction retourne une date au même format que l'objet
#' `date_ts` avec la période inclus entre 1 et la fréquence.
#'
#' @details Ici le formattage correspond à une réécriture de la date sans
#' en changer la valeur. Alors que l'objet c(2020L, 12L) désigne le mois de
#' décembre 2020 et c(2021L, 1L) le mois de janvier 2021, on peut imaginer que
#' la date_ts c(2021L, 0L) peut aussi représenter le mois de décembre 2020.
#' Si l'argument `test` est mis à FALSE, alors aucun test ne sera effectué sur
#' les données en entrée.
#' @export
#'
#' @examples
#'
#' # Formattage inchangée
#' normalize_date_ts(c(2020L, 1L), frequency_ts = 4L) # 1er trimestre de 2020
#' normalize_date_ts(c(2020L, 8L), frequency_ts = 12L) # Aout 2020
#'
#' # Retour dans le passé
#' normalize_date_ts(c(2020L, 0L), frequency_ts = 4L) # 4ème trimestre de 2019
#' normalize_date_ts(c(2020L, -10L), frequency_ts = 12L) # février 2019
#'
#' # Avancée dans le futur
#' normalize_date_ts(c(2020L, 7L), frequency_ts = 4L) # 3ème trimestre de 2021
#' normalize_date_ts(c(2020L, 13L), frequency_ts = 4L) # janvier 2021
#'
normalize_date_ts <- function(date_ts, frequency_ts, test = TRUE) {
    if (test) {
        coll <- checkmate::makeAssertCollection()

        # Check de la fréquence
        frequency_ts <- assert_frequency(
            frequency_ts,
            add = coll,
            .var.name = "frequency_ts"
        )
        # Check du format date_ts
        if (isTRUE(check_frequency(frequency_ts, warn = FALSE))) {
            date_ts <- assert_date_ts(
                x = date_ts,
                frequency_ts,
                add = coll,
                .var.name = "date_ts"
            )
        }

        checkmate::reportAssertions(coll)
    }

    date_ts <- as.integer(date_ts)

    if (length(date_ts) == 2L) {
        year <- date_ts[1L]
        period <- date_ts[2L]
        return(c(
            year + ((period - 1L) %/% frequency_ts),
            1L + ((period - 1L) %% frequency_ts)
        ))
    } else {
        return(c(date_ts, 1L))
    }
}

#' @title Vérifie le format de date
#'
#' @description La fonction `assert_date_ts` vérifie qu'un objet est de type
#' `AAAA`, `c(AAAA, MM)` ou `c(AAAA, TT)`
#'
#' @param x un vecteur numérique, de préférence `integer` au format `AAAA`,
#' `c(AAAA, MM)` ou `c(AAAA, TT)`
#' @param frequency_ts un entier qui vaut `4L` (ou `4.0`) pour les séries
#' trimestrielles et `12L` (ou `12.0`) pour les séries mensuelles.
#' @param add Collection pour stocker les messages d'erreurs (Default is NULL)
#' @param .var.name Nom de l'objet à vérifier pour afficher dans les messages
#' @param warn un booleen
#'
#' @returns En sortie la fonction retourne l'objet `x` de manière invisible ou
#' une erreur.
#' @details Les fonctions du package date4ts sont faites pour fonctionner
#' avec des times-series de fréquence mensuelle ou trimestrielle et basés sur le
#' système des mois, trimestres et années classiques.
#' On cherche donc à favoriser l'utilisation de vecteur `c(AAAA, MM)` pour
#' désigner la date choisie.
#' Lorsque l'objet `x` en entrée est au mauvais format, il est corrigé pendant
#' la checks et l'objet en sortie est au bon format.
#' Si l'argument `warn` est `FALSE`, alors la fonction ne retournera pas de
#' warning lors de l'évaluation.
#'
#' Ici, l'argument `frequency_ts` est nécessaire car une date sous la forme
#' c(AAAA, PP), avec PP le nombre de période, ne désigne pas une date absolue.
#' Par exemple, c(2020L 5L) désigne mai 2020 pour une fréquence mensuelle et le
#' 1er trimestre 2021 pour une fréquence trimestrielle.
#'
#' Selon le préfixe de la fonction :
#'
#' \itemize{
#' \item si le check réussi : \itemize{
#'     \item la fonction `assert_date_ts` retourne l'objet `x` de manière
#'          invisible;
#'     \item la fonction `check_date_ts` retourne le booléen `TRUE`.
#' }
#' \item si le check échoue : \itemize{
#'     \item la fonction `assert_date_ts` retourne un message d'erreur;
#'     \item la fonction `check_date_ts` retourne une chaîne de caractère
#'          signalant le problème.
#' }
#' }
#'
#' @export
#'
#' @examples
#' # De bons formats de date
#' assert_date_ts(c(2020L, 8L), frequency_ts = 12L)
#' assert_date_ts(c(2020L, 2L), frequency_ts = 4L)
#' check_date_ts(2022L, frequency_ts = 12L)
#'
#' # Format double --> génération d'un warning
#' assert_date_ts(c(2020., 4.0), frequency_ts = 4L)
#' assert_date_ts(2022., frequency_ts = 12L)
#' check_date_ts(2022., frequency_ts = 12L)
#'
#' # Fréquence au format double --> génération d'un warning
#' assert_date_ts(c(2020L, 6L), frequency_ts = 4.0)
#' assert_date_ts(c(2020L, 42L), frequency_ts = 12.0)
#'
#' # Dépassement la fréquence --> génération d'un warning
#' assert_date_ts(c(2020L, 6L), frequency_ts = 4L)
#' assert_date_ts(c(2020L, 42L), frequency_ts = 12L)
#' assert_date_ts(c(2020L, -4L), frequency_ts = 12L)
#'
#' # Avec des erreurs
#' check_date_ts(1:10, frequency_ts = 12L)
#'
check_date_ts <- function(
    x,
    frequency_ts,
    .var.name = checkmate::vname(x),
    warn = TRUE
) {
    output <- make_check_collection()

    check_1 <- checkmate::check_flag(warn, na.ok = FALSE, null.ok = FALSE)
    check_2 <- check_frequency(
        frequency_ts,
        warn = warn,
        .var.name = "frequency_ts"
    )
    check_3 <- checkmate::check_integerish(
        x,
        any.missing = FALSE,
        min.len = 1L,
        max.len = 2L,
        null.ok = FALSE
    )
    check_4 <- checkmate::check_integer(x)

    output <- add_check_collection(
        coll = output,
        check_output = check_1,
        .var.name = "warn"
    )
    output <- add_check_collection(
        coll = output,
        check_output = check_3,
        .var.name = .var.name
    )

    if (isTRUE(output) && warn && !isTRUE(check_4)) {
        err <- try(
            expr = checkmate::assert_integer(x, .var.name = .var.name),
            silent = TRUE
        )
        warning(attr(err, "condition")$message, call. = FALSE)
    }

    output <- add_check_collection(
        coll = output,
        check_output = check_2,
        .var.name = "frequency_ts"
    )

    if (
        isTRUE(output) &&
            warn &&
            (length(x) == 2L) &&
            !isTRUE(checkmate::check_integerish(
                x = x[2L],
                lower = 1L,
                upper = frequency_ts
            ))
    ) {
        err <- try(
            expr = checkmate::assert_integerish(
                x[2L],
                lower = 1L,
                upper = frequency_ts,
                .var.name = "period"
            ),
            silent = TRUE
        )
        warning(attr(err, "condition")$message, call. = FALSE)
    }

    return(output)
}

#' @rdname check_date_ts
#' @export
#'
assert_date_ts <- function(
    x,
    frequency_ts,
    add = NULL,
    .var.name = checkmate::vname(x),
    warn = TRUE
) {
    if (is.null(add)) {
        coll <- checkmate::makeAssertCollection()
    } else {
        coll <- add
    }

    # Check de warn
    checkmate::assert_flag(
        warn,
        add = coll,
        .var.name = "warn",
        na.ok = FALSE,
        null.ok = FALSE
    )
    check_warn <- checkmate::check_flag(warn, na.ok = FALSE, null.ok = FALSE)

    # Check de la fréquence
    if (isTRUE(check_warn)) {
        frequency_ts <- assert_frequency(
            frequency_ts,
            add = coll,
            .var.name = "frequency_ts",
            warn = warn
        )
    } else {
        frequency_ts <- assert_frequency(
            frequency_ts,
            add = coll,
            .var.name = "frequency_ts",
            warn = TRUE
        )
    }

    checkmate::assert_numeric(
        x,
        any.missing = FALSE,
        add = coll,
        .var.name = .var.name,
        min.len = 1L,
        max.len = 2L,
        finite = TRUE
    )
    x_corr <- x

    # Check du type
    # Ici on passe d'abord par un check car il y a une génération de warning
    # non voulue sinon... (voir issue mllg/checkmate#242)
    if (
        isTRUE(checkmate::check_numeric(
            x,
            any.missing = FALSE,
            min.len = 1L,
            max.len = 2L,
            finite = TRUE
        ))
    ) {
        x_corr <- checkmate::assert_integerish(
            x,
            coerce = TRUE,
            any.missing = FALSE,
            add = coll,
            .var.name = .var.name
        )
    }

    if (is.null(add)) {
        checkmate::reportAssertions(coll)
    }

    if (isTRUE(check_date_ts(x, frequency_ts, warn = FALSE))) {
        if (isTRUE(check_warn) && warn) {
            if (!isTRUE(checkmate::check_integer(x))) {
                err <- try(
                    expr = checkmate::assert_integer(x, .var.name = .var.name),
                    silent = TRUE
                )
                warning(attr(err, "condition")$message, call. = FALSE)
            }

            if (
                (length(x) == 2L) &&
                    !isTRUE(checkmate::check_integerish(
                        x = x[2L],
                        lower = 1L,
                        upper = frequency_ts
                    ))
            ) {
                err <- try(
                    expr = checkmate::assert_integerish(
                        x = x[2L],
                        lower = 1L,
                        upper = frequency_ts,
                        .var.name = "period"
                    ),
                    silent = TRUE
                )
                warning(attr(err, "condition")$message, call. = FALSE)
            }
        }

        x <- normalize_date_ts(x_corr, frequency_ts, test = FALSE)
    }

    return(invisible(x))
}

#' @title Vérifie la conformité d'un objet ts
#'
#' @description Les fonctions `assert_ts` et `check_ts` vérifient qu'un objet ts
#' est bien conforme.
#'
#' @param x Un objet ts unidimensionnel
#' @param add Collection pour stocker les messages d'erreurs (Default is NULL)
#' @param .var.name Nom de l'objet à vérifier pour afficher dans les messages
#' @param allow_mts Booleen. Est ce que les objects \code{mts} sont acceptés ?
#'
#' @returns En sortie la fonction retourne l'objet `x` de manière invisible ou
#' une erreur.
#' @details Les fonctions du package date4ts sont faites pour fonctionner
#' avec des times-series de fréquence mensuelle ou trimestrielle et basées sur
#' le système des mois, trimestres et années classiques.
#' On travaille avec des données numériques (integer, double ou logical) mais
#' les autres types atomic sont acceptés également.
#' On cherche donc à favoriser l'utilisation de séries temporelles classiques
#' utilisants des types atomiques.
#' Lorsque l'objet `x` en entrée est au mauvais format, une erreur est généré.
#'
#' Selon le préfixe de la fonction :
#'
#' - si le check réussi :
#'     - la fonction `assert_ts` retourne l'objet `x` de manière invisible;
#'     - la fonction `check_ts` retourne le booléen `TRUE`.
#'
#' - si le check échoue :
#'     - la fonction `assert_ts` retourne un message d'erreur;
#'     - la fonction `check_ts` retourne une chaîne de caractère signalant le
#'     problème.
#'
#' @export
#'
#' @examples
#' ts1 <- ts(1:100, start = 2010L, frequency = 12L)
#' ts2 <- ts(1:10, start = c(2020L, 4L), frequency = 4L)
#'
#' assert_ts(ts1)
#' assert_ts(ts2)
#'
#' check_ts(ts1)
#' check_ts(ts2)
#'
#' # Exemples avec des erreurs
#'
#' check_ts(1)
#' check_ts(ts(1:10, start = 2010L, frequency = 2L))
#' check_ts(1:10)
#'
check_ts <- function(x, .var.name = checkmate::vname(x), allow_mts = FALSE) {
    output <- make_check_collection()

    # Check de la classe de l'objet
    check_1 <- checkmate::check_class(x, classes = "ts", null.ok = FALSE)
    output <- add_check_collection(
        coll = output,
        check_output = check_1,
        .var.name = .var.name
    )

    if (isTRUE(check_1)) {
        # Check de la fréquence
        check_2 <- check_expression({
            stats::frequency(x)
        })
        output <- add_check_collection(coll = output, check_output = check_2)

        if (isTRUE(check_2)) {
            frequency_ts <- stats::frequency(x)

            check_3 <- check_frequency(
                frequency_ts,
                warn = FALSE,
                .var.name = "frequency_ts"
            )
            output <- add_check_collection(
                coll = output,
                check_output = check_3
            )

            if (isTRUE(check_3)) {
                # Check de la temporalité - start
                check_4 <- check_expression(expr = {
                    stats::start(x)
                })
                output <- add_check_collection(
                    coll = output,
                    check_output = check_4
                )

                if (isTRUE(check_4)) {
                    start_ts <- stats::start(x)

                    check_5 <- check_date_ts(
                        start_ts,
                        frequency_ts = frequency_ts,
                        warn = FALSE,
                        .var.name = paste0("stats::start(", .var.name, ")")
                    )
                    output <- add_check_collection(
                        coll = output,
                        check_output = check_5
                    )
                }

                # Check de la temporalité - end
                check_6 <- check_expression(expr = {
                    stats::end(x)
                })
                output <- add_check_collection(
                    coll = output,
                    check_output = check_6
                )

                if (isTRUE(check_6)) {
                    end_ts <- stats::end(x)

                    check_7 <- check_date_ts(
                        end_ts,
                        frequency_ts = frequency_ts,
                        warn = FALSE,
                        .var.name = paste0("stats::end(", .var.name, ")")
                    )
                    output <- add_check_collection(
                        coll = output,
                        check_output = check_7
                    )
                }
            }
        }

        # Check que l'objet ne soit pas un mts
        check_8 <- ifelse(
            test = allow_mts,
            yes = TRUE,
            no = checkmate::check_false(stats::is.mts(x))
        )
        output <- add_check_collection(
            coll = output,
            check_output = check_8,
            .var.name = paste0("stats::is.mts(", .var.name, ")")
        )

        # Check du type de données
        check_9 <- checkmate::check_atomic_vector(x)
        check_10 <- checkmate::check_matrix(x, mode = "atomic", null.ok = FALSE)

        cond_mts <- isTRUE(check_9) || isTRUE(check_10)
        check_11 <- ifelse(
            test = cond_mts,
            yes = TRUE,
            no = paste(check_9, check_10, sep = " OR ")
        )

        check_12 <- ifelse(
            test = allow_mts,
            yes = check_11,
            no = check_9
        )

        output <- add_check_collection(
            coll = output,
            check_output = check_12,
            .var.name = .var.name
        )
    }

    return(output)
}

#' @rdname check_ts
#' @export
#'
assert_ts <- function(
    x,
    add = NULL,
    .var.name = checkmate::vname(x),
    allow_mts = FALSE
) {
    if (is.null(add)) {
        coll <- checkmate::makeAssertCollection()
    } else {
        coll <- add
    }

    checkmate::assert_class(
        x,
        classes = "ts",
        add = coll,
        .var.name = .var.name
    )

    # Check de la fréquence
    frequency_ts <- assert_expression(
        expr = stats::frequency(x)
    )
    frequency_ts <- assert_frequency(
        frequency_ts,
        add = coll,
        .var.name = "frequency_ts",
        warn = FALSE
    )

    # Check de la temporalité
    start_ts <- assert_expression(
        expr = stats::start(x)
    )
    start_ts <- assert_date_ts(
        start_ts,
        frequency_ts = frequency_ts,
        add = coll,
        .var.name = "start",
        warn = FALSE
    )

    end_ts <- assert_expression(
        expr = stats::end(x)
    )
    end_ts <- assert_date_ts(
        end_ts,
        frequency_ts = frequency_ts,
        add = coll,
        .var.name = "end",
        warn = FALSE
    )

    # Check de la classe de l'objet
    if (!allow_mts) {
        checkmate::assert_false(
            x = stats::is.mts(x),
            add = coll,
            .var.name = paste0("is.mts(", .var.name, ")")
        )
    }

    # Check du type de données
    cond_mts <- isTRUE(checkmate::check_atomic_vector(x)) ||
        isTRUE(checkmate::check_matrix(x, mode = "atomic", null.ok = FALSE))
    check_1 <- checkmate::check_atomic_vector(x)
    check_2 <- checkmate::check_matrix(x, mode = "atomic", null.ok = FALSE)

    if (allow_mts && !cond_mts) {
        coll$push(msg = paste(check_1, check_2, sep = " OR "))
    } else if (!allow_mts) {
        checkmate::assert_atomic_vector(x, add = coll, .var.name = .var.name)
    }

    if (is.null(add)) {
        checkmate::reportAssertions(coll)
    }

    return(invisible(x))
}

#' @title Vérifie la conformité d'un objet TimeUnits
#'
#' @description La fonction `assert_timeunits` vérifie qu'un objet est un
#' TimeUnits.
#'
#' @param x un numérique qui représente le time units de
#' @param frequency_ts un entier qui vaut `4L` (ou `4.0`) pour les séries
#' trimestrielles et `12L` (ou `12.0`) pour les séries mensuelles.
#' @param add Collection pour stocker les messages d'erreurs (Default is NULL)
#' @param .var.name Nom de l'objet à vérifier pour afficher dans les messages
#'
#' @returns En sortie la fonction retourne l'objet `x` de manière invisible ou
#' une erreur.
#'
#' @details
#' Un objet de type TimeUnits est un numérique qui désigne l'année et la période
#' en cours avec ses décimales.
#' Ainsi pour une série temporelle mensuelle, `2020.5` représente la moitié de
#' l'année donc juillet 2020 et s'écrit `c(2020L, 7L)` au format date_ts.
#'
#' Selon le préfixe de la fonction :
#'
#' - si le check réussi :
#'     - la fonction `assert_timeunits` retourne l'objet `x` de manière
#'     invisible;
#'     - la fonction `check_timeunits` retourne le booléen `TRUE`.
#'
#' - si le check échoue :
#'     - la fonction `assert_timeunits` retourne un message d'erreur;
#'     - la fonction `check_timeunits` retourne une chaîne de caractère
#'     signalant le problème.
#'
#' @export
#'
#' @examples
#'
#' assert_timeunits(2020.5, frequency_ts = 12L)
#' assert_timeunits(2020.5, frequency_ts = 4L)
#' assert_timeunits(2023., frequency_ts = 12L)
#'
#' assert_timeunits(2000. + 5. / 12.0, frequency_ts = 12L)
#' assert_timeunits(2015. + 3. / 4.0, frequency_ts = 4L)
#'
#' check_timeunits(2020.5, frequency_ts = 12L)
#' check_timeunits(2015. + 3. / 4.0, frequency_ts = 4L)
#'
#' # Avec erreur
#'
#' check_timeunits(list(1.), frequency_ts = 12L)
#' check_timeunits(2000., frequency_ts = 1L)
#'
check_timeunits <- function(x, frequency_ts, .var.name = checkmate::vname(x)) {
    output <- make_check_collection()

    check_1 <- check_frequency(frequency_ts, .var.name = "frequency_ts")
    check_2 <- checkmate::check_number(x, finite = TRUE, na.ok = FALSE)

    output <- add_check_collection(coll = output, check_output = check_1)
    output <- add_check_collection(
        coll = output,
        check_output = check_2,
        .var.name = .var.name
    )

    if (isTRUE(check_1) && isTRUE(check_2)) {
        check_3 <- checkmate::check_int(x * frequency_ts)
        output <- add_check_collection(
            coll = output,
            check_output = check_3,
            .var.name = paste(.var.name, "* frequency_ts")
        )
    }

    return(output)
}

#' @rdname check_timeunits
#' @export
#'
assert_timeunits <- function(
    x,
    frequency_ts,
    add = NULL,
    .var.name = checkmate::vname(x)
) {
    if (is.null(add)) {
        coll <- checkmate::makeAssertCollection()
    } else {
        coll <- add
    }

    # Check de la fréquence
    frequency_ts <- assert_frequency(
        frequency_ts,
        add = coll,
        .var.name = "frequency_ts"
    )
    # Check de l'objet x (TimeUnits)
    checkmate::assert_number(
        x,
        add = coll,
        .var.name = .var.name,
        finite = TRUE
    )

    if (
        isTRUE(check_frequency(frequency_ts, warn = FALSE)) &&
            isTRUE(checkmate::check_number(x, finite = TRUE))
    ) {
        checkmate::assert_int(
            x * frequency_ts,
            add = coll,
            .var.name = paste0(.var.name, " * frequency_ts")
        )
    }

    if (is.null(add)) {
        checkmate::reportAssertions(coll)
    }

    return(invisible(x))
}

#' @title Vérifie la conformité d'une fréquence
#'
#' @param x un entier qui vaut `4L` (ou `4.0`) pour les séries trimestrielles et
#' `12L` (ou `12.0`) pour les séries mensuelles.
#' @param warn un booleen
#' @param add Collection pour stocker les messages d'erreurs (Default is NULL)
#' @param .var.name Nom de l'objet à vérifier pour afficher dans les messages
#'
#' @details La fréquence d'une série temporelle est soit mensuelle (`12L` ou
#' `12.0`) soit trimestrielle (`4L` ou `4.0`). Les autres fréquences ne sont pas
#' acceptées.
#' Cette fonction s'appuie essentiellement sur les fonctions
#' `checkmate::check_numeric`, `checkmate::check_int` et
#' `checkmate::check_choice`.
#' Il y a néanmoins une petite subtilité : on vérifie si l'objet `x` est de type
#' double ou integer.
#' Dans le premier cas, on affichera un warning et on corrigera l'objet au
#' format integer pour les traitements ultérieurs. En sortie, `x` est retourné
#' de manière invisible.
#' Si l'argument `warn` est `FALSE`, alors la fonction ne retournera pas de
#' warning lors de l'évaluation.
#'
#' Selon le préfixe de la fonction :
#'
#' - si le check réussi :
#'     - la fonction `assert_frequency` retourne l'objet `x` de manière
#'     invisible;
#'     - la fonction `check_frequency` retourne le booléen `TRUE`.
#'
#' - si le check échoue :
#'     - la fonction `assert_frequency` retourne un message d'erreur;
#'     - la fonction `check_frequency` retourne une chaîne de caractère
#'     signalant le problème.
#'
#' @returns En sortie la fonction retourne l'objet `x` de manière invisible ou
#' une erreur.
#'
#' @export
#'
#' @examples
#'
#' assert_frequency(4L)
#' assert_frequency(12L)
#'
#' check_frequency(4L)
#' check_frequency(12L)
#'
#' # Avec des erreurs,
#'
#' check_frequency(Inf, warn = FALSE)
#' check_frequency(1:10)
#' check_frequency(1L)
#'
check_frequency <- function(x, .var.name = checkmate::vname(x), warn = TRUE) {
    output <- make_check_collection()

    check_1 <- checkmate::check_numeric(x, finite = TRUE, any.missing = FALSE)
    check_2 <- checkmate::check_flag(warn, na.ok = FALSE, null.ok = FALSE)
    check_3 <- checkmate::check_integer(x)
    check_4 <- checkmate::check_int(x)
    check_5 <- checkmate::check_choice(x, choices = c(4L, 12L))

    output <- add_check_collection(
        coll = output,
        check_output = check_1,
        .var.name = .var.name
    )
    output <- add_check_collection(
        coll = output,
        check_output = check_2,
        .var.name = "warn"
    )

    if (isTRUE(check_1)) {
        output <- add_check_collection(
            coll = output,
            check_output = check_4,
            .var.name = .var.name
        )
        output <- add_check_collection(
            coll = output,
            check_output = check_5,
            .var.name = .var.name
        )

        if (isTRUE(check_2) && warn && !isTRUE(check_3)) {
            err <- try(
                expr = checkmate::assert_integer(
                    x = x,
                    .var.name = .var.name
                ),
                silent = TRUE
            )
            warning(attr(err, "condition")$message, call. = FALSE)
        }
    }

    return(output)
}

#' @rdname check_frequency
#' @export
#'
assert_frequency <- function(
    x,
    add = NULL,
    .var.name = checkmate::vname(x),
    warn = TRUE
) {
    if (is.null(add)) {
        coll <- checkmate::makeAssertCollection()
    } else {
        coll <- add
    }

    # Check de warn
    checkmate::assert_flag(
        warn,
        add = coll,
        .var.name = "warn",
        na.ok = FALSE,
        null.ok = FALSE
    )
    check_warn <- checkmate::check_flag(warn, na.ok = FALSE, null.ok = FALSE)

    # Check du type
    # Ici on passe d'abord par un check car il y a une génération de warning
    # non voulue sinon... (voir issue #242)
    check_1 <- checkmate::check_numeric(x, any.missing = FALSE, finite = TRUE)
    if (isTRUE(check_1)) {
        x_corr <- checkmate::assert_int(
            x,
            coerce = TRUE,
            add = coll,
            .var.name = .var.name
        )
        checkmate::assert_choice(
            x_corr,
            choices = c(4L, 12L),
            add = coll,
            .var.name = .var.name
        )
    } else {
        checkmate::assert_numeric(
            x,
            any.missing = FALSE,
            finite = TRUE,
            add = coll,
            .var.name = .var.name
        )
    }

    if (is.null(add)) {
        checkmate::reportAssertions(coll)
    }

    if (isTRUE(check_frequency(x, warn = FALSE))) {
        if (
            isTRUE(check_warn) && warn && (!isTRUE(checkmate::check_integer(x)))
        ) {
            err <- try(
                expr = checkmate::assert_integer(x, .var.name = .var.name),
                silent = TRUE
            )
            warning(attr(err, "condition")$message, call. = FALSE)
        }
        x <- x_corr
    }

    return(invisible(x))
}


#' @title Vérifie la conformité d'un entier scalaire
#'
#' @param x un entier relatif (positif, négatif ou nul)
#' @param add Collection pour stocker les messages d'erreurs (Default is NULL)
#' @param .var.name Nom de l'objet à vérifier pour afficher dans les messages
#' @param warn un booleen
#'
#' @returns En sortie la fonction retourne l'objet `x` de manière invisible ou
#' une erreur.
#'
#' @details
#' On vérifie que l'objet `x` en entrée est bien un entier.
#' Cette fonction s'appuie essentiellement sur la fonction
#' `checkmate::assert_int`.
#' Il y a néanmoins une petite subtilité : on vérifie si l'objet `x` est de
#' type double ou integer. Si l'objet est de type double (et non integer), la
#' fonction retournera aussi un warning.
#' Dans le premier cas, on affichera un warning et on corrigera l'objet au
#' format integer pour les traitements ultérieurs. En sortie, `x` est retourné
#' de manière invisible.
#' Si l'argument `warn` vaut `FALSE`, alors la fonction ne retournera pas de
#' warning lors de l'évaluation.
#'
#' Selon le préfixe de la fonction :
#'
#' - si le check réussi :
#'     - la fonction `assert_scalar_integer` retourne l'objet `x` de manière
#' invisible;
#'     - la fonction `check_scalar_integer` retourne le booléen `TRUE`.
#'
#' - si le check échoue :
#'     - la fonction `assert_scalar_integer` retourne un message d'erreur;
#'     - la fonction `check_scalar_integer` retourne une chaîne de caractère
#' signalant le problème.
#'
#' @export
#'
#' @seealso [check_scalar_natural()], [assert_scalar_natural()]
#'
#' @examples
#'
#' assert_scalar_integer(1L)
#' assert_scalar_integer(100L)
#' assert_scalar_integer(-4L)
#' assert_scalar_integer(0L)
#'
#' check_scalar_integer(1L)
#' check_scalar_integer(100L)
#' check_scalar_integer(-4L)
#' check_scalar_integer(0L)
#'
#' # Avec des erreurs,
#'
#' check_scalar_integer(Inf)
#' check_scalar_integer(1:10)
#' check_scalar_integer(pi)
#' check_scalar_integer(2.)
#'
check_scalar_integer <- function(x, warn = TRUE) {
    output <- make_check_collection()

    check_1 <- checkmate::check_int(x)
    check_2 <- checkmate::check_flag(warn, na.ok = FALSE, null.ok = FALSE)
    check_3 <- checkmate::check_integer(x)

    output <- add_check_collection(
        coll = output,
        check_output = check_1,
        .var.name = "x"
    )
    output <- add_check_collection(
        coll = output,
        check_output = check_2,
        .var.name = "warn"
    )

    if (isTRUE(output) && warn && !isTRUE(check_3)) {
        err <- try(
            checkmate::assert_integer(x),
            silent = TRUE
        )
        warning(attr(err, "condition")$message, call. = FALSE)
    }

    return(output)
}

#' @rdname check_scalar_integer
#' @export
#'
assert_scalar_integer <- function(
    x,
    add = NULL,
    .var.name = checkmate::vname(x),
    warn = TRUE
) {
    if (is.null(add)) {
        coll <- checkmate::makeAssertCollection()
    } else {
        coll <- add
    }

    # Qqs variables de check
    check_warn <- checkmate::check_flag(warn, na.ok = FALSE, null.ok = FALSE)
    check_int_var <- checkmate::check_int(x)

    # Check de warn
    checkmate::assert_flag(
        warn,
        add = coll,
        .var.name = "warn",
        na.ok = FALSE,
        null.ok = FALSE
    )
    checkmate::assert_int(x, add = coll, .var.name = .var.name)

    if (isTRUE(check_int_var) && !isTRUE(checkmate::check_integer(x))) {
        if (isTRUE(check_warn) && warn) {
            err <- try(
                checkmate::assert_integer(x, .var.name = .var.name),
                silent = TRUE
            )
            warning(attr(err, "condition")$message, call. = FALSE)
        }

        x <- checkmate::assert_int(
            x,
            coerce = TRUE,
            add = coll,
            .var.name = .var.name
        )
    }

    if (is.null(add)) {
        checkmate::reportAssertions(coll)
    }

    return(invisible(x))
}

#' @title Vérifie la conformité d'un entier naturel
#'
#' @description
#' Le but de cett fonction est de tester si une variable x est un nombre naturel
#' strictement positif.
#'
#' @param x un entier naturel strictement positif
#' @param add Collection pour stocker les messages d'erreurs (Default is NULL)
#' @param .var.name Nom de l'objet à vérifier pour afficher dans les messages
#' @param warn un booleen
#'
#' @returns En sortie la fonction retourne l'objet `x` de manière invisible ou
#' une erreur.
#'
#' @details
#' Cette fonction s'appuie essentiellement sur la fonction
#' `checkmate::assert_count`.
#' Il y a néanmoins une petite subtilité : on vérifie si l'objet `x` est de type
#' double ou integer.
#' Dans le premier cas, on affichera un warning et on corrigera l'objet au
#' format integer pour les traitements ultérieurs. En sortie, `x` est retourné
#' de manière invisible.
#' Si l'argument `warn` est `FALSE`, alors la fonction ne retournera pas de
#' warning lors de l'évaluation.
#'
#' Selon le préfixe de la fonction :
#'
#' - si le check réussi :
#'     - la fonction `assert_scalar_natural` retourne l'objet `x` de manière
#' invisible;
#'     - la fonction `check_scalar_natural` retourne le booléen `TRUE`.
#'
#' - si le check échoue :
#'     - la fonction `assert_scalar_natural` retourne un message d'erreur;
#'     - la fonction `check_scalar_natural` retourne une chaîne de caractère
#' signalant le problème.
#'
#' @export
#'
#' @seealso [check_scalar_integer()], [assert_scalar_integer()]
#'
#' @examples
#'
#' # Avec des entier integer
#' assert_scalar_natural(1L)
#' assert_scalar_natural(100L)
#'
#' # Avec des entiers double
#' assert_scalar_natural(2.)
#' assert_scalar_natural(457)
#'
check_scalar_natural <- function(x, warn = TRUE) {
    output <- make_check_collection()

    check_1 <- checkmate::check_count(
        x,
        positive = TRUE,
        na.ok = FALSE,
        null.ok = FALSE
    )
    check_2 <- checkmate::check_flag(warn, na.ok = FALSE, null.ok = FALSE)
    check_3 <- checkmate::check_integer(x)

    output <- add_check_collection(
        coll = output,
        check_output = check_1,
        .var.name = "x"
    )
    output <- add_check_collection(
        coll = output,
        check_output = check_2,
        .var.name = "warn"
    )

    if (isTRUE(output) && warn && !isTRUE(check_3)) {
        err <- try(
            checkmate::assert_integer(x),
            silent = TRUE
        )
        warning(attr(err, "condition")$message, call. = FALSE)
    }

    return(output)
}

#' @rdname check_scalar_natural
#' @export
#'
assert_scalar_natural <- function(
    x,
    add = NULL,
    .var.name = checkmate::vname(x),
    warn = TRUE
) {
    if (is.null(add)) {
        coll <- checkmate::makeAssertCollection()
    } else {
        coll <- add
    }

    # Check de warn
    checkmate::assert_flag(
        warn,
        add = coll,
        .var.name = "warn",
        na.ok = FALSE,
        null.ok = FALSE
    )
    check_warn <- checkmate::check_flag(warn, na.ok = FALSE, null.ok = FALSE)

    # Check du type
    # Ici on passe d'abord par un check car il y a une génération de warning
    # non voulue sinon... (voir issue #242)
    check_1 <- checkmate::check_numeric(x, any.missing = FALSE, finite = TRUE)
    x_corr <- x
    if (isTRUE(check_1)) {
        x_corr <- checkmate::assert_count(
            x,
            coerce = TRUE,
            positive = TRUE,
            add = coll,
            .var.name = .var.name
        )
    } else {
        checkmate::assert_numeric(
            x,
            any.missing = FALSE,
            finite = TRUE,
            add = coll,
            .var.name = .var.name
        )
    }

    if (is.null(add)) {
        checkmate::reportAssertions(coll)
    }

    if (
        isTRUE(check_1) &&
            isTRUE(check_warn) &&
            warn &&
            !isTRUE(checkmate::check_integer(x))
    ) {
        err <- try(
            expr = checkmate::assert_integer(x, .var.name = .var.name),
            silent = TRUE
        )
        warning(attr(err, "condition")$message, call. = FALSE)
    }
    x <- x_corr

    return(invisible(x))
}

#' @title Vérifie la conformité d'une date scalaire
#'
#' @param x un objet de type `Date`.
#' @param add Collection pour stocker les messages d'erreurs (Default is NULL)
#' @param .var.name Nom de l'objet à vérifier pour afficher dans les messages
#'
#' @returns En sortie la fonction retourne l'objet `x` de manière invisible ou
#' une erreur.
#'
#' @details
#' On vérifie que l'objet `x` en entrée est bien au format `Date` et qu'il
#' s'agit d'un scalaire (vecteur de taille 1).
#' Cette fonction s'appuie essentiellement sur la fonction
#' `checkmate::assert_date`.
#'
#' Selon le préfixe de la fonction :
#'
#' - si le check réussi :
#'     - la fonction `assert_scalar_date` retourne l'objet `x` de manière
#'     invisible;
#'     - la fonction `check_scalar_date` retourne le booléen `TRUE`.
#'
#' - si le check échoue :
#'     - la fonction `assert_scalar_date` retourne un message d'erreur;
#'     - la fonction `check_scalar_date` retourne la chaîne de caractère
#'     correspondante à l'erreur du check.
#'
#' @export
#'
#' @examples
#'
#' assert_scalar_date(as.Date("2018-01-24"))
#' assert_scalar_date(as.Date("2000-02-29"))
#' assert_scalar_date(Sys.Date())
#'
#' check_scalar_date(as.Date("2018-01-24"))
#' check_scalar_date(as.Date("2000-02-29"))
#' check_scalar_date(Sys.Date())
#'
#' # Avec des erreurs
#'
#' check_scalar_date(2L)
#' check_scalar_date(seq(from = as.Date("2000-01-01"), to = Sys.Date(), by =
#' "year"))
#'
check_scalar_date <- function(x) {
    output <- make_check_collection()

    output <- add_check_collection(output, checkmate::check_date(x))
    output <- add_check_collection(output, checkmate::check_scalar(x))

    output <- paste(output, collapse = "\n")

    return(output)
}

#' @rdname check_scalar_date
#' @export
#'
assert_scalar_date <- function(x, add = NULL, .var.name = checkmate::vname(x)) {
    if (is.null(add)) {
        coll <- checkmate::makeAssertCollection()
    } else {
        coll <- add
    }
    checkmate::assert_date(
        x,
        add = coll,
        .var.name = .var.name,
        any.missing = FALSE,
        len = 1L
    )

    if (is.null(add)) {
        checkmate::reportAssertions(coll)
    }

    return(invisible(x))
}
#' @title Vérifie la conformité d'une expression
#'
#' @param expr une expression à évaluer
#'
#' @returns En sortie la fonction retourne l'objet `x` (le résultat de
#' l'évaluation de l'expression `expr`) de manière invisible ou une erreur.
#'
#' @details La fonction évalue l'expression `expr`. Le check vérifie si la
#' fonction génère une erreur ou un warning. Si elle ne génère aucun message
#' particulier, on retourne alors l'objet `x` (le résultat de l'évaluation de
#' l'expression `expr`), sans erreur.
#'
#' Selon le préfixe de la fonction :
#'
#' - si le check réussi :
#'     - la fonction `assert_expression` retourne l'objet `x` de manière
#'     invisible;
#'     - la fonction `check_expression` retourne le booléen `TRUE`.
#'
#' - si le check échoue :
#'     - la fonction `assert_expression` retourne un message d'erreur;
#'     - la fonction `check_expression` retourne la chaîne de caractère
#'     "Invalid expression".
#'
#' @export
#'
#' @examples
#'
#' assert_expression(expr = {2 + 2})
#' assert_expression(expr = {is.integer(1L)})
#' try(assert_expression(expr = {log("a")}), silent = TRUE)
#'
#' check_expression(expr = {2 + 2})
#' check_expression(expr = {is.integer(1L)})
#' check_expression(expr = {log("a")})
#'
check_expression <- function(expr) {
    out <- tryCatch(
        expr,
        error = function(e) e,
        warning = function(w) w
    )

    if (inherits(out, "warning") || inherits(out, "error")) {
        return(paste("Invalid expression :", deparse(substitute(expr))))
    }

    return(TRUE)
}

#' @rdname check_expression
#' @export
#'
assert_expression <- function(expr) {
    out <- tryCatch(
        expr,
        error = function(e) e,
        warning = function(w) w
    )

    if (inherits(out, "warning") || inherits(out, "error")) {
        stop("Invalid expression :", deparse(substitute(expr)), call. = FALSE)
    }

    return(invisible(expr))
}

#' @title Crée une collection de checks
#'
#' @description
#' Initialise une collection vide destinée à stocker les résultats des checks
#' des différents arguments, fonctions et contenus.
#' Cette collection peut ensuite être enrichie progressivement à l’aide de
#' la fonction \code{add_check_collection()}.
#'
#' @details
#' La collection de vérifications est représentée par une valeur logique `TRUE`
#' au moment de son initialisation. Cela indique qu’aucune erreur ou alerte
#' n’a encore été détectée.
#' Lorsqu’un contrôle échoue, des messages sont ajoutés à cette collection
#' sous forme de chaînes de caractères.
#'
#' @return
#' Un objet logique `TRUE` représentant une collection vide de vérifications.
#'
#' @examples
#' coll <- make_check_collection()
#' coll
#'
#' @keywords internal
#' @noRd
make_check_collection <- function() {
    return(TRUE)
}

#' @title Ajoute un résultat de checks à une collection
#'
#' @description
#' Ajoute un message de checks à une collection existante créée par
#' \code{make_check_collection()}.
#' Cette fonction permet d’agréger plusieurs messages d’erreurs ou
#' d’avertissements dans un seul objet, utile pour des fonctions de validation
#' interne.
#'
#' @param coll Une collection de checks, généralement créée avec
#' \code{make_check_collection()}.
#' Peut être une valeur logique (`TRUE`) ou une chaîne de caractères contenant
#' déjà des messages.
#' @param check_output Résultat du contrôle. Doit être `TRUE` si le contrôle
#' est validé, ou une chaîne de caractères décrivant le problème sinon.
#' @param .var.name (optionnel) Nom de la variable ou de l’objet concerné
#' par le contrôle. Si fourni, il est ajouté au début du message.
#'
#' @details
#' Si `check_output` vaut `TRUE`, la collection est retournée telle quelle.
#' Dans le cas contraire, le message est ajouté au texte existant dans la
#' collection. Le résultat final est une chaîne de caractères contenant
#' l’ensemble des messages, séparés par des sauts de ligne.
#'
#' @return
#' Une chaîne de caractères contenant tous les messages de checks,
#' ou `TRUE` si aucun problème n’a été détecté.
#'
#' @examples
#' coll <- make_check_collection()
#' coll <- add_check_collection(coll, "Valeur hors intervalle", .var.name = "x")
#' coll
#'
#' @keywords internal
#' @noRd
add_check_collection <- function(coll, check_output, .var.name = NULL) {
    if (isTRUE(check_output)) {
        return(coll)
    } else if (!is.null(.var.name)) {
        check_output <- paste("*", .var.name, check_output)
    }

    if (isTRUE(coll)) {
        return(check_output)
    } else {
        return(paste(coll, "\n", check_output))
    }
}

# Initialisation ---------------------------------------------------------------

set.seed(2024L)


# Tests de résultats positifs --------------------------------------------------

testthat::test_that("good result for integer date month", {
    for (year in good_years) {
        testthat::expect_silent(assert_date_ts(x = year, frequency_ts = 12L))
        for (month in good_months) {
            testthat::expect_silent(
                assert_date_ts(x = c(year, month), frequency_ts = 12L)
            )
        }
    }
})

testthat::test_that("good result for integer date quarter", {
    for (year in good_years) {
        testthat::expect_silent(assert_date_ts(year, frequency_ts = 4L))
        for (quarter in good_quarters) {
            testthat::expect_silent(
                assert_date_ts(x = c(year, quarter), frequency_ts = 4L)
            )
        }
    }
})


# Tests positifs avec warning --------------------------------------------------

## Mensuel -----------------------------------------------------------------

testthat::test_that("warning for integer month", {
    for (good_year in good_years) {
        for (warning_month in warning_integer_months) {
            warning_date <- c(good_year, warning_month)
            testthat::expect_warning(
                assert_date_ts(x = warning_date, frequency_ts = 12L),
                regexp = invalid_monthly_period
            )
        }
    }
})

testthat::test_that("warning for double date", {
    for (warning_year in double_years) {
        warning_date <- warning_year
        testthat::expect_warning(
            assert_date_ts(x = warning_date, frequency_ts = 12L),
            regexp = message_double("warning_date")
        )

        for (good_month in good_months) {
            warning_date <- c(warning_year, good_month)
            testthat::expect_warning(
                assert_date_ts(x = warning_date, frequency_ts = 12L),
                regexp = message_double("warning_date")
            )
        }
    }

    for (good_year in good_years) {
        for (warning_month in double_months) {
            warning_date <- c(good_year, warning_month)
            testthat::expect_warning(
                assert_date_ts(x = warning_date, frequency_ts = 12L),
                regexp = message_double("warning_date")
            )
        }
    }

    for (warning_year in double_years) {
        for (warning_month in double_months) {
            warning_date <- c(warning_year, warning_month)
            testthat::expect_warning(
                assert_date_ts(x = warning_date, frequency_ts = 12L),
                regexp = message_double("warning_date")
            )
        }
    }
})

testthat::test_that("several warning", {
    for (warning_year in double_years) {
        for (warning_month in warning_integer_months) {
            warning_date <- c(warning_year, warning_month)
            w <- testthat::capture_warnings(
                assert_date_ts(x = warning_date, frequency_ts = 12L)
            )
            testthat::expect_match(
                object = w,
                regexp = message_double("warning_date"),
                all = FALSE
            )
            testthat::expect_match(
                object = w,
                regexp = invalid_monthly_period,
                all = FALSE
            )
        }
    }

    for (warning_year in double_years) {
        for (warning_month in warning_double_months) {
            warning_date <- c(warning_year, warning_month)
            w <- testthat::capture_warnings(
                assert_date_ts(x = warning_date, frequency_ts = 12L)
            )
            testthat::expect_match(
                object = w,
                regexp = message_double("warning_date"),
                all = FALSE
            )
            testthat::expect_match(
                object = w,
                regexp = invalid_monthly_period,
                all = FALSE
            )
        }
    }

    for (good_year in good_years) {
        for (warning_month in warning_double_months) {
            warning_date <- c(warning_year, warning_month)
            w <- testthat::capture_warnings(
                assert_date_ts(x = warning_date, frequency_ts = 12L)
            )
            testthat::expect_match(
                object = w,
                regexp = message_double("warning_date"),
                all = FALSE
            )
            testthat::expect_match(
                object = w,
                regexp = invalid_monthly_period,
                all = FALSE
            )
        }
    }
})


## Trimestriel -----------------------------------------------------------------

testthat::test_that("warning for integer date", {
    for (good_year in good_years) {
        for (warning_quarter in warning_integer_quarters) {
            warning_date <- c(good_year, warning_quarter)
            testthat::expect_warning(
                {
                    boolRes <- assert_date_ts(
                        x = warning_date,
                        frequency_ts = 4L
                    )
                },
                regexp = invalid_quaterly_period
            )
            testthat::expect_silent(boolRes)
        }
    }
})

testthat::test_that("warning for double date", {
    for (warning_year in double_years) {
        warning_date <- warning_year
        testthat::expect_warning(
            {
                boolRes <- assert_date_ts(x = warning_date, frequency_ts = 4L)
            },
            regexp = message_double("warning_date")
        )
        testthat::expect_silent(boolRes)

        for (good_quarter in good_quarters) {
            warning_date <- c(warning_year, good_quarter)
            testthat::expect_warning(
                {
                    boolRes <- assert_date_ts(
                        x = warning_date,
                        frequency_ts = 4L
                    )
                },
                regexp = message_double("warning_date")
            )
            testthat::expect_silent(boolRes)
        }
    }

    for (good_year in good_years) {
        for (warning_quarter in double_quarters) {
            warning_date <- c(good_year, warning_quarter)
            testthat::expect_warning(
                {
                    boolRes <- assert_date_ts(
                        x = warning_date,
                        frequency_ts = 4L
                    )
                },
                regexp = message_double("warning_date")
            )
            testthat::expect_silent(boolRes)
        }
    }

    for (warning_year in double_years) {
        for (warning_quarter in double_quarters) {
            warning_date <- c(warning_year, warning_quarter)
            testthat::expect_warning(
                {
                    boolRes <- assert_date_ts(
                        x = warning_date,
                        frequency_ts = 4L
                    )
                },
                regexp = message_double("warning_date")
            )
            testthat::expect_silent(boolRes)
        }
    }
})

testthat::test_that("several warning", {
    for (warning_year in double_years) {
        for (warning_quarter in warning_integer_quarters) {
            warning_date <- c(warning_year, warning_quarter)
            w <- testthat::capture_warnings({
                boolRes <- assert_date_ts(x = warning_date, frequency_ts = 4L)
            })
            testthat::expect_match(
                object = w,
                regexp = message_double("warning_date"),
                all = FALSE
            )
            testthat::expect_match(
                object = w,
                regexp = invalid_quaterly_period,
                all = FALSE
            )

            testthat::expect_silent(boolRes)
        }
    }

    for (warning_year in double_years) {
        for (warning_quarter in warning_double_quarters) {
            warning_date <- c(warning_year, warning_quarter)
            w <- testthat::capture_warnings({
                boolRes <- assert_date_ts(x = warning_date, frequency_ts = 4L)
            })
            testthat::expect_match(
                object = w,
                regexp = message_double("warning_date"),
                all = FALSE
            )
            testthat::expect_match(
                object = w,
                regexp = invalid_quaterly_period,
                all = FALSE
            )

            testthat::expect_silent(boolRes)
        }
    }

    for (good_year in good_years) {
        for (warning_quarter in warning_double_quarters) {
            warning_date <- c(warning_year, warning_quarter)
            w <- testthat::capture_warnings({
                boolRes <- assert_date_ts(x = warning_date, frequency_ts = 4L)
            })
            testthat::expect_match(
                object = w,
                regexp = message_double("warning_date"),
                all = FALSE
            )
            testthat::expect_match(
                object = w,
                regexp = invalid_quaterly_period,
                all = FALSE
            )

            testthat::expect_silent(boolRes)
        }
    }
})


## Fréquence -------------------------------------------------------------------

testthat::test_that("warning for integer date", {
    for (good_year in good_years) {
        for (good_quarter in good_quarters) {
            good_date <- c(good_year, good_quarter)
            testthat::expect_warning(
                {
                    boolRes <- assert_date_ts(x = good_date, frequency_ts = 4.0)
                },
                regexp = message_double("frequency_ts")
            )
            testthat::expect_silent(boolRes)
        }
    }

    for (good_year in good_years) {
        for (good_month in good_months) {
            good_date <- c(good_year, good_month)
            testthat::expect_warning(
                assert_date_ts(x = good_date, frequency_ts = 12.0),
                regexp = message_double("frequency_ts")
            )
        }
    }
})


# Tests de résultats négatifs --------------------------------------------------

testthat::test_that("miscellaneous warn are not allowed", {
    for (wrong_warn in c(list(0., 0L), object_bank_R[-29L])) {
        testthat::expect_error(assert_date_ts(
            x = 2020L,
            frequency_ts = 12L,
            warn = wrong_warn
        ))
    }
})

testthat::test_that("miscellaneous warn are not allowed in collection", {
    for (wrong_warn in c(list(0., 0L), object_bank_R[-29L])) {
        coll <- checkmate::makeAssertCollection()
        assert_date_ts(
            x = 2020L,
            frequency_ts = 12L,
            warn = wrong_warn,
            add = coll
        )
        testthat::expect_error(reportAssertions(coll))
    }
})

testthat::test_that("detection of wrong dates", {
    for (wrong_date in c(list_wrong_date_ts, object_bank_R[-10L], rnorm(10L))) {
        testthat::expect_error(assert_date_ts(
            x = wrong_date,
            frequency_ts = 12L
        ))
        testthat::expect_error(assert_date_ts(
            x = wrong_date,
            frequency_ts = 4L
        ))
    }
})

testthat::test_that("detection of wrong dates in collection", {
    for (wrong_date in c(list_wrong_date_ts, object_bank_R[-10L], rnorm(10L))) {
        coll <- checkmate::makeAssertCollection()
        assert_date_ts(x = wrong_date, frequency_ts = 12L, add = coll)
        testthat::expect_error(reportAssertions(coll))

        coll <- checkmate::makeAssertCollection()
        assert_date_ts(x = wrong_date, frequency_ts = 4L, add = coll)
        testthat::expect_error(reportAssertions(coll))
    }
})

testthat::test_that("miscellaneous frequency are not allowed", {
    for (wrong_frequency in c(object_bank_R, weird_frequency)) {
        testthat::expect_error(
            assert_date_ts(
                x = 2020L,
                frequency_ts = wrong_frequency
            )
        )
    }
})

testthat::test_that("miscellaneous frequency are not allowed in collection", {
    for (wrong_frequency in c(object_bank_R, weird_frequency)) {
        coll <- checkmate::makeAssertCollection()
        assert_date_ts(x = 2020L, wrong_frequency, warn = TRUE, add = coll)
        testthat::expect_error(reportAssertions(coll))
    }
})

testthat::test_that("good result", {
    testthat::expect_no_error(assert_date_ts(x = 2020L, frequency_ts = 12L))
    testthat::expect_no_error(assert_date_ts(
        x = c(2020L, 12L),
        frequency_ts = 12L
    ))
    testthat::expect_no_error(assert_date_ts(
        x = c(2020L, 2L),
        frequency_ts = 4L
    ))
})

# Initialisation ---------------------------------------------------------------

set.seed(2038L)


# Tests de résultats positifs --------------------------------------------------

testthat::test_that("good result for Date", {
    list_date <- c(Sys.Date(), create_random_type("Date", len = 10L))

    for (k in seq_along(list_date)) {
        date <- list_date[k]
        res <- testthat::expect_silent(assert_scalar_date(x = date))
        testthat::expect_identical(res, date)
    }
})


# Tests de résultats négatifs --------------------------------------------------

testthat::test_that("Several Date are not allowed", {
    testthat::expect_error(
        assert_scalar_date(
            x = seq(
                from = as.Date("2023-08-04"),
                to = Sys.Date(),
                by = "months"
            )
        )
    )

    for (len in list_len[-2L]) {
        testthat::expect_error(
            assert_scalar_date(x = create_random_type("Date", len = len))
        )
    }
})

testthat::test_that("miscellaneous Object are not allowed", {
    for (wrong_date in object_bank_R[-34L]) {
        testthat::expect_error(
            assert_scalar_date(x = wrong_date)
        )
    }
})

# Initialisation ---------------------------------------------------------------

set.seed(2039L)


# Tests de résultats positifs --------------------------------------------------

testthat::test_that("good result for no error", {
    testthat::expect_silent(assert_expression(2 + 2))
    testthat::expect_silent(assert_expression(log(1)))
})


# Tests de résultats négatifs --------------------------------------------------

testthat::test_that("wrong result in case of warnings", {
    testthat::expect_error(assert_expression(expr = log(-1)))
    testthat::expect_error(assert_expression(warning("Un petit warning")))
})

testthat::test_that("wrong result in case of error", {
    testthat::expect_error(assert_expression(log("a")))
    testthat::expect_error(assert_expression(stop("Une petit erreur")))
})

testthat::test_that("wrong result in case of warnings and errors", {
    testthat::expect_error(assert_expression({
        warning("D'abord un warning")
        stop("Puis une erreur")
    }))
})

# Initialisation ---------------------------------------------------------------

set.seed(2037L)


# Tests de résultats positifs --------------------------------------------------

testthat::test_that("good result for integer", {
    list_integer <- abs(c(
        list_len,
        list_lag,
        create_random_type("integer", len = 10L)
    )) +
        1L

    for (k in list_integer) {
        res <- testthat::expect_silent(assert_scalar_natural(x = k))
        testthat::expect_identical(res, k)
    }
})

testthat::test_that("good result for double without warning", {
    list_double <- as.double(
        abs(c(list_lag, list_len, create_random_type("integer", len = 10L))) +
            1L
    )

    for (k in list_double) {
        res <- testthat::expect_silent(assert_scalar_natural(
            x = k,
            warn = FALSE
        ))
        testthat::expect_identical(res, as.integer(k))
    }
})


# Tests positifs avec warning --------------------------------------------------

testthat::test_that("warning for integer date", {
    list_double <- as.double(
        abs(c(list_lag, list_len, create_random_type("integer", len = 10L))) +
            1L
    )

    for (k in list_double) {
        testthat::expect_warning(
            {
                res1 <- assert_scalar_natural(x = k, warn = TRUE)
            },
            regexp = double_instead_of_integer
        )
        testthat::expect_identical(res1, as.integer(k))

        testthat::expect_warning(
            {
                res2 <- assert_scalar_natural(x = k)
            },
            regexp = double_instead_of_integer
        )
        testthat::expect_identical(res2, as.integer(k))
    }
})


# Tests de résultats négatifs --------------------------------------------------

testthat::test_that("miscellaneous integer x are not allowed", {
    wrong_integers <- c(
        list(0., 0L),
        list_wrong_date_ts,
        object_bank_R[-10L],
        rnorm(10L),
        as.double(
            -abs(c(
                list_lag,
                list_len,
                create_random_type("integer", len = 10L)
            ))
        ),
        -abs(c(list_len, list_lag, create_random_type("integer", len = 10L)))
    )

    for (wrong_integer in wrong_integers) {
        testthat::expect_error(
            assert_scalar_natural(x = wrong_integer, warn = FALSE)
        )
    }
})

# Tests de résultats positifs --------------------------------------------------

testthat::test_that("good result for integer frequency", {
    testthat::expect_silent(
        object = {
            res <- assert_frequency(x = 12L)
        }
    )
    testthat::expect_identical(res, 12L)

    testthat::expect_silent(
        object = {
            res <- assert_frequency(x = 4L)
        }
    )
    testthat::expect_identical(res, 4L)
})

testthat::test_that("good result for double frequency without warning", {
    testthat::expect_silent(
        object = {
            res <- assert_frequency(x = 12.0, warn = FALSE)
        }
    )
    testthat::expect_identical(res, 12L)

    testthat::expect_silent(
        object = {
            res <- assert_frequency(x = 4.0, warn = FALSE)
        }
    )
    testthat::expect_identical(object = res, expected = 4L)
})


# Tests positifs avec warning --------------------------------------------------

testthat::test_that("warning for integer date", {
    testthat::expect_warning(
        object = {
            res <- assert_frequency(x = 12.0, warn = TRUE)
        },
        regexp = double_instead_of_integer
    )
    testthat::expect_identical(object = res, expected = 12L)

    testthat::expect_warning(
        object = {
            res <- assert_frequency(x = 4.0, warn = TRUE)
        },
        regexp = double_instead_of_integer
    )
    testthat::expect_identical(object = res, expected = 4L)

    testthat::expect_warning(
        object = {
            res <- assert_frequency(x = 12.0)
        },
        regexp = double_instead_of_integer
    )
    testthat::expect_identical(res, 12L)

    testthat::expect_warning(
        object = {
            res <- assert_frequency(x = 4.0)
        },
        regexp = double_instead_of_integer
    )
    testthat::expect_identical(object = res, expected = 4L)
})


# Tests de résultats négatifs --------------------------------------------------

testthat::test_that("miscellaneous frequency are not allowed", {
    for (wrong_frequency in c(object_bank_R, weird_frequency)) {
        testthat::expect_error(
            object = assert_frequency(x = wrong_frequency, warn = FALSE)
        )
    }
})

# Tests de résultats positifs --------------------------------------------------

testthat::test_that("good result for Date", {
    date <- Sys.Date()
    res <- testthat::expect_silent(assert_scalar_date(x = date))
    testthat::expect_identical(res, date)
})


# Tests de résultats négatifs --------------------------------------------------

testthat::test_that("Several Date are not allowed", {
    testthat::expect_error(
        assert_scalar_date(
            x = seq(
                from = as.Date("2023-08-04"),
                to = as.Date("2024-08-04"),
                by = "months"
            )
        )
    )
})

#' @title Conversion au format date_ts
#'
#' @description Les fonctions `as_yyyytt` et `as_yyyymm` convertissent une date
#' du format TimeUnits au format `date_ts`.
#'
#' @param timeunits une date en année (Par exemple 2015.25 pour le 2ème
#' trimestre 2015 ou `2021.83333333333` pour novembre 2021)
#'
#' @returns En sortie, ces fonctions retournent la date au format `date_ts`
#' (c'est-à-dire un vecteur d'entiers de la forme `AAAA`, `c(AAAA, MM)` ou
#' `c(AAAA, TT)`)
#' @details
#' La fonction `as_yyyytt` retourne la date par trimestre et la fonction
#' `as_yyyymm` retourne la date par mois.
#'
#' @export
#'
#' @examples
#' as_yyyytt(2019.75) # 4ème trimestre 2019
#' as_yyyytt(2020) # 1er trimestre 2020
#' as_yyyytt(2022 + 1 / 4) # 2ème trimestre 2022
#'
#' as_yyyymm(2019.75) # Octobre 2019
#' as_yyyymm(2020) # Janvier 2020
#' as_yyyymm(2020 + 1 / 12) # Février 2020
#' as_yyyymm(2020 + 12 / 12) # Janvier 2021
#'
as_yyyytt <- function(timeunits) {
    # Check de l'objet TimeUnits
    assert_timeunits(timeunits, frequency_ts = 4L, .var.name = "timeunits")

    return(as.integer(round(c(timeunits %/% 1L, (timeunits %% 1L) * 4L + 1L))))
}

#' @rdname as_yyyytt
#'
#' @export
#'
as_yyyymm <- function(timeunits) {
    # Check de l'objet TimeUnits
    assert_timeunits(timeunits, frequency_ts = 12L, .var.name = "timeunits")

    return(as.integer(round(c(timeunits %/% 1L, (timeunits %% 1L) * 12L + 1L))))
}

#' @title Conversion entre date mensuelle et trimestrielle
#'
#' @description Les fonctions `trim2mens` et `mens2trim` convertissent une
#' `date_ts` du format mensuelle `c(AAAA, MM)` au format trimestrielle `c(AAAA,
#' TT)`.
#'
#' @param date_ts un vecteur numérique, de préférence `integer`, au format
#' `AAAA`, `c(AAAA, MM)` ou `c(AAAA, TT)`
#'
#' @returns En sortie, la fonction retourne la date toujours au format
#' `date_ts`.
#'
#' @export
#'
#' @examples
#' trim2mens(c(2019L, 4L)) # 4ème trimestre 2019 --> Octobre 2019
#' trim2mens(c(2020L, 1L)) # 1er trimestre 2020 --> Janvier 2020
#'
#' mens2trim(c(2019L, 4L)) # Avril 2019 --> 2ème trimestre 2019
#' mens2trim(c(2020L, 11L)) # Novembre 2020 --> 4ème trimestre 2020
#'
trim2mens <- function(date_ts) {
    # Check du format date_ts
    date_ts <- assert_date_ts(
        x = date_ts,
        frequency_ts = 4L,
        .var.name = "date_ts"
    )

    year <- date_ts[1L] + (date_ts[2L] - 1L) %/% 4L
    trim <- (date_ts[2L] - 1L) %% 4L + 1L
    return(as.integer(c(year, trim * 3L - 2L)))
}

#' @rdname trim2mens
#'
#' @export
#'
mens2trim <- function(date_ts) {
    # Check du format date_ts
    date_ts <- assert_date_ts(
        x = date_ts,
        frequency_ts = 12L,
        .var.name = "date_ts"
    )

    year <- date_ts[1L] + (date_ts[2L] - 1L) %/% 12L
    month <- (date_ts[2L] - 1L) %% 12L + 1L
    return(as.integer(c(year, 1L + ((month - 1L) %/% 3L))))
}

#' @title Conversion d'une date du format date_ts au format TimeUnits
#'
#' @inheritParams trim2mens
#' @param frequency_ts un entier qui vaut `4L` (ou `4.0`) pour les séries
#' trimestrielles et `12L` (ou `12.0`) pour les séries mensuelles.
#'
#' @returns En sortie, la fonction retourne la date au format `AAAA + TT/4` ou
#' `AAAA + MM/12` (un numeric de longueur 1).
#'
#' @details
#' `AAAA` signifie que l'année est au format numérique avec 4 chiffres
#' (Exemple : l'année deux mille vingt-deux s'écrit 2022 et non 22)
#' MM signifie que le mois est au format numérique (Exemple : le mois de mai
#' s'écrit 5, le moi de décembre s'écrit 12)
#' TT signifie que le trimestre est au format numérique (Exemple : le troisième
#' trimestre s'écrit 3)
#' @export
#'
#' @examples
#'
#' # Avril 2020
#' date_ts2timeunits(date_ts = c(2020L, 4L), frequency_ts = 12L)
#' # Novembre 2020
#' date_ts2timeunits(date_ts = c(2022L, 11L), frequency_ts = 12L)
#'
#' # 4ème trimestre de 2022
#' date_ts2timeunits(date_ts = c(2022, 4L), frequency_ts = 4L)
#' # 2ème trimestre de 1995
#' date_ts2timeunits(date_ts = c(1995L, 2L), frequency_ts = 4L)
#'
date_ts2timeunits <- function(date_ts, frequency_ts) {
    # Check de la fréquence
    frequency_ts <- assert_frequency(
        frequency_ts,
        add = NULL,
        .var.name = "frequency_ts"
    )
    # Check du format date_ts
    date_ts <- assert_date_ts(
        x = date_ts,
        frequency_ts = frequency_ts,
        add = NULL,
        .var.name = "date_ts"
    )

    # date_ts must be of size 2 else:
    #   - an error would have been raised
    #   - or date_ts would have been modified

    return(date_ts[1L] + (date_ts[2L] - 1L) / frequency_ts)
}

#' @title Conversion d'une date au format TS
#'
#' @description La fonction `date2date_ts` prend en argument une date au format
#' date (integer avec une class Date) et la convertit au format `date_ts` :
#' `c(AAAA, MM)` ou `c(AAAA, TT)` avec le mois ou trimestre en cours.
#'
#' @param date un objet de type Date
#' @param frequency_ts un entier qui vaut `4L` (ou `4.0`) pour les séries
#' trimestrielles et `12L` (ou `12.0`) pour les séries mensuelles.
#'
#' @returns En sortie, la fonction retourne la date au format `date_ts`
#' (`c(AAAA, MM)` ou `c(AAAA, TT)`) avec le mois ou trimestre en cours selon
#' l'argument `frequency_ts`.
#' @export
#'
#' @examples
#'
#' date2date_ts(as.Date("2000-01-01"))
#' date2date_ts(as.Date("2000-01-01"), frequency_ts = 12L)
#'
#' date2date_ts(as.Date("2021-10-01"), frequency_ts = 12L)
#' date2date_ts(as.Date("2021-10-01"), frequency_ts = 4L)
#'
date2date_ts <- function(date, frequency_ts = 12L) {
    coll <- checkmate::makeAssertCollection()

    # Check de l'objet date
    assert_scalar_date(date, add = coll, .var.name = "date")
    # Check de la fréquence
    frequency_ts <- assert_frequency(
        frequency_ts,
        add = coll,
        .var.name = "frequency_ts"
    )

    checkmate::reportAssertions(coll)

    year <- as.integer(format(date, format = "%Y"))
    month <- as.integer(format(date, format = "%m"))

    if (frequency_ts == 4L) {
        month <- 1L + ((month - 1L) %/% 3L)
    }

    return(c(year, month))
}

#' @title Retire une année à une date
#'
#' @param date un objet de type Date
#' @param n un entier
#'
#' @description La fonction `substr_year` retire `n` annnée(s) à une date.
#' @returns En sortie, la fonction retourne un objet de type Date (atomic) de
#' longueur 1.
#' @export
#'
#' @examples
#'
#' substr_year(as.Date("2000-02-29"), n = 1L)
#' substr_year(as.Date("2000-02-29"), n = 3L)
#' substr_year(as.Date("2000-02-29"), n = 4L)
#' substr_year(as.Date("2000-02-29"), n = 16L)
#'
#' substr_year(as.Date("2023-01-25"), n = 10L)
#' substr_year(as.Date("2022-11-01"), n = 3L)
#'
substr_year <- function(date, n = 1L) {
    coll <- checkmate::makeAssertCollection()

    assert_scalar_date(date, add = coll, .var.name = "date")
    n <- assert_scalar_natural(n, add = coll, .var.name = "n")

    checkmate::reportAssertions(coll)

    date_1 <- as.Date(paste("2000", format(date, format = "%m-%d"), sep = "-"))
    date_2 <- as.Date("2000-02-28")
    before_leap <- date_1 <= date_2

    year <- as.integer(format(date, format = "%Y")) - before_leap
    years <- year:(year - n + 1L)
    leap_year <- sum(as.logical(
        (years %% 4L == 0L) -
            (years %% 100L == 0L) +
            (years %% 400L == 0L)
    ))

    return(date - 365L * n - leap_year)
}

#' @title Conversion d'une date du format TS au format date
#'
#' @inheritParams date_ts2timeunits
#'
#' @returns En sortie, la fonction retourne un objet de type Date (atomic) de
#' longueur 1 qui correspond à l'objet `date_ts`.
#' @export
#'
#' @examples
#'
#' date_ts2date(date_ts = c(2020L, 11L), frequency_ts = 12L)
#' date_ts2date(date_ts = c(1995L, 2L), frequency_ts = 4L)
#'
date_ts2date <- function(date_ts, frequency_ts) {
    coll <- checkmate::makeAssertCollection()

    # Check de la fréquence
    frequency_ts <- assert_frequency(
        frequency_ts,
        add = coll,
        .var.name = "frequency_ts"
    )
    # Check du format date_ts
    if (isTRUE(check_frequency(frequency_ts, warn = FALSE))) {
        date_ts <- assert_date_ts(
            x = date_ts,
            frequency_ts,
            add = coll,
            .var.name = "date_ts"
        )
    }

    checkmate::reportAssertions(coll)

    year <- date_ts[1L]

    if (length(date_ts) == 2L) {
        if (frequency_ts == 4L) {
            month <- sprintf("%02d", date_ts[2L] * 3L - 2L)
        } else if (frequency_ts == 12L) {
            month <- sprintf("%02d", date_ts[2L])
        }
    }

    if (year < 0L) {
        return(substr_year(
            date = as.Date(paste("0000", month, "01", sep = "-")),
            n = -year
        ))
    } else {
        return(as.Date(paste(year, month, "01", sep = "-")))
    }
}

#' @title Convertit un objet \code{ts} en \code{data.frame}
#'
#' @param x un objet de type \code{ts}.
#'
#' @returns En sortie la fonction retourne un \code{data.frame} avec autant de
#' colonnes que \code{x} et une de plus pour la date.
#'
#' @examples
#' ts2df(AirPassengers)
#' ts2df(Seatbelts)
#'
#' @export
#'
ts2df <- function(x) {
    # Check de l'objet x
    assert_ts(x, .var.name = "x", allow_mts = TRUE)

    if (stats::is.mts(x)) {
        length_series <- nrow(x)
    } else {
        length_series <- length(x)
    }

    frequency_ts <- as.integer(stats::frequency(x))
    first_timeunits <- stats::time(x)[1L]

    if (frequency_ts == 12L) {
        first_date_ts <- as_yyyymm(first_timeunits)
        date_vec <- seq.Date(
            from = date_ts2date(first_date_ts, frequency_ts = 12L),
            length.out = length_series,
            by = "month"
        )
    } else if (frequency_ts == 4L) {
        first_date_ts <- as_yyyymm(first_timeunits)
        date_vec <- seq.Date(
            from = date_ts2date(first_date_ts, frequency_ts = 4L),
            length.out = length_series,
            by = "quarter"
        )
    }

    rownames_libelles <- libelles(
        date_ts = first_date_ts,
        frequency_ts = frequency_ts,
        n = length_series
    )
    x <- data.frame(date = date_vec, x)
    rownames(x) <- rownames_libelles

    return(x)
}

# Initialisation ---------------------------------------------------------------

set.seed(2027L)


# Tests de résultats positifs --------------------------------------------------

testthat::test_that("good result for integer date", {
    for (quarter in c(warning_integer_quarters, good_quarters)) {
        for (year in good_years) {
            TSdate <- as.integer(c(year + quarter %/% 4L, quarter %% 4L + 1L))
            res <- as_yyyytt(year + quarter / 4L)
            testthat::expect_type(res, "integer")
            testthat::expect_identical(res, TSdate)
        }
    }
})

# Tests de résultats négatifs --------------------------------------------------

testthat::test_that("miscellaneous input are not allowed", {
    for (wrong_time in object_bank_R[-c(10L, 16L)]) {
        testthat::expect_error(as_yyyytt(wrong_time))
    }
    for (wrong_time in list_wrong_timeunits) {
        testthat::expect_error(as_yyyytt(wrong_time))
    }
})

testthat::test_that("good result for integer date", {
    testthat::expect_identical(
        object = as_yyyymm(2020L),
        expected = c(2020L, 1L)
    )
    testthat::expect_identical(
        object = as_yyyymm(2020L),
        expected = c(2020L, 2L)
    )
})

testthat::test_that("good result for integer freq", {
    testthat::expect_identical(
        object = as_yyyymm(2020L),
        expected = c(2020L, 1L)
    )
    testthat::expect_identical(
        object = libelles_one_date(2020L, 12),
        expected = "Jan 2020"
    )
})

# Initialisation ---------------------------------------------------------------

set.seed(2029L)

# Tests de résultats positifs --------------------------------------------------

testthat::test_that("good result for integer date", {
    for (good_year in good_years) {
        for (quarter in good_quarters) {
            real_quarter <- (quarter - 1L) %% 4L + 1L
            year_real <- good_year + (quarter - 1L) %/% 4L
            date_expected <- as.integer(c(
                year_real,
                conversion_quarter_month[real_quarter, "month"]
            ))
            res <- trim2mens(c(good_year, quarter))

            testthat::expect_identical(res, date_expected)
            testthat::expect_type(res, "integer")
        }
    }
})


# Tests de résultats négatifs --------------------------------------------------

testthat::test_that("detection of wrong dates", {
    for (wrong_date in list_wrong_date_ts) {
        testthat::expect_error(trim2mens(wrong_date))
    }
})


# Tests positifs avec warning --------------------------------------------------

testthat::test_that("warning for integer date", {
    for (good_year in good_years) {
        for (warning_quarter in warning_integer_quarters) {
            testthat::expect_warning(
                {
                    resMonthly <- trim2mens(c(good_year, warning_quarter))
                },
                regexp = "Assertion on 'period' failed: Element 1 is not <= 4.|Assertion on 'period' failed: Element 1 is not >= 1."
            )

            real_quarter <- (warning_quarter - 1L) %% 4L + 1L
            year_real <- good_year + (warning_quarter - 1L) %/% 4L
            date_expected <- as.integer(c(
                year_real,
                conversion_quarter_month[real_quarter, "month"]
            ))

            testthat::expect_identical(resMonthly, date_expected)
            testthat::expect_type(resMonthly, "integer")
        }
    }
})

testthat::test_that("warning for double date", {
    for (warning_year in double_years) {
        for (good_quarter in good_quarters) {
            testthat::expect_warning(
                {
                    resMonthly <- trim2mens(c(warning_year, good_quarter))
                },
                regexp = message_double("date_ts")
            )

            real_quarter <- (good_quarter - 1L) %% 4L + 1L
            year_real <- warning_year + (good_quarter - 1L) %/% 4L
            date_expected <- as.integer(c(
                year_real,
                conversion_quarter_month[real_quarter, "month"]
            ))

            testthat::expect_identical(resMonthly, date_expected)
            testthat::expect_type(resMonthly, "integer")
        }
    }

    for (good_year in good_years) {
        for (warning_quarter in double_quarters) {
            testthat::expect_warning(
                {
                    resMonthly <- trim2mens(c(good_year, warning_quarter))
                },
                regexp = message_double("date_ts")
            )

            real_quarter <- (warning_quarter - 1L) %% 4L + 1L
            year_real <- good_year + (warning_quarter - 1L) %/% 4L
            date_expected <- as.integer(c(
                year_real,
                conversion_quarter_month[real_quarter, "month"]
            ))

            testthat::expect_identical(resMonthly, date_expected)
            testthat::expect_type(resMonthly, "integer")
        }
    }

    for (warning_year in double_years) {
        for (warning_quarter in double_quarters) {
            testthat::expect_warning(
                {
                    resMonthly <- trim2mens(c(good_year, warning_quarter))
                },
                regexp = message_double("date_ts")
            )

            real_quarter <- (warning_quarter - 1L) %% 4L + 1L
            year_real <- good_year + (warning_quarter - 1L) %/% 4L
            date_expected <- as.integer(c(
                year_real,
                conversion_quarter_month[real_quarter, "month"]
            ))

            testthat::expect_identical(resMonthly, date_expected)
            testthat::expect_type(resMonthly, "integer")
        }
    }
})

testthat::test_that("several warning", {
    for (warning_year in double_years) {
        for (warning_quarter in warning_integer_quarters) {
            w <- testthat::capture_warnings({
                resMonthly <- trim2mens(c(warning_year, warning_quarter))
            })
            testthat::expect_match(
                object = w,
                regexp = message_double("date_ts"),
                all = FALSE
            )
            testthat::expect_match(
                object = w,
                regexp = paste(
                    "Assertion on 'period' failed: Element 1 is not <= 4.",
                    "Assertion on 'period' failed: Element 1 is not >= 1.",
                    sep = "|"
                ),
                all = FALSE
            )

            real_quarter <- (warning_quarter - 1L) %% 4L + 1L
            year_real <- warning_year + (warning_quarter - 1L) %/% 4L
            date_expected <- as.integer(c(
                year_real,
                conversion_quarter_month[real_quarter, "month"]
            ))

            testthat::expect_identical(resMonthly, date_expected)
            testthat::expect_type(resMonthly, "integer")
        }
    }

    for (warning_year in double_years) {
        for (warning_quarter in warning_double_quarters) {
            w <- testthat::capture_warnings({
                resMonthly <- trim2mens(c(warning_year, warning_quarter))
            })
            testthat::expect_match(
                object = w,
                regexp = message_double("date_ts"),
                all = FALSE
            )
            testthat::expect_match(
                object = w,
                regexp = paste(
                    "Assertion on 'period' failed: Element 1 is not <= 4.",
                    "Assertion on 'period' failed: Element 1 is not >= 1.",
                    sep = "|"
                ),
                all = FALSE
            )

            real_quarter <- (warning_quarter - 1L) %% 4L + 1L
            year_real <- warning_year + (warning_quarter - 1L) %/% 4L
            date_expected <- as.integer(c(
                year_real,
                conversion_quarter_month[real_quarter, "month"]
            ))

            testthat::expect_identical(resMonthly, date_expected)
            testthat::expect_type(resMonthly, "integer")
        }
    }

    for (good_year in good_years) {
        for (warning_quarter in warning_double_quarters) {
            w <- testthat::capture_warnings({
                resMonthly <- trim2mens(c(good_year, warning_quarter))
            })
            testthat::expect_match(
                object = w,
                regexp = message_double("date_ts"),
                all = FALSE
            )
            testthat::expect_match(
                object = w,
                regexp = paste(
                    "Assertion on 'period' failed: Element 1 is not <= 4.",
                    "Assertion on 'period' failed: Element 1 is not >= 1.",
                    sep = "|"
                ),
                all = FALSE
            )

            real_quarter <- (warning_quarter - 1L) %% 4L + 1L
            year_real <- good_year + (warning_quarter - 1L) %/% 4L
            date_expected <- as.integer(c(
                year_real,
                conversion_quarter_month[real_quarter, "month"]
            ))

            testthat::expect_identical(resMonthly, date_expected)
            testthat::expect_type(resMonthly, "integer")
        }
    }
})

# Initialisation ---------------------------------------------------------------

set.seed(2031L)


# Tests de résultats positifs --------------------------------------------------

testthat::test_that("good result for integer date", {
    for (year in good_years) {
        testthat::expect_equal(
            date_ts2timeunits(date_ts = year, frequency_ts = 12L),
            year
        )
        for (month in good_months) {
            testthat::expect_equal(
                date_ts2timeunits(date_ts = c(year, month), frequency_ts = 12L),
                year + (month - 1) / 12
            )
        }
    }
})

testthat::test_that("good result for integer date", {
    for (year in good_years) {
        testthat::expect_equal(
            date_ts2timeunits(date_ts = year, frequency_ts = 4L),
            year
        )
        for (quarter in good_quarters) {
            testthat::expect_equal(
                date_ts2timeunits(
                    date_ts = c(year, quarter),
                    frequency_ts = 4L
                ),
                year + (quarter - 1) / 4
            )
        }
    }
})


# Test de résultat positif avec warnings ----------------------------------

testthat::test_that("warning for integer date", {
    for (year in good_years) {
        for (month in warning_integer_months) {
            testthat::expect_warning(
                {
                    resTU <- date_ts2timeunits(
                        date_ts = c(year, month),
                        frequency_ts = 12L
                    )
                },
                regexp = invalid_monthly_period
            )
            testthat::expect_equal(resTU, year + (month - 1) / 12)
        }
    }
})


testthat::test_that("good result for integer date", {
    for (year in good_years) {
        for (quarter in warning_integer_quarters) {
            testthat::expect_warning(
                {
                    resTU <- date_ts2timeunits(
                        date_ts = c(year, quarter),
                        frequency_ts = 4L
                    )
                },
                regexp = invalid_quaterly_period
            )
            testthat::expect_equal(resTU, year + (quarter - 1) / 4)
        }
    }
})


# Tests de résultats négatifs --------------------------------------------------

testthat::test_that("miscellaneous date are not allowed", {
    for (wrong_date in list_wrong_date_ts) {
        testthat::expect_error(date_ts2timeunits(
            date_ts = wrong_date,
            frequency_ts = 12L
        ))
    }
    for (wrong_date in list_wrong_date_ts) {
        testthat::expect_error(date_ts2timeunits(
            date_ts = wrong_date,
            frequency_ts = 4L
        ))
    }
})

testthat::test_that("miscellaneous frequency are not allowed", {
    for (wrong_frequency in c(object_bank_R, weird_frequency)) {
        testthat::expect_error(date_ts2timeunits(
            date_ts = create_random_date_ts(),
            frequency_ts = wrong_frequency
        ))
    }
})

#' @title Récupère des valeurs d'un ts
#'
#' @description La fonction `get_value_ts` permet de récupérer des valeurs.
#'
#' @inheritParams first_date
#' @param date_from un vecteur numérique, de préférence `integer` au format
#' `AAAA`, `c(AAAA, MM)` ou `c(AAAA, TT)`
#' @param date_to un vecteur numérique, de préférence `integer` au format
#' `AAAA`, `c(AAAA, MM)` ou `c(AAAA, TT)`
#' @param n un entier
#'
#' @returns En sortie, la fonction retourne un vecteur (atomic) de même type que
#' `series` avec les valeurs extraites.
#'
#' @details
#' Il faut qu'exactement 2 arguments parmi `date_to`, `date_to` et `n` soient
#' renseignés. L'argument `n` combiné avec `date_to` ou `date_from` permet de
#' déterminer combien de période seront retourné à partir de ou jusqu'à la date
#' renseignée.
#'
#' Il faudrait parler d'extraction car contrairement à la fonction
#' \code{window}, ici on retourne un vecteur devaleur et plus un objet ts.
#'
#' @export
#'
#' @examples
#'
#' ts1 <- ts(1:100, start = 2012L, frequency = 12L)
#' ts2 <- ts(letters, start = 2014L, frequency = 4L)
#' ts3 <- ts(exp(-(1:50)), start = 2015L, frequency = 12L)
#'
#' get_value_ts(series = ts1, date_from = c(2015L, 7L), date_to = c(2018L, 6L))
#' get_value_ts(series = ts2, date_from = c(2018L, 4L), n = 4L)
#' get_value_ts(series = ts3, date_to = c(2018L, 4L), n = 14L)
#'
get_value_ts <- function(series, date_from, date_to, n) {
    coll <- NULL

    # Check de l'objet series
    assert_ts(series, add = coll, .var.name = "series")

    frequency_ts <- as.integer(stats::frequency(series))

    if ((missing(date_to) + missing(date_from) + missing(n)) != 1L) {
        stop(
            c(
                "Exactement 2 des arguments `date_from`, `date_to` et ",
                "`n` doivent \u00eatre renseign\u00e9s et un manquant."
            ),
            call. = FALSE
        )
    } else if (missing(date_from)) {
        # Check l'argument n
        n <- assert_scalar_natural(n, add = coll, .var.name = "n")
        # Check du format date_ts
        date_to <- assert_date_ts(
            x = date_to,
            frequency_ts = frequency_ts,
            .var.name = "date_from"
        )

        date_from <- previous_date_ts(
            date_ts = date_to,
            frequency_ts = frequency_ts,
            lag = n - 1L
        )
    } else if (missing(date_to)) {
        # Check l'argument n
        n <- assert_scalar_natural(n, add = coll, .var.name = "n")
        # Check du format date_ts
        date_from <- assert_date_ts(
            x = date_from,
            frequency_ts = frequency_ts,
            .var.name = "date_from"
        )

        date_to <- next_date_ts(
            date_ts = date_from,
            frequency_ts = frequency_ts,
            lag = n - 1L
        )
    } else if (missing(n)) {
        # Check du format date_ts
        date_from <- assert_date_ts(
            x = date_from,
            frequency_ts = frequency_ts,
            .var.name = "date_from"
        )
        # Check du format date_ts
        date_to <- assert_date_ts(
            x = date_to,
            frequency_ts = frequency_ts,
            .var.name = "date_from"
        )
        checkmate::assert_true(is_before(
            a = date_from,
            b = date_to,
            frequency_ts = frequency_ts,
            strict = FALSE
        ))
    }

    end_ts <- normalize_date_ts(
        date_ts = stats::end(series),
        frequency_ts = frequency_ts,
        test = FALSE
    )
    start_ts <- normalize_date_ts(
        date_ts = stats::start(series),
        frequency_ts = frequency_ts,
        test = FALSE
    )

    output_value <- series

    if (is.raw(series)) {
        if (
            is_before(
                a = date_to,
                b = start_ts,
                frequency_ts = frequency_ts,
                strict = TRUE
            ) ||
                is_before(
                    a = end_ts,
                    b = date_from,
                    frequency_ts = frequency_ts,
                    strict = TRUE
                )
        ) {
            return(rep(
                x = as.raw(0x00),
                times = diff_periode(
                    a = date_to,
                    b = date_from,
                    frequency_ts = frequency_ts
                )
            ))
        }

        if (
            is_before(
                a = date_to,
                b = end_ts,
                strict = FALSE,
                frequency_ts = frequency_ts
            )
        ) {
            output_value <- stats::window(
                x = output_value,
                end = date_to
            )
            after <- 0L
        } else {
            after <- diff_periode(
                a = date_to,
                b = end_ts,
                frequency_ts = frequency_ts
            ) -
                1L
        }

        if (
            is_before(
                a = start_ts,
                b = date_from,
                strict = FALSE,
                frequency_ts = frequency_ts
            )
        ) {
            output_value <- stats::window(
                x = output_value,
                start = date_from
            )
            before <- 0L
        } else {
            before <- diff_periode(
                a = start_ts,
                b = date_from,
                frequency_ts = frequency_ts
            ) -
                1L
        }

        attributes(output_value) <- NULL
        output_value <- c(
            rep(x = as.raw(0x00), times = before),
            output_value,
            rep(x = as.raw(0x00), times = after)
        )
    } else {
        output_value <- stats::window(
            x = output_value,
            start = date_from,
            end = date_to,
            extend = TRUE
        )
        attributes(output_value) <- NULL
    }

    return(output_value)
}

#' @title Change certaines valeurs d'un ts
#'
#' @description
#' La fonction `set_value_ts` modifie la ou les valeurs d'un objet
#' ts à une date donnée.
#'
#' @inheritParams first_date
#' @inheritParams trim2mens
#' @param replacement un vecteur de même type que le ts `series`
#'
#' @returns En sortie, la fonction retourne une copie de l'objet `series`
#' modifié avec les valeurs de `replacement` imputés à partir de la date
#' `date_ts`.
#'
#' @export
#'
#' @examples
#' set_value_ts(
#'     series = ev_pib,
#'     date_ts = c(2021L, 2L),
#'     replacement = c(1, 2, 3)
#' )
#'
set_value_ts <- function(series, date_ts, replacement) {
    coll <- checkmate::makeAssertCollection()

    # Check de l'objet series
    assert_ts(series, add = coll, .var.name = "series")

    # Check de l'objet replacement un vecteur atomic
    checkmate::assert_atomic_vector(
        replacement,
        add = coll,
        .var.name = "replacement"
    )

    check_1 <- checkmate::check_atomic_vector(x = replacement)
    check_2 <- checkmate::anyMissing(x = replacement)
    if (isTRUE(check_1) && isTRUE(check_2)) {
        warning(
            "the argument replacement ",
            checkmate::check_atomic_vector(
                x = replacement,
                any.missing = FALSE
            ),
            call. = FALSE
        )
    }

    # Check des types des objets
    if (!isTRUE(typeof(series) == typeof(replacement))) {
        error_message <- paste(
            "Les objets `series` et `replacement` doivent",
            "\u00eatre de m\u00eame type."
        )
        coll$push(msg = error_message)
    }

    checkmate::reportAssertions(coll)

    frequency_ts <- as.integer(stats::frequency(series))

    # Check du format date_ts
    date_ts <- assert_date_ts(x = date_ts, frequency_ts, .var.name = "date_ts")

    ts_output <- series

    if (is.raw(ts_output)) {
        ts_output <- set_value_ts(
            series = stats::ts(
                data = as.integer(ts_output),
                start = stats::start(ts_output),
                frequency = stats::frequency(ts_output)
            ),
            date_ts = date_ts,
            replacement = as.integer(replacement)
        )
        ts_output <- stats::ts(
            data = as.raw(ts_output),
            start = stats::start(ts_output),
            frequency = stats::frequency(ts_output)
        )
    } else {
        start_ts <- normalize_date_ts(
            date_ts = date_ts,
            frequency_ts = as.integer(stats::frequency(series))
        )
        end_ts <- next_date_ts(
            date_ts,
            frequency_ts = as.integer(stats::frequency(series)),
            lag = length(replacement) - 1L
        )
        stats::window(
            x = ts_output,
            start = start_ts,
            end = end_ts,
            extend = TRUE
        ) <- replacement
    }

    return(ts_output)
}

#' @title Combiner 2 ts
#'
#' @description
#' La fonction `combine2ts` combine (comme c()) 2 time series de
#' même fréquence (mensuelle ou trimestrielle).
#'
#' @param a un objet ts unidimensionnel conforme aux règles de assert_ts
#' @param b un objet ts unidimensionnel conforme aux règles de assert_ts
#'
#' @returns En sortie, la fonction retourne un ts qui contient les valeurs de
#' `a` aux temps de `a` et les valeurs de `b` aux temps de `b`.
#' @details Si `a` et `b` ont une période en commun, les valeurs de `b` écrasent
#' celles de a sur la période concernée.
#' Si il existe une période sur laquelle ni `a` ni `b` ne prennent de valeur
#' (mais qu'il existe des valeurs à des dates ultérieures et antérieures) alors
#' le ts en sortie prendra NA sur cette période.
#' @export
#'
#' @examples
#'
#' trim_1 <- stats::ts(rep(1, 4), start = 2021, frequency = 4L)
#'
#' mens_1 <- stats::ts(rep(1, 4), start = 2020, frequency = 12L)
#' mens_2 <- stats::ts(rep(2, 4), start = 2022, frequency = 12L)
#'
#' # La série de PIB est écrasé par trim_1 sur la période temporelle de trim_1
#' combine2ts(ev_pib, trim_1)
#'
#' # La période entre les séries temporelles mens_1 et mens_2 est complétée par
#' # des NA
#' combine2ts(mens_1, mens_2)
#'
combine2ts <- function(a, b) {
    coll <- checkmate::makeAssertCollection()

    # Check de l'objet a
    assert_ts(a, add = coll, .var.name = "a")
    # Check de l'objet b
    assert_ts(b, add = coll, .var.name = "b")

    if (isTRUE(check_ts(a)) && isTRUE(check_ts(b))) {
        # Check same frequency_ts
        if (!isTRUE(stats::frequency(a) == stats::frequency(b))) {
            error_message <- paste(
                "Les objets `a` et `b` doivent",
                "avoir la m\u00eame fr\u00e9quence."
            )
            coll$push(msg = error_message)
        }

        # Check des types des objets
        if (!isTRUE(typeof(a) == typeof(b))) {
            error_message <- paste(
                "Les objets `a` et `b` doivent",
                "\u00eatre de m\u00eame type."
            )
            coll$push(msg = error_message)
        }
    }

    checkmate::reportAssertions(coll)

    frequency_ts <- stats::frequency(a)
    ts_output <- a

    if (is.raw(a)) {
        a <- stats::ts(
            data = as.integer(a),
            start = stats::start(a),
            frequency = frequency_ts
        )
        b <- stats::ts(
            data = as.integer(b),
            start = stats::start(b),
            frequency = frequency_ts
        )

        ts_output <- combine2ts(a, b)

        ts_output <- stats::ts(
            data = as.raw(ts_output),
            start = stats::start(ts_output),
            frequency = frequency_ts
        )

        # Fréquence entière
    } else if (isTRUE(checkmate::check_int(frequency_ts))) {
        stats::window(
            x = ts_output,
            start = stats::start(b),
            end = stats::end(b),
            extend = TRUE
        ) <- b

        # Fréquence décimale
    } else if (isTRUE(checkmate::check_number(frequency_ts))) {
        df_output <- as.data.frame(cbind(a, b))
        if (any(is.na(df_output$a) & (!is.na(df_output$b)))) {
            warning(
                "extending time series when replacing values",
                call. = FALSE
            )
        }
        df_output$res <- df_output$a
        df_output$res[!is.na(df_output$b)] <- df_output$b[!is.na(df_output$b)]

        ts_output <- stats::ts(
            data = df_output$res,
            frequency = frequency_ts,
            start = min(
                date_ts2timeunits(
                    as.integer(stats::start(a)),
                    frequency_ts = frequency_ts
                ),
                date_ts2timeunits(
                    as.integer(stats::start(b)),
                    frequency_ts = frequency_ts
                )
            )
        )
    }
    return(ts_output)
}

#' @title Ajoute de nouvelles valeurs à un ts
#'
#' @description
#' La fonction `extend_ts` ajoute de nouvelles valeurs à un ts.
#'
#' @inheritParams set_value_ts
#' @param replace_na un booléen.
#' @param date_ts_to un vecteur numérique, de préférence `integer`, au format
#' date_ts, c'est-à-dire `AAAA`, `c(AAAA, MM)` ou `c(AAAA, TT)`.
#' @param times un entier qui précise le nombre de fois où \code{replacement}
#' doit être répété, le vecteur entier.
#' @param each un entier qui précise le nombre de fois où \code{replacement}
#' doit être répété mais élément par élément.
#'
#' @returns En sortie, la fonction retourne une copie de l'objet `series`
#' complété avec le vecteur `replacement`.
#'
#' @details
#' \code{date_ts_to} désigne la date jusqu'à laquelle le remplacement
#' s'effectue. Par défault, cette valeur vaut \code{NULL}.
#'
#' Si `replace_na` vaut `TRUE` alors le remplacement commence dès que
#' l'objet ne contient que des NA. Dans le cas contraire, le ts est étendu,
#' qu'il contienne des NA ou non à la fin.
#' Si le vecteur `replacement` est de taille un sous-multiple de la différence
#' de période entre la date de fin de `series` et `date_ts_to`, le vecteur
#' `replacement` est répété jusqu'à la date `date_ts_to`. Sinon une erreur est
#' générée.
#'
#' Les arguments \code{times} et \code{each} en sont utilisé que si
#' \code{date_ts} est manquant (non fourni par l'utilisateur). Si tel est le
#' cas, ils se comporte comme si \code{replacement} devenait
#' \code{rep(replacement, times = times, each = each)}.
#'
#' @export
#'
#' @examples
#'
#' ts1 <- ts(
#'     data = c(rep(NA_integer_, 3L), 1L:10L, rep(NA_integer_, 3L)),
#'     start = 2020,
#'     frequency = 12
#' )
#' x <- rep(3L, 2L)
#'
#' extend_ts(series = ts1, replacement = x)
#' extend_ts(series = ts1, replacement = x, replace_na = FALSE)
#' extend_ts(series = ts1, replacement = x,
#'           date_ts_to = c(2021L, 7L), replace_na = TRUE)
#'
extend_ts <- function(
    series,
    replacement,
    date_ts_to = NULL,
    replace_na = TRUE,
    times = 1L,
    each = 1L
) {
    coll <- checkmate::makeAssertCollection()

    # Check de l'objet series
    assert_ts(series, add = coll, .var.name = "series")
    # Check de l'objet replacement un vecteur atomic
    checkmate::assert_atomic_vector(
        replacement,
        add = coll,
        .var.name = "replacement"
    )
    # Check de replace_na
    checkmate::assert_flag(replace_na, add = coll, .var.name = "replace_na")

    if (!isTRUE(typeof(series) == typeof(replacement))) {
        error_message <- paste(
            "Les objets `series` et `replacement` doivent",
            "\u00eatre de m\u00eame type."
        )
        coll$push(msg = error_message)
    }

    checkmate::reportAssertions(coll)

    frequency_ts <- as.integer(stats::frequency(series))

    # Check du format date_ts_to
    if (!is.null(date_ts_to)) {
        date_ts_to <- assert_date_ts(
            x = date_ts_to,
            frequency_ts,
            add = coll,
            .var.name = "date_ts_to"
        )
    } else {
        replacement <- rep(replacement, times = times, each = each)
    }

    if (is.raw(series)) {
        new_series <- stats::ts(
            data = as.integer(series),
            start = stats::start(series),
            frequency = frequency_ts
        )
        new_replacement <- as.integer(replacement)
        ts_output <- extend_ts(
            series = new_series,
            replacement = new_replacement,
            date_ts_to = date_ts_to,
            replace_na = replace_na
        )

        ts_output <- stats::ts(
            data = as.raw(ts_output),
            start = stats::start(ts_output),
            frequency = frequency_ts
        )

        return(ts_output)
    }

    start_replacement <- next_date_ts(
        date_ts = ifelse(
            test = replace_na,
            yes = last_date(series),
            no = as.integer(stats::end(series))
        ),
        frequency_ts = frequency_ts
    )

    if (!is.null(date_ts_to)) {
        if (
            !is_before(
                start_replacement,
                date_ts_to,
                frequency_ts = frequency_ts
            )
        ) {
            stop(
                c(
                    "La date de fin de remplacement est",
                    " ant\u00e9rieur \u00e0 la date de fin des donn\u00e9es."
                ),
                call. = FALSE
            )
        }
        length_replacement <- diff_periode(
            a = start_replacement,
            b = date_ts_to,
            frequency_ts = frequency_ts
        )
        if (length_replacement %% length(replacement) != 0L) {
            stop(
                c(
                    "number of values supplied is not a",
                    " sub-multiple of the number of values to be replaced"
                ),
                call. = FALSE
            )
        }
        end_replacement <- date_ts_to
    } else {
        end_replacement <- next_date_ts(
            date_ts = start_replacement,
            lag = length(replacement) - 1L,
            frequency_ts = frequency_ts
        )
    }

    stats::window(
        series,
        start = start_replacement,
        end = end_replacement,
        extend = TRUE
    ) <- replacement
    return(series)
}

#' @title Supprime les NA aux bords
#'
#' @description
#' La fonction `na_trim` supprime les NA en début et en fin de
#' période.
#'
#' @inheritParams first_date
#' @param sides une chaine de caractere qui spécifie quelle NA doivent être
#' retirés (au début et à la fin ("both"), juste au début ("left") ou juste à
#' la fin ("right"))
#'
#' @returns En sortie, la fonction retourne une copie de l'objet `series`
#' corrigée des NA et début et fin de série.
#' @details L'objet retourné commence et finis par des valeurs non manquantes.
#' @export
#'
#' @examples
#'
#' ts1 <- ts(c(rep(NA, 3L), 1:10, rep(NA, 3L)), start = 2020, frequency = 12L)
#' ts2 <- ts(c(1:10, rep(NA, 3L)), start = c(2023, 2), frequency = 4L)
#' ts3 <- ts(c(rep(NA, 3L), 1:10), start = 2000, frequency = 12L)
#'
#' na_trim(ts1)
#' na_trim(ts2)
#' na_trim(ts3)
#'
na_trim <- function(series, sides = c("both", "left", "right")) {
    coll <- NULL

    # Check de l'objet series
    assert_ts(series, add = coll, .var.name = "series")
    # Check du contenu (pas que des NA)
    checkmate::assert_atomic_vector(
        series,
        all.missing = FALSE,
        add = coll,
        .var.name = "series"
    )
    # Check de l'argument sides
    sides <- match.arg(sides)

    non_na <- seq_along(series)[!is.na(series)]

    frequency_ts <- assert_frequency(
        x = stats::frequency(series),
        add = coll,
        warn = FALSE
    )
    start_ts <- normalize_date_ts(
        date_ts = stats::start(series),
        frequency_ts = frequency_ts,
        test = FALSE
    )
    content <- switch(
        EXPR = sides,
        both = series[min(non_na):max(non_na)],
        right = series[1L:max(non_na)],
        left = series[min(non_na):length(series)]
    )
    if (sides != "right") {
        start_ts <- next_date_ts(
            date_ts = start_ts,
            frequency_ts = frequency_ts,
            lag = min(non_na) - 1L
        )
    }
    return(stats::ts(
        data = content,
        start = start_ts,
        frequency = frequency_ts
    ))
}

#' @title Libelé pour une date
#'
#' @description La fonction `libelles_one_date` créé le libellé pour une date
#' à une fréquence donnée.modifie la ou les valeurs d'un objet ts à une date
#' donnée.
#'
#' @inheritParams date_ts2timeunits
#' @param warn un booleen
#'
#' @returns En sortie, la fonction retourne une chaîne de caractère de longueur
#' 1 qui correspond au libellé de la date `date_ts`.
#'
#' @details
#' Si l'argument `warn` est `FALSE`, alors la fonction ne retournera pas de
#' warning lors de l'évaluation.
#'
#' @keywords internal
#' @noRd
#'
#' @examples
#'
#' libelles_one_date(2020L, 12L)
#' libelles_one_date(c(2020L, 3L), 4L)
#'
libelles_one_date <- function(date_ts, frequency_ts, warn = TRUE) {
    # Check de warn
    checkmate::assert_flag(
        warn,
        add = NULL,
        .var.name = "warn",
        na.ok = FALSE,
        null.ok = FALSE
    )

    # Check de la fréquence
    frequency_ts <- assert_frequency(
        frequency_ts,
        .var.name = "frequency_ts",
        warn = warn
    )

    # Check du format date_ts
    date_ts <- assert_date_ts(
        x = date_ts,
        frequency_ts,
        .var.name = "date_ts",
        warn = warn
    )

    date_obj <- date_ts2date(date_ts, frequency_ts = frequency_ts)

    year <- date_ts[1L]
    if (frequency_ts == 4L) {
        return(paste(quarters(date_obj), year))
    } else if (frequency_ts == 12L) {
        return(paste(months(date_obj, abbreviate = TRUE), year))
    }
}

#' @title Libelés pour une période
#'
#' @description La fonction `libelles` créé un vecteur de chaines de caractère
#' contenant les libelés de toutes les dates sur une période
#'
#' @inheritParams date_ts2timeunits
#' @param n un entier
#' @param warn un booleen
#'
#' @returns En sortie, la fonction retourne un vecteur de chaine de caractère de
#' longueur `n` avec les libellés de la période (de la date `date_ts` à la date
#' `date_ts + n périodes`.
#'
#' @details Pour choisir la période, il faut spécifier une date de début
#' `date_ts`, une fréquence `frequency_ts` pour le pas entre 2 dates
#' (trimestrielle ou mensuelle) et un nombre de valeur `n` (nombre de période).
#'
#' Si l'argument `warn` est `FALSE`, alors la fonction ne retournera pas de
#' warning lors de l'évaluation.
#'
#' @export
#'
#' @examples
#' libelles(date_ts = c(2019L, 10L), frequency_ts = 12L, n = 9L)
#' libelles(date_ts = c(2019L, 4L), frequency_ts = 4L, n = 3L)
#'
libelles <- function(date_ts, frequency_ts, n = 1L, warn = TRUE) {
    # Check de warn
    checkmate::assert_flag(
        warn,
        add = NULL,
        .var.name = "warn",
        na.ok = FALSE,
        null.ok = FALSE
    )

    # Check de la fréquence
    frequency_ts <- assert_frequency(
        frequency_ts,
        add = NULL,
        .var.name = "frequency_ts",
        warn = warn
    )

    # Check du format date_ts
    date_ts <- assert_date_ts(
        x = date_ts,
        frequency_ts,
        add = NULL,
        .var.name = "date_ts",
        warn = warn
    )

    # Check de l'argument n
    n <- assert_scalar_natural(n, add = NULL, .var.name = "n")

    decale_libele <- function(x) {
        date_temp <- next_date_ts(
            date_ts = date_ts,
            frequency_ts = frequency_ts,
            lag = x
        )
        return(libelles_one_date(
            date_ts = date_temp,
            frequency_ts = frequency_ts,
            warn = FALSE
        ))
    }

    return(vapply(
        X = seq_len(n) - 1L,
        FUN = decale_libele,
        FUN.VALUE = character(1L)
    ))
}

testthat::test_that("good result for integer date", {
    testthat::expect_identical(
        object = libelles_one_date(date_ts = c(2020L, 2L), frequency_ts = 12L),
        expected = "Feb 2020"
    )
})

#' @title Évolution du PIB français jusqu'au T1 2022
#'
#' @description
#' Ce jeu de données contient une série `ts` de l'évolution trimestrielle du
#' produit intérieur brut français.
#' Toutes les infos complémentaires sur cette série se trouve sur la page de la
#' \href{https://www.insee.fr/fr/statistiques/2830547}{publication} sur le site
#' de l'\href{https://www.insee.fr/fr/accueil}{Insee}.
#'
#' @format Un ts unidimensionnel :
#' \describe{
#' \item{start}{le ts commence au T1 1970 mais la série de PIB ne commence qu'au
#' T2 1980.}
#' \item{end}{le ts finit au T3 2022 mais la série de PIB finit au T1 2022.}
#' \item{frequency_ts}{la fréquence est trimestrielle}
#' }
#' @source \url{https://www.insee.fr/fr/statistiques/2830547}
"ev_pib"

# Options ----------------------------------------------------------------------

withr::local_locale(.new = c(LC_TIME = "en_US"))
withr::with_envvar(new = c(lang = "en_US"), {
    # Création de fonctions --------------------------------------------------------

    create_random_type <- function(type = list_type, len = NULL) {
        type <- match.arg(type)
        if (is.null(len)) {
            len <- sample.int(n = 1000L, size = 1L)
        }

        output <- switch(
            EXPR = type,
            character = strsplit(
                x = intToUtf8(sample(
                    c(1L:55295L, 57344L:1114111L),
                    size = len,
                    replace = TRUE
                )),
                split = "",
                fixed = TRUE
            )[[1L]],
            integer = sample(-20000000L:20000000L, size = len, replace = TRUE),
            double = runif(n = len, min = -10000L, max = 10000L),
            logical = sample(x = c(TRUE, FALSE), size = len, replace = TRUE),
            complex = complex(
                real = create_random_type(type = "double", len = len),
                imaginary = create_random_type(type = "double", len = len)
            ),
            raw = as.raw(sample.int(n = 255L, size = len, replace = TRUE)),
            Date = as.Date(
                x = sample.int(n = 27000L, size = len, replace = TRUE) - 7000L,
                origin = "1970-01-01"
            )
        )
        return(output)
    }

    create_random_date_ts <- function(frequency_ts = NULL) {
        if (runif(n = 1L, min = 0L, max = 1L) > 0.5) {
            return(sample(1950L:2022L, size = 1L))
        }

        if (!is.null(frequency_ts)) {
            return(c(
                1950L + sample.int(n = 80L, size = 1L),
                sample.int(n = frequency_ts, size = 1L)
            ))
        }

        return(c(
            sample(1950L:2022L, size = 1L),
            sample(-20L:20L, size = 1L)
        ))
    }

    create_random_ts <- function(
        type = NULL,
        len = NULL,
        start = NULL,
        frequency = NULL
    ) {
        if (is.null(type)) {
            type <- sample(x = list_type, size = 1L)
        }
        if (is.null(len)) {
            len <- sample.int(n = 1000L, size = 1L)
        }
        if (is.null(frequency)) {
            frequency <- sample(c(4L, 12L), size = 1L)
        }
        if (is.null(start)) {
            start <- create_random_date_ts()
        }

        content <- create_random_type(type, len)

        return(ts(content, start = start, frequency = frequency))
    }

    create_NA_type <- function(type = list_type, len = 1L) {
        type <- match.arg(type)
        output <- rep(
            x = switch(
                EXPR = type,
                character = NA_character_,
                integer = NA_integer_,
                double = NA_real_,
                logical = NA,
                complex = complex(real = NA, imaginary = 0), # NA_complex_,
                raw = as.raw(0x00),
                Date = as.Date(NA_integer_)
            ),
            times = len
        )
        return(output)
    }

    # Variables globales de test ---------------------------------------------------

    ## Types d'objets --------------------------------------------------------------

    list_type <- c(
        "integer",
        "character",
        "double",
        "logical",
        "complex",
        "raw",
        "Date"
    )

    object_bank_R <- fuzzr::test_all()

    wrong_type_ts <- list(
        ts(list(2L), start = 2010L, frequency = 12L),
        ts(list(2, 3, c(1, 2)), start = 2010L, frequency = 12L),
        ts(list(2L, 3L, 4L), start = 2010L, frequency = 12L),
        ts(list(2L, list("3L"), 4L:15L), start = 2010L, frequency = 12L)
    )

    ## Fréquences ------------------------------------------------------------------

    list_frequence <- c(4L, 12L)
    weird_frequency <- list(1L, 2, 7, 0.1, 1 / 3, 3.5, 365, 365.25, pi)

    ## Dates -----------------------------------------------------------------------

    # Time Units
    list_wrong_timeunits <- list(2020 + 1 / 7, pi, 2020 - 1 / 13, Inf)

    # Années
    good_years <- c(
        -200L,
        -1L,
        0L,
        1L,
        2L,
        1950L,
        1999L,
        2000L,
        2001L,
        2022L,
        3000L
    )
    double_years <- c(-200., -1., 0., 1., 2., 1950., 2000., 2022., 3000.)

    # Month
    warning_double_months <- c(-200., -20., -5., -1., 0., 13., 46., 200.)
    warning_integer_months <- c(
        -200L,
        -20L,
        -12L,
        -5L:0L,
        13L:15L,
        24L,
        46L,
        200L
    )

    double_months <- c(1., 2., 3., 4.0, 5., 6., 7., 8., 9., 10., 11., 12.0)
    good_months <- 1L:12L

    # Quarter
    warning_double_quarters <- c(
        -200.,
        -20.,
        -5.,
        -3.,
        -2.,
        -1.,
        0.,
        5.,
        12.0,
        13.,
        46.
    )
    warning_integer_quarters <- c(
        -200L,
        -20L,
        -5L,
        -3L,
        -2L,
        -1L,
        0L,
        5L,
        12L,
        13L,
        46L
    )

    double_quarters <- c(1., 2., 3., 4.0)
    good_quarters <- 1L:4L

    # Dates

    list_start <- list(
        c(2020L, -1L),
        c(2020L, 0L),
        c(2020L, 4L),
        c(2020L, 5L),
        c(2020L, 12L),
        c(2020L, 13L),
        2019L
    )

    list_wrong_date_ts <- c(
        fuzzr::test_all()[-10L],
        list(
            list(2020L, 5L),
            list(2L, "a", 3.5),
            list(NULL),
            list(2005),
            list(c(2022L, 8L)),
            list(c(2022L, 8.))
        ),
        lapply(list_type[-c(1L, 3L)], create_random_type, len = 2L),
        lapply(list_type, create_random_type, len = 3),
        list(
            2019.5,
            2020. + 1. / 12.0,
            pi / 4.0,
            c(2020., 2.5),
            c(2010.25, 3.),
            c(2002., 3., 1.),
            c("2002", "3")
        ),
        list(
            c(2020L, NA_integer_),
            c(NA_integer_, 5L),
            c(NA_integer_, NA_integer_),
            c(2020, NA_real_),
            c(NA_real_, 5.),
            c(NA_real_, NA_real_)
        ),
        list(
            2L:4L,
            c(2020.0, 7., 1.),
            c(2020L, 0L, NA_integer_),
            numeric(0),
            integer(0),
            Inf,
            c(2000L, Inf),
            c(Inf, 4.0)
        ),
        rnorm(10L)
    )

    ## Conversions et labels -------------------------------------------------------

    # Conversion
    conversion_quarter_month <- data.frame(
        quarter = 1L:4L,
        month = c(1L, 4L, 7L, 10L)
    )
    conversion_month_quarter <- data.frame(
        month = 1L:12L,
        quarter = rep(1L:4L, each = 3L)
    )

    # Labels
    # list_months_name <- c("janv.", "f\u00e9vr.", "mars", "avr.", "mai", "juin", "juil.", "ao\u00fbt", "sept.", "oct.", "nov.", "d\u00e9c.")
    list_months_name <- c(
        "Jan",
        "Feb",
        "Mar",
        "Apr",
        "May",
        "Jun",
        "Jul",
        "Aug",
        "Sep",
        "Oct",
        "Nov",
        "Dec"
    )

    # Autres objets ----------------------------------------------------------------

    list_len <- c(0L:5L, 10L, 100L, 10000L)
    list_lag <- c(-1000L, -5L, -2L:2L, 5L, 1000L)

    # Error / warning messages -----------------------------------------------------

    message_double <- function(var) {
        return(paste0(
            "Assertion on '",
            var,
            "' failed: Must be of type 'integer', not 'double'."
        ))
    }

    invalid_monthly_period <- "Assertion on 'period' failed: Element 1 is not >= 1.|Assertion on 'period' failed: Element 1 is not <= 12."
    invalid_quaterly_period <- "Assertion on 'period' failed: Element 1 is not >= 1.|Assertion on 'period' failed: Element 1 is not <= 4."

    double_instead_of_integer <- "Must be of type 'integer', not 'double'."
    warning_extend <- "extending time series when replacing values"
})

# Initialisation ---------------------------------------------------------------

set.seed(2034L)


# Tests de résultats positifs --------------------------------------------------

testthat::test_that("good result for integer date month", {
    for (year in good_years) {
        res <- testthat::expect_silent(assert_timeunits(
            x = year,
            frequency_ts = 12L
        ))
        testthat::expect_identical(res, year)
        for (month in good_months) {
            good_timeunits <- year + (month - 1) / 12
            res <- testthat::expect_silent(assert_timeunits(
                x = good_timeunits,
                frequency_ts = 12L
            ))
            testthat::expect_identical(res, good_timeunits)
        }
    }
})

testthat::test_that("good result for integer date quarter", {
    for (year in good_years) {
        res <- testthat::expect_silent(assert_timeunits(
            x = year,
            frequency_ts = 4L
        ))
        testthat::expect_identical(res, year)
        for (quarter in good_quarters) {
            good_timeunits <- year + (quarter - 1) / 4
            res <- testthat::expect_silent(assert_timeunits(
                x = good_timeunits,
                frequency_ts = 4L
            ))
            testthat::expect_identical(res, good_timeunits)
        }
    }
})


# Tests positifs avec warning --------------------------------------------------

## Fréquence -------------------------------------------------------------------

testthat::test_that("warning for integer date", {
    for (year in good_years) {
        for (month in good_months) {
            good_timeunits <- year + (month - 1L) / 12L
            testthat::expect_warning(
                {
                    res <- assert_timeunits(
                        x = good_timeunits,
                        frequency_ts = 12.0
                    )
                },
                regexp = message_double("frequency_ts")
            )
            testthat::expect_identical(res, good_timeunits)
        }
    }

    for (year in good_years) {
        for (quarter in good_quarters) {
            good_timeunits <- year + (quarter - 1L) / 4L
            testthat::expect_warning(
                {
                    res <- assert_timeunits(
                        x = good_timeunits,
                        frequency_ts = 4.0
                    )
                },
                regexp = message_double("frequency_ts")
            )
            testthat::expect_identical(res, good_timeunits)
        }
    }
})


# Tests de résultats négatifs --------------------------------------------------

testthat::test_that("detection of wrong object", {
    for (wrong_timeunits in c(
        object_bank_R[-c(10L, 16L)],
        list_wrong_timeunits
    )) {
        testthat::expect_error(assert_timeunits(
            x = wrong_timeunits,
            frequency_ts = 12L
        ))
        testthat::expect_error(assert_timeunits(
            x = wrong_timeunits,
            frequency_ts = 4L
        ))
    }
})


testthat::test_that("detection of wrong timesunits in quarterly", {
    for (year in good_years) {
        for (month in setdiff(good_months, (1:4) * 3) + 1) {
            wrong_timeunits <- year + (month - 1) / 12
            testthat::expect_error(assert_timeunits(
                x = wrong_timeunits,
                frequency_ts = 4L
            ))
        }
    }
})

testthat::test_that("miscellaneous frequency are not allowed", {
    for (wrong_frequency in c(object_bank_R, weird_frequency)) {
        testthat::expect_error(
            assert_timeunits(
                x = create_random_date_ts(),
                frequency_ts = wrong_frequency
            )
        )
    }
})

# Initialisation ---------------------------------------------------------------

set.seed(2026L)


# Tests de résultats positifs --------------------------------------------------

testthat::test_that("good result for monthly date", {
    for (month in good_months) {
        for (year in good_years) {
            for (len in list_len[c(-1L, -9L)]) {
                testthat::expect_identical(
                    libelles(
                        date = c(year, month),
                        frequency_ts = 12L,
                        n = len
                    ),
                    paste(
                        list_months_name[
                            ((month:(month + len - 1L)) - 1L) %% 12L + 1L
                        ],
                        year + ((month:(month + len - 1L)) - 1L) %/% 12L
                    )
                )
            }
        }
    }
})

testthat::test_that("good result for quarter date", {
    for (quarter in good_quarters) {
        for (year in good_years) {
            for (len in list_len[c(-1L, -9L)]) {
                testthat::expect_identical(
                    libelles(
                        date = c(year, quarter),
                        frequency_ts = 4L,
                        n = len
                    ),
                    paste0(
                        "Q",
                        ((quarter:(quarter + len - 1L)) - 1L) %% 4L + 1L,
                        " ",
                        year + ((quarter:(quarter + len - 1L)) - 1L) %/% 4L
                    )
                )
            }
        }
    }
})


# Tests de résultats positifs avec warning -------------------------------------

testthat::test_that("warning result for monthly date", {
    for (month in warning_integer_months) {
        for (year in good_years) {
            for (len in list_len[c(-1L, -9L)]) {
                testthat::expect_warning(
                    testthat::expect_identical(
                        libelles(
                            date = c(year, month),
                            frequency_ts = 12L,
                            n = len,
                            warn = TRUE
                        ),
                        paste(
                            list_months_name[
                                ((month:(month + len - 1L)) - 1L) %% 12L + 1L
                            ],
                            year + ((month:(month + len - 1L)) - 1L) %/% 12L
                        )
                    )
                )
                testthat::expect_warning(
                    testthat::expect_identical(
                        libelles(
                            date = c(year, month),
                            frequency_ts = 12L,
                            n = len
                        ),
                        paste(
                            list_months_name[
                                ((month:(month + len - 1L)) - 1L) %% 12L + 1L
                            ],
                            year + ((month:(month + len - 1L)) - 1L) %/% 12L
                        )
                    )
                )
                testthat::expect_no_warning(
                    testthat::expect_identical(
                        libelles(
                            date = c(year, month),
                            frequency_ts = 12L,
                            n = len,
                            warn = FALSE
                        ),
                        paste(
                            list_months_name[
                                ((month:(month + len - 1L)) - 1L) %% 12L + 1L
                            ],
                            year + ((month:(month + len - 1L)) - 1L) %/% 12L
                        )
                    )
                )
            }
        }
    }
})

testthat::test_that("warning result for quarter date", {
    for (quarter in warning_integer_quarters) {
        for (year in good_years) {
            for (len in list_len[c(-1L, -9L)]) {
                testthat::expect_warning(
                    testthat::expect_identical(
                        libelles(
                            date = c(year, quarter),
                            frequency_ts = 4L,
                            n = len,
                            warn = TRUE
                        ),
                        paste0(
                            "Q",
                            ((quarter:(quarter + len - 1L)) - 1L) %% 4L + 1L,
                            " ",
                            year + ((quarter:(quarter + len - 1L)) - 1L) %/% 4L
                        )
                    )
                )
                testthat::expect_warning(
                    testthat::expect_identical(
                        libelles(
                            date = c(year, quarter),
                            frequency_ts = 4L,
                            n = len
                        ),
                        paste0(
                            "Q",
                            ((quarter:(quarter + len - 1L)) - 1L) %% 4L + 1L,
                            " ",
                            year + ((quarter:(quarter + len - 1L)) - 1L) %/% 4L
                        )
                    )
                )
                testthat::expect_no_warning(
                    testthat::expect_identical(
                        libelles(
                            date = c(year, quarter),
                            frequency_ts = 4L,
                            n = len,
                            warn = FALSE
                        ),
                        paste0(
                            "Q",
                            ((quarter:(quarter + len - 1L)) - 1L) %% 4L + 1L,
                            " ",
                            year + ((quarter:(quarter + len - 1L)) - 1L) %/% 4L
                        )
                    )
                )
            }
        }
    }
})


testthat::test_that("warning result for monthly date double", {
    for (month in double_months) {
        for (year in good_years) {
            for (len in list_len[c(-1L, -9L)]) {
                testthat::expect_warning(
                    testthat::expect_identical(
                        libelles(
                            date = c(year, month),
                            frequency_ts = 12L,
                            n = len,
                            warn = TRUE
                        ),
                        paste(
                            list_months_name[
                                ((month:(month + len - 1L)) - 1L) %% 12L + 1L
                            ],
                            year + ((month:(month + len - 1L)) - 1L) %/% 12L
                        )
                    )
                )
                testthat::expect_warning(
                    testthat::expect_identical(
                        libelles(
                            date = c(year, month),
                            frequency_ts = 12L,
                            n = len
                        ),
                        paste(
                            list_months_name[
                                ((month:(month + len - 1L)) - 1L) %% 12L + 1L
                            ],
                            year + ((month:(month + len - 1L)) - 1L) %/% 12L
                        )
                    )
                )
                testthat::expect_no_warning(
                    testthat::expect_identical(
                        libelles(
                            date = c(year, month),
                            frequency_ts = 12L,
                            n = len,
                            warn = FALSE
                        ),
                        paste(
                            list_months_name[
                                ((month:(month + len - 1L)) - 1L) %% 12L + 1L
                            ],
                            year + ((month:(month + len - 1L)) - 1L) %/% 12L
                        )
                    )
                )
            }
        }
    }
    for (month in good_months) {
        for (year in double_years) {
            for (len in list_len[c(-1L, -9L)]) {
                testthat::expect_warning(
                    testthat::expect_identical(
                        libelles(
                            date = c(year, month),
                            frequency_ts = 12L,
                            n = len,
                            warn = TRUE
                        ),
                        paste(
                            list_months_name[
                                ((month:(month + len - 1L)) - 1L) %% 12L + 1L
                            ],
                            year + ((month:(month + len - 1L)) - 1L) %/% 12L
                        )
                    )
                )
                testthat::expect_warning(
                    testthat::expect_identical(
                        libelles(
                            date = c(year, month),
                            frequency_ts = 12L,
                            n = len
                        ),
                        paste(
                            list_months_name[
                                ((month:(month + len - 1L)) - 1L) %% 12L + 1L
                            ],
                            year + ((month:(month + len - 1L)) - 1L) %/% 12L
                        )
                    )
                )
                testthat::expect_no_warning(
                    testthat::expect_identical(
                        libelles(
                            date = c(year, month),
                            frequency_ts = 12L,
                            n = len,
                            warn = FALSE
                        ),
                        paste(
                            list_months_name[
                                ((month:(month + len - 1L)) - 1L) %% 12L + 1L
                            ],
                            year + ((month:(month + len - 1L)) - 1L) %/% 12L
                        )
                    )
                )
            }
        }
    }
})

testthat::test_that("warning result for quarter date double", {
    for (quarter in double_quarters) {
        for (year in good_years) {
            for (len in list_len[c(-1L, -9L)]) {
                testthat::expect_warning(
                    testthat::expect_identical(
                        libelles(
                            date = c(year, quarter),
                            frequency_ts = 4L,
                            n = len,
                            warn = TRUE
                        ),
                        paste0(
                            "Q",
                            ((quarter:(quarter + len - 1L)) - 1L) %% 4L + 1L,
                            " ",
                            year + ((quarter:(quarter + len - 1L)) - 1L) %/% 4L
                        )
                    )
                )
                testthat::expect_warning(
                    testthat::expect_identical(
                        libelles(
                            date = c(year, quarter),
                            frequency_ts = 4L,
                            n = len
                        ),
                        paste0(
                            "Q",
                            ((quarter:(quarter + len - 1L)) - 1L) %% 4L + 1L,
                            " ",
                            year + ((quarter:(quarter + len - 1L)) - 1L) %/% 4L
                        )
                    )
                )
                testthat::expect_no_warning(
                    testthat::expect_identical(
                        libelles(
                            date = c(year, quarter),
                            frequency_ts = 4L,
                            n = len,
                            warn = FALSE
                        ),
                        paste0(
                            "Q",
                            ((quarter:(quarter + len - 1L)) - 1L) %% 4L + 1L,
                            " ",
                            year + ((quarter:(quarter + len - 1L)) - 1L) %/% 4L
                        )
                    )
                )
            }
        }
    }
    for (quarter in good_quarters) {
        for (year in double_years) {
            for (len in list_len[c(-1L, -9L)]) {
                testthat::expect_warning(
                    testthat::expect_identical(
                        libelles(
                            date = c(year, quarter),
                            frequency_ts = 4L,
                            n = len,
                            warn = TRUE
                        ),
                        paste0(
                            "Q",
                            ((quarter:(quarter + len - 1L)) - 1L) %% 4L + 1L,
                            " ",
                            year + ((quarter:(quarter + len - 1L)) - 1L) %/% 4L
                        )
                    )
                )
                testthat::expect_warning(
                    testthat::expect_identical(
                        libelles(
                            date = c(year, quarter),
                            frequency_ts = 4L,
                            n = len
                        ),
                        paste0(
                            "Q",
                            ((quarter:(quarter + len - 1L)) - 1L) %% 4L + 1L,
                            " ",
                            year + ((quarter:(quarter + len - 1L)) - 1L) %/% 4L
                        )
                    )
                )
                testthat::expect_no_warning(
                    testthat::expect_identical(
                        libelles(
                            date = c(year, quarter),
                            frequency_ts = 4L,
                            n = len,
                            warn = FALSE
                        ),
                        paste0(
                            "Q",
                            ((quarter:(quarter + len - 1L)) - 1L) %% 4L + 1L,
                            " ",
                            year + ((quarter:(quarter + len - 1L)) - 1L) %/% 4L
                        )
                    )
                )
            }
        }
    }
})


# Tests de résultats négatifs --------------------------------------------------

testthat::test_that("miscellaneous date are not allowed", {
    for (wrong_date in list_wrong_date_ts) {
        testthat::expect_error(libelles(
            date = wrong_date,
            frequency_ts = 12L,
            warn = FALSE
        ))
        testthat::expect_error(libelles(
            date = wrong_date,
            frequency_ts = 4L,
            warn = FALSE
        ))
    }
})

testthat::test_that("miscellaneous frequency are not allowed", {
    for (wrong_frequency in c(object_bank_R, weird_frequency)) {
        testthat::expect_error(libelles(
            date = create_random_date_ts(),
            frequency_ts = wrong_frequency,
            warn = FALSE
        ))
    }
})

testthat::test_that("miscellaneous n are not allowed", {
    list_wrong_n <- c(
        list(0., 0L),
        list_wrong_date_ts,
        object_bank_R[-10L],
        rnorm(10L),
        as.double(
            -abs(c(
                list_lag,
                list_len,
                create_random_type("integer", len = 10L)
            ))
        ),
        -abs(c(list_len, list_lag, create_random_type("integer", len = 10L)))
    )

    for (wrong_n in list_wrong_n) {
        testthat::expect_error(libelles(
            date = create_random_date_ts(),
            frequency_ts = 12L,
            n = wrong_n,
            warn = FALSE
        ))
        testthat::expect_error(libelles(
            date = create_random_date_ts(),
            frequency_ts = 4L,
            n = wrong_n,
            warn = FALSE
        ))
    }
})

testthat::test_that("miscellaneous warn are not allowed", {
    for (wrong_warn in c(list(0., 0L), object_bank_R[-29L])) {
        testthat::expect_error(libelles(
            date = create_random_date_ts(),
            frequency_ts = 12L,
            n = 5L,
            warn = wrong_warn
        ))
        testthat::expect_error(libelles(
            date = create_random_date_ts(),
            frequency_ts = 4L,
            n = 5L,
            warn = wrong_warn
        ))
    }
})

# Initialisation ---------------------------------------------------------------

set.seed(2033L)


# Tests de résultats positifs --------------------------------------------------

testthat::test_that("good result", {
    for (year in good_years) {
        for (len in list_lag) {
            date_a <- year
            date_b <- year + len

            res1 <- diff_periode(date_a, date_b, frequency_ts = 12L)
            res2 <- diff_periode(date_b, date_a, frequency_ts = 12L)

            testthat::expect_type(res1, "integer")
            testthat::expect_type(res2, "integer")
            testthat::expect_identical(res1, abs(len) * 12L + 1L)
            testthat::expect_identical(res2, abs(len) * 12L + 1L)

            res1 <- diff_periode(date_a, date_b, frequency_ts = 4L)
            res2 <- diff_periode(date_b, date_a, frequency_ts = 4L)

            testthat::expect_type(res1, "integer")
            testthat::expect_type(res2, "integer")
            testthat::expect_identical(res1, abs(len) * 4L + 1L)
            testthat::expect_identical(res2, abs(len) * 4L + 1L)

            for (month in good_months) {
                date_a <- c(year, month)
                date_b <- c(
                    year + (month + len - 1L) %/% 12L,
                    (month + len - 1L) %% 12L + 1L
                )

                res1 <- diff_periode(date_a, date_b, frequency_ts = 12L)
                res2 <- diff_periode(date_b, date_a, frequency_ts = 12L)

                testthat::expect_type(res1, "integer")
                testthat::expect_type(res2, "integer")
                testthat::expect_identical(res1, abs(len) + 1L)
                testthat::expect_identical(res2, abs(len) + 1L)
            }

            for (quarter in good_quarters) {
                date_a <- c(year, quarter)
                date_b <- c(
                    year + (quarter + len - 1L) %/% 4L,
                    (quarter + len - 1L) %% 4L + 1L
                )

                res1 <- diff_periode(date_a, date_b, frequency_ts = 4L)
                res2 <- diff_periode(date_b, date_a, frequency_ts = 4L)

                testthat::expect_type(res1, "integer")
                testthat::expect_type(res2, "integer")
                testthat::expect_identical(res1, abs(len) + 1L)
                testthat::expect_identical(res2, abs(len) + 1L)
            }
        }
    }
})


# Tests de résultats positif avec warning --------------------------------------

testthat::test_that("warning with double years", {
    for (year in double_years) {
        for (len in list_lag) {
            date_a <- year
            date_b <- year + len

            w <- testthat::capture_warnings({
                res1 <- diff_periode(date_a, date_b, frequency_ts = 12L)
            })
            testthat::expect_match(
                object = w,
                regexp = message_double("a"),
                all = FALSE
            )
            testthat::expect_match(
                object = w,
                regexp = message_double("a"),
                all = FALSE
            )

            w <- testthat::capture_warnings({
                res2 <- diff_periode(date_b, date_a, frequency_ts = 12L)
            })
            testthat::expect_match(
                object = w,
                regexp = message_double("a"),
                all = FALSE
            )
            testthat::expect_match(
                object = w,
                regexp = message_double("a"),
                all = FALSE
            )

            testthat::expect_type(res1, "integer")
            testthat::expect_type(res2, "integer")
            testthat::expect_identical(res1, abs(len) * 12L + 1L)
            testthat::expect_identical(res2, abs(len) * 12L + 1L)

            w <- testthat::capture_warnings({
                res1 <- diff_periode(date_a, date_b, frequency_ts = 4L)
            })
            testthat::expect_match(
                object = w,
                regexp = message_double("a"),
                all = FALSE
            )
            testthat::expect_match(
                object = w,
                regexp = "Assertion on 'b' failed: Must be of type 'integer', not 'double'.",
                all = FALSE
            )

            w <- testthat::capture_warnings({
                res2 <- diff_periode(date_b, date_a, frequency_ts = 4L)
            })
            testthat::expect_match(
                object = w,
                regexp = message_double("a"),
                all = FALSE
            )
            testthat::expect_match(
                object = w,
                regexp = "Assertion on 'b' failed: Must be of type 'integer', not 'double'.",
                all = FALSE
            )

            testthat::expect_type(res1, "integer")
            testthat::expect_type(res2, "integer")
            testthat::expect_identical(res1, abs(len) * 4L + 1L)
            testthat::expect_identical(res2, abs(len) * 4L + 1L)

            for (month in c(
                warning_double_months,
                warning_integer_months,
                double_months
            )) {
                date_a <- c(year, month)
                date_b <- c(year, month + len)

                w <- testthat::capture_warnings({
                    res1 <- diff_periode(date_a, date_b, frequency_ts = 12L)
                })
                testthat::expect_match(
                    object = w,
                    regexp = message_double("a"),
                    all = FALSE
                )
                testthat::expect_match(
                    object = w,
                    regexp = message_double("a"),
                    all = FALSE
                )

                w <- testthat::capture_warnings({
                    res2 <- diff_periode(date_b, date_a, frequency_ts = 12L)
                })
                testthat::expect_match(
                    object = w,
                    regexp = message_double("a"),
                    all = FALSE
                )
                testthat::expect_match(
                    object = w,
                    regexp = message_double("a"),
                    all = FALSE
                )

                testthat::expect_type(res1, "integer")
                testthat::expect_type(res2, "integer")
                testthat::expect_identical(res1, abs(len) + 1L)
                testthat::expect_identical(res2, abs(len) + 1L)
            }

            for (quarter in c(
                warning_double_quarters,
                warning_integer_quarters,
                double_quarters
            )) {
                date_a <- c(year, quarter)
                date_b <- c(year, quarter + len)

                w <- testthat::capture_warnings({
                    res1 <- diff_periode(date_a, date_b, frequency_ts = 4L)
                })
                testthat::expect_match(
                    object = w,
                    regexp = message_double("a"),
                    all = FALSE
                )
                testthat::expect_match(
                    object = w,
                    regexp = "Assertion on 'b' failed: Must be of type 'integer', not 'double'.",
                    all = FALSE
                )

                w <- testthat::capture_warnings({
                    res2 <- diff_periode(date_b, date_a, frequency_ts = 4L)
                })
                testthat::expect_match(
                    object = w,
                    regexp = message_double("a"),
                    all = FALSE
                )
                testthat::expect_match(
                    object = w,
                    regexp = "Assertion on 'b' failed: Must be of type 'integer', not 'double'.",
                    all = FALSE
                )

                testthat::expect_type(res1, "integer")
                testthat::expect_type(res2, "integer")
                testthat::expect_identical(res1, abs(len) + 1L)
                testthat::expect_identical(res2, abs(len) + 1L)
            }
        }
    }
})

testthat::test_that("warning frequency", {
    for (year in good_years) {
        for (len in list_lag) {
            date_a <- year
            date_b <- year + len

            testthat::expect_warning(
                object = {
                    res1 <- diff_periode(date_a, date_b, frequency_ts = 12.0)
                },
                regexp = message_double("frequency_ts")
            )
            testthat::expect_warning(
                object = {
                    res2 <- diff_periode(date_b, date_a, frequency_ts = 12.0)
                },
                regexp = message_double("frequency_ts")
            )

            testthat::expect_type(res1, "integer")
            testthat::expect_type(res2, "integer")
            testthat::expect_identical(res1, abs(len) * 12L + 1L)
            testthat::expect_identical(res2, abs(len) * 12L + 1L)

            testthat::expect_warning(
                object = {
                    res1 <- diff_periode(date_a, date_b, frequency_ts = 4.0)
                },
                regexp = message_double("frequency_ts")
            )
            testthat::expect_warning(
                object = {
                    res2 <- diff_periode(date_b, date_a, frequency_ts = 4.0)
                },
                regexp = message_double("frequency_ts")
            )

            testthat::expect_type(res1, "integer")
            testthat::expect_type(res2, "integer")
            testthat::expect_identical(res1, abs(len) * 4L + 1L)
            testthat::expect_identical(res2, abs(len) * 4L + 1L)

            for (month in good_months) {
                date_a <- c(year, month)
                date_b <- c(
                    year + (month + len - 1L) %/% 12L,
                    (month + len - 1L) %% 12L + 1L
                )

                testthat::expect_warning(
                    object = {
                        res1 <- diff_periode(
                            date_a,
                            date_b,
                            frequency_ts = 12.0
                        )
                    },
                    regexp = message_double("frequency_ts")
                )
                testthat::expect_warning(
                    object = {
                        res2 <- diff_periode(
                            date_b,
                            date_a,
                            frequency_ts = 12.0
                        )
                    },
                    regexp = message_double("frequency_ts")
                )

                testthat::expect_type(res1, "integer")
                testthat::expect_type(res2, "integer")
                testthat::expect_identical(res1, abs(len) + 1L)
                testthat::expect_identical(res2, abs(len) + 1L)
            }

            for (quarter in good_quarters) {
                date_a <- c(year, quarter)
                date_b <- c(
                    year + (quarter + len - 1L) %/% 4L,
                    (quarter + len - 1L) %% 4L + 1L
                )

                testthat::expect_warning(
                    object = {
                        res1 <- diff_periode(date_a, date_b, frequency_ts = 4.0)
                    },
                    regexp = message_double("frequency_ts")
                )
                testthat::expect_warning(
                    object = {
                        res2 <- diff_periode(date_b, date_a, frequency_ts = 4.0)
                    },
                    regexp = message_double("frequency_ts")
                )

                testthat::expect_type(res1, "integer")
                testthat::expect_type(res2, "integer")
                testthat::expect_identical(res1, abs(len) + 1L)
                testthat::expect_identical(res2, abs(len) + 1L)
            }
        }
    }
})


# Tests de résultats négatifs (erreur) -----------------------------------------

testthat::test_that("miscellaneous date are not allowed", {
    for (wrong_date in list_wrong_date_ts) {
        date_a <- create_random_date_ts() |>
            normalize_date_ts(frequency_ts = 12L, test = FALSE)
        testthat::expect_error(diff_periode(
            a = wrong_date,
            b = date_a,
            frequency_ts = 12L
        ))
        testthat::expect_error(diff_periode(
            b = wrong_date,
            a = date_a,
            frequency_ts = 12L
        ))

        date_a <- create_random_date_ts() |>
            normalize_date_ts(frequency_ts = 4L, test = FALSE)
        date_b <- create_random_date_ts() |>
            normalize_date_ts(frequency_ts = 4L, test = FALSE)
        testthat::expect_error(diff_periode(
            a = wrong_date,
            b = date_a,
            frequency_ts = 4L
        ))
        testthat::expect_error(diff_periode(
            b = wrong_date,
            a = date_a,
            frequency_ts = 4L
        ))
    }
})

testthat::test_that("miscellaneous frequency are not allowed", {
    for (wrong_frequency in c(object_bank_R, weird_frequency)) {
        date_a <- create_random_date_ts() |>
            normalize_date_ts(frequency_ts = 12L, test = FALSE)
        date_b <- create_random_date_ts() |>
            normalize_date_ts(frequency_ts = 12L, test = FALSE)

        testthat::expect_error(diff_periode(
            a = date_a,
            b = date_b,
            frequency_ts = wrong_frequency
        ))
    }
})

# Initialisation ---------------------------------------------------------------

set.seed(2022L)

# Tests de résultat ------------------------------------------------------------

typeA <- "complex"
for (typeA in list_type) {
    for (frequenceA in list_frequence) {
        for (startA in list_start) {
            for (lenA in list_len[-1L]) {
                A_content <- create_random_type(type = typeA, len = lenA)
                ts_A <- ts(A_content, start = startA, frequency = frequenceA)
                for (param1 in list_len) {
                    for (param2 in list_len) {
                        test_name <- paste0(
                            "expected result with ",
                            "\ntypeA = '",
                            typeA,
                            "'\nfrequenceA = ",
                            frequenceA,
                            "\nstartA = ",
                            deparse(startA),
                            "\nlenA = ",
                            lenA,
                            "\nparam1 = ",
                            param1,
                            "\nparam2 = ",
                            param2
                        )

                        testthat::test_that(desc = test_name, {
                            # Cas 1
                            if (param1 < lenA & param1 + param2 > 0L) {
                                B1_content <- create_random_type(
                                    type = typeA,
                                    len = param1 + param2
                                )
                                startB1 <- end(ts_A)
                                if (length(startB1) == 1L) {
                                    startB1 <- c(startB1, 1L)
                                }
                                startB1[2L] <- startB1[2L] - (param1 - 1L)
                                ts_B1 <- ts(
                                    B1_content,
                                    start = startB1,
                                    frequency = frequenceA
                                )
                                # ts_B1 <- ts(
                                #     data = B1_content,
                                #     start = date_ts2timeunits(
                                #         as.integer(end(ts_A)),
                                #         frequency = frequenceA) - (param1 - 1L) / frequenceA,
                                #     frequency = frequenceA
                                # )

                                ts_ResAB1 <- ts(
                                    c(
                                        A_content[1L:(lenA - param1)],
                                        B1_content
                                    ),
                                    start = startA,
                                    frequency = frequenceA
                                )
                                if (param2 == 0L) {
                                    ts_ResB1A <- ts_A
                                } else {
                                    ts_ResB1A <- ts(
                                        c(
                                            A_content,
                                            B1_content[
                                                (param1 + 1L):(param1 + param2)
                                            ]
                                        ),
                                        start = startA,
                                        frequency = frequenceA
                                    )
                                }

                                if (param2 > 0L) {
                                    testthat::expect_warning(
                                        {
                                            resAB1 <- combine2ts(ts_A, ts_B1)
                                        },
                                        regexp = warning_extend
                                    )
                                } else {
                                    resAB1 <- combine2ts(ts_A, ts_B1)
                                }
                                testthat::expect_warning(
                                    {
                                        resB1A <- combine2ts(ts_B1, ts_A)
                                    },
                                    regexp = warning_extend
                                )
                                if (typeA != "Date") {
                                    testthat::expect_type(resAB1, typeA)
                                    testthat::expect_type(resB1A, typeA)
                                }

                                testthat::expect_equal(resAB1, ts_ResAB1)
                                testthat::expect_equal(resB1A, ts_ResB1A)
                            }

                            # Cas 2
                            if (param2 < lenA & param1 + param2 > 0L) {
                                B2_content <- create_random_type(
                                    type = typeA,
                                    len = param1 + param2
                                )
                                startB2 <- startA
                                if (length(startB2) == 1L) {
                                    startB2 <- c(startB2, 1L)
                                }
                                startB2[2L] <- startB2[2L] - param1
                                ts_B2 <- ts(
                                    B2_content,
                                    start = startB2,
                                    frequency = frequenceA
                                )
                                # ts_B2 <- ts(B2_content,  start = date_ts2timeunits(startA, frequency = frequenceA) - param1 / frequenceA, frequency = frequenceA)

                                ts_ResAB2 <- ts(
                                    c(
                                        B2_content,
                                        A_content[(param2 + 1L):lenA]
                                    ),
                                    start = start(ts_B2),
                                    frequency = frequenceA
                                )
                                if (param1 == 0L) {
                                    ts_ResB2A <- ts_A
                                } else {
                                    ts_ResB2A <- ts(
                                        c(B2_content[1L:param1], A_content),
                                        start = start(ts_B2),
                                        frequency = frequenceA
                                    )
                                }

                                if (param1 > 0L) {
                                    testthat::expect_warning(
                                        {
                                            resAB2 <- combine2ts(ts_A, ts_B2)
                                        },
                                        regexp = warning_extend
                                    )
                                } else {
                                    resAB2 <- combine2ts(ts_A, ts_B2)
                                }
                                testthat::expect_warning(
                                    {
                                        resB2A <- combine2ts(ts_B2, ts_A)
                                    },
                                    regexp = warning_extend
                                )

                                if (typeA != "Date") {
                                    testthat::expect_type(resAB2, typeA)
                                    testthat::expect_type(resB2A, typeA)
                                }

                                testthat::expect_equal(resAB2, ts_ResAB2)
                                testthat::expect_equal(resB2A, ts_ResB2A)
                            }

                            # Cas 3
                            if (param1 > 0L) {
                                B3_content <- create_random_type(
                                    type = typeA,
                                    len = param1
                                )
                                startB3 <- startA
                                if (length(startB3) == 1L) {
                                    startB3 <- c(startB3, 1L)
                                }
                                startB3[2L] <- startB3[2L] - (param1 + param2)
                                ts_B3 <- ts(
                                    B3_content,
                                    start = startB3,
                                    frequency = frequenceA
                                )
                                # ts_B3 <- ts(
                                #     data = B3_content,
                                #     start = date_ts2timeunits(
                                #         startA,
                                #         frequency = frequenceA) - (param1 + param2) / frequenceA,
                                #     frequency = frequenceA
                                # )

                                if (typeA == "raw") {
                                    ts_ResAB3 <- ts(
                                        c(
                                            B3_content,
                                            rep(as.raw(0L), param2),
                                            A_content
                                        ),
                                        start = start(ts_B3),
                                        frequency = frequenceA
                                    )
                                    ts_ResB3A <- ts(
                                        c(
                                            B3_content,
                                            rep(as.raw(0L), param2),
                                            A_content
                                        ),
                                        start = start(ts_B3),
                                        frequency = frequenceA
                                    )
                                    if (param2 > 0L) {
                                        testthat::expect_warning(
                                            testthat::expect_warning(
                                                {
                                                    resAB3 <- combine2ts(
                                                        ts_A,
                                                        ts_B3
                                                    )
                                                },
                                                regexp = warning_extend
                                            ),
                                            regexp = "out-of-range values treated as 0 in coercion to raw"
                                        )
                                        testthat::expect_warning(
                                            testthat::expect_warning(
                                                {
                                                    resB3A <- combine2ts(
                                                        ts_B3,
                                                        ts_A
                                                    )
                                                },
                                                regexp = warning_extend
                                            ),
                                            regexp = "out-of-range values treated as 0 in coercion to raw"
                                        )
                                    } else {
                                        testthat::expect_warning(
                                            {
                                                resAB3 <- combine2ts(
                                                    ts_A,
                                                    ts_B3
                                                )
                                            },
                                            regexp = warning_extend
                                        )
                                        testthat::expect_warning(
                                            {
                                                resB3A <- combine2ts(
                                                    ts_B3,
                                                    ts_A
                                                )
                                            },
                                            regexp = warning_extend
                                        )
                                    }
                                } else {
                                    ts_ResAB3 <- ts(
                                        c(
                                            B3_content,
                                            rep(NA, param2),
                                            A_content
                                        ),
                                        start = start(ts_B3),
                                        frequency = frequenceA
                                    )
                                    ts_ResB3A <- ts(
                                        c(
                                            B3_content,
                                            rep(NA, param2),
                                            A_content
                                        ),
                                        start = start(ts_B3),
                                        frequency = frequenceA
                                    )
                                    testthat::expect_warning(
                                        {
                                            resAB3 <- combine2ts(ts_A, ts_B3)
                                        },
                                        regexp = warning_extend
                                    )
                                    testthat::expect_warning(
                                        {
                                            resB3A <- combine2ts(ts_B3, ts_A)
                                        },
                                        regexp = warning_extend
                                    )
                                }

                                if (typeA != "Date") {
                                    testthat::expect_type(resAB3, typeA)
                                    testthat::expect_type(resB3A, typeA)
                                }

                                testthat::expect_equal(resAB3, ts_ResAB3)
                                testthat::expect_equal(resB3A, ts_ResB3A)
                            }

                            # Cas 4
                            if (param2 > 0L) {
                                B4_content <- create_random_type(
                                    type = typeA,
                                    len = param2
                                )
                                startB4 <- end(ts_A)
                                if (length(startB4) == 1L) {
                                    startB4 <- c(startB4, 1L)
                                }
                                startB4[2L] <- startB4[2L] + param1 + 1L
                                ts_B4 <- ts(
                                    B4_content,
                                    start = startB4,
                                    frequency = frequenceA
                                )
                                # ts_B4 <- ts(
                                #     data = B4_content,
                                #     start = date_ts2timeunits(
                                #         as.integer(end(ts_A)),
                                #         frequency = frequenceA) + (param1 + 1L) / frequenceA,
                                #     frequency = frequenceA
                                # )

                                if (typeA == "raw") {
                                    ts_ResAB4 <- ts(
                                        c(
                                            A_content,
                                            rep(as.raw(0L), param1),
                                            B4_content
                                        ),
                                        start = startA,
                                        frequency = frequenceA
                                    )
                                    ts_ResB4A <- ts(
                                        c(
                                            A_content,
                                            rep(as.raw(0L), param1),
                                            B4_content
                                        ),
                                        start = startA,
                                        frequency = frequenceA
                                    )
                                    if (param1 > 0L) {
                                        testthat::expect_warning(
                                            testthat::expect_warning(
                                                {
                                                    resAB4 <- combine2ts(
                                                        ts_A,
                                                        ts_B4
                                                    )
                                                },
                                                regexp = warning_extend
                                            ),
                                            regexp = "out-of-range values treated as 0 in coercion to raw"
                                        )
                                        testthat::expect_warning(
                                            testthat::expect_warning(
                                                {
                                                    resB4A <- combine2ts(
                                                        ts_B4,
                                                        ts_A
                                                    )
                                                },
                                                regexp = warning_extend
                                            ),
                                            regexp = "out-of-range values treated as 0 in coercion to raw"
                                        )
                                    } else {
                                        testthat::expect_warning(
                                            {
                                                resAB4 <- combine2ts(
                                                    ts_A,
                                                    ts_B4
                                                )
                                            },
                                            regexp = warning_extend
                                        )
                                        testthat::expect_warning(
                                            {
                                                resB4A <- combine2ts(
                                                    ts_B4,
                                                    ts_A
                                                )
                                            },
                                            regexp = warning_extend
                                        )
                                    }
                                } else {
                                    ts_ResAB4 <- ts(
                                        c(
                                            A_content,
                                            rep(NA, param1),
                                            B4_content
                                        ),
                                        start = startA,
                                        frequency = frequenceA
                                    )
                                    ts_ResB4A <- ts(
                                        c(
                                            A_content,
                                            rep(NA, param1),
                                            B4_content
                                        ),
                                        start = startA,
                                        frequency = frequenceA
                                    )
                                    testthat::expect_warning(
                                        {
                                            resAB4 <- combine2ts(ts_A, ts_B4)
                                        },
                                        regexp = warning_extend
                                    )
                                    testthat::expect_warning(
                                        {
                                            resB4A <- combine2ts(ts_B4, ts_A)
                                        },
                                        regexp = warning_extend
                                    )
                                }

                                if (typeA != "Date") {
                                    testthat::expect_type(resAB4, typeA)
                                    testthat::expect_type(resB4A, typeA)
                                }

                                testthat::expect_equal(resAB4, ts_ResAB4)
                                testthat::expect_equal(resB4A, ts_ResB4A)
                            }

                            # Cas 5
                            B5_content <- create_random_type(
                                type = typeA,
                                len = param1 + param2 + lenA
                            )
                            startB5 <- startA
                            if (length(startB5) == 1L) {
                                startB5 <- c(startB5, 1L)
                            }
                            startB5[2L] <- startB5[2L] - param1
                            ts_B5 <- ts(
                                B5_content,
                                start = startB5,
                                frequency = frequenceA
                            )
                            # ts_B5 <- ts(B5_content,  start = date_ts2timeunits(startA, frequency = frequenceA) - param1 / frequenceA, frequency = frequenceA)
                            ts_ResAB5 <- ts_B5

                            if (param1 == 0L & param2 == 0L) {
                                ts_ResB5A <- ts_A
                            } else if (param1 == 0L) {
                                ts_ResB5A <- ts(
                                    c(
                                        A_content,
                                        B5_content[
                                            (param1 + lenA + 1):(param1 +
                                                param2 +
                                                lenA)
                                        ]
                                    ),
                                    start = start(ts_B5),
                                    frequency = frequenceA
                                )
                            } else if (param2 == 0L) {
                                ts_ResB5A <- ts(
                                    c(B5_content[1L:param1], A_content),
                                    start = start(ts_B5),
                                    frequency = frequenceA
                                )
                            } else {
                                ts_ResB5A <- ts(
                                    data = c(
                                        B5_content[1L:param1],
                                        A_content,
                                        B5_content[
                                            (param1 + lenA + 1):(param1 +
                                                param2 +
                                                lenA)
                                        ]
                                    ),
                                    start = start(ts_B5),
                                    frequency = frequenceA
                                )
                            }

                            if (param1 + param2 > 0L) {
                                testthat::expect_warning(
                                    {
                                        resAB5 <- combine2ts(ts_A, ts_B5)
                                    },
                                    regexp = warning_extend
                                )
                            } else {
                                resAB5 <- combine2ts(ts_A, ts_B5)
                            }
                            resB5A <- combine2ts(ts_B5, ts_A)

                            if (typeA != "Date") {
                                testthat::expect_type(resAB5, typeA)
                                testthat::expect_type(resB5A, typeA)
                            }

                            testthat::expect_equal(resAB5, ts_ResAB5)
                            testthat::expect_equal(resB5A, ts_ResB5A)

                            # Cas 6
                            if (param1 + param2 < lenA & param2 > 0L) {
                                B6_content <- create_random_type(
                                    type = typeA,
                                    len = param2
                                )
                                startB6 <- startA
                                if (length(startB6) == 1L) {
                                    startB6 <- c(startB6, 1L)
                                }
                                startB6[2L] <- startB6[2L] + param1
                                ts_B6 <- ts(
                                    B6_content,
                                    start = startB6,
                                    frequency = frequenceA
                                )
                                # ts_B6 <- ts(B6_content,  start = date_ts2timeunits(startA, frequency = frequenceA) + param1 / frequenceA, frequency = frequenceA)
                                if (param1 == 0L) {
                                    ts_ResAB6 <- ts(
                                        c(
                                            B6_content,
                                            A_content[
                                                (param1 + param2 + 1L):lenA
                                            ]
                                        ),
                                        start = startA,
                                        frequency = frequenceA
                                    )
                                } else {
                                    ts_ResAB6 <- ts(
                                        c(
                                            A_content[1L:param1],
                                            B6_content,
                                            A_content[
                                                (param1 + param2 + 1L):lenA
                                            ]
                                        ),
                                        start = startA,
                                        frequency = frequenceA
                                    )
                                }
                                ts_ResB6A <- ts_A

                                resAB6 <- combine2ts(ts_A, ts_B6)
                                testthat::expect_warning(
                                    {
                                        resB6A <- combine2ts(ts_B6, ts_A)
                                    },
                                    regexp = warning_extend
                                )

                                if (typeA != "Date") {
                                    testthat::expect_type(resAB6, typeA)
                                    testthat::expect_type(resB6A, typeA)
                                }

                                testthat::expect_equal(resAB6, ts_ResAB6)
                                testthat::expect_equal(resB6A, ts_ResB6A)
                            }
                        })
                    }
                }
            }
        }
    }
}

# Tests sur les erreurs de mts -------------------------------------------------

testthat::test_that("Several dimensions are not allowed", {
    for (typeA in list_type) {
        for (frequenceA in list_frequence) {
            for (startA in list_start) {
                for (lenA in list_len[-1L]) {
                    ts_A <- create_random_ts(
                        type = typeA,
                        start = startA,
                        frequency = frequenceA,
                        len = lenA
                    )
                    B_content <- as.data.frame(lapply(
                        1L:5L,
                        function(i) create_random_type(type = typeA, len = 100L)
                    ))
                    startB <- create_random_date_ts()

                    if (typeA == "complex") {
                        mts_B <- do.call(
                            what = cbind,
                            args = lapply(
                                X = B_content,
                                FUN = ts,
                                start = startB,
                                frequency = frequenceA
                            )
                        )
                    } else {
                        mts_B <- ts(
                            B_content,
                            start = startB,
                            frequency = frequenceA
                        )
                    }

                    testthat::expect_error(
                        object = combine2ts(a = ts_A, b = mts_B),
                        regexp = "Variable 'b': Must be of type 'atomic vector'"
                    )
                    testthat::expect_error(
                        object = combine2ts(a = mts_B, b = ts_A),
                        regexp = "Variable 'a': Must be of type 'atomic vector'"
                    )
                }
            }
        }
    }
})

# Tests sur les erreurs d'input ------------------------------------------------

testthat::test_that("miscellaneous input are not allowed", {
    for (typeA in list_type) {
        ts_A <- create_random_ts(type = typeA)

        for (objA in object_bank_R) {
            testthat::expect_error(
                combine2ts(ts_A, objA)
            )
            testthat::expect_error(
                combine2ts(objA, ts_A)
            )
            for (objB in object_bank_R) {
                testthat::expect_error(
                    combine2ts(objA, objB)
                )
            }
        }
    }
})

# Tests sur les erreurs de type d'objets ---------------------------------------

testthat::test_that("different input type are not allowed", {
    for (typeA in list_type[-7L]) {
        objA <- create_random_ts(type = typeA, frequency = 12L)
        for (typeB in list_type[-7L]) {
            objB <- create_random_ts(type = typeB, frequency = 12L)
            if (typeA != typeB) {
                testthat::expect_error(
                    combine2ts(objA, objB),
                    regexp = "Les objets `a` et `b` doivent \u00eatre de m\u00eame type."
                )
            }
        }
    }
})

# Tests sur les erreurs de temporalité -----------------------------------------

testthat::test_that("arguments have same frequency", {
    for (typeA in list_type) {
        objA <- create_random_ts(type = typeA, frequency = 12L)
        objB <- create_random_ts(type = typeA, frequency = 4L)
        testthat::expect_error(
            combine2ts(objA, objB),
            regexp = "Les objets `a` et `b` doivent avoir la m\u00eame fr\u00e9quence."
        )
        testthat::expect_error(
            combine2ts(objB, objA),
            regexp = "Les objets `a` et `b` doivent avoir la m\u00eame fr\u00e9quence."
        )
    }
})

testthat::test_that("arguments are monthly or quarterly", {
    for (typeA in list_type) {
        for (freq_A in c(weird_frequency)) {
            for (freq_B in c(weird_frequency, list_frequence)) {
                objA <- create_random_ts(type = typeA, frequency = freq_A)
                objB <- create_random_ts(type = typeA, frequency = freq_B)
                testthat::expect_error(combine2ts(objA, objB))
                testthat::expect_error(combine2ts(objB, objA))
            }
        }
    }
})

testthat::test_that("arguments are temporally consistent", {
    for (typeA in list_type) {
        ts_A <- create_random_ts(type = typeA, start = 2015L, frequency = 12L)
        ts_B <- create_random_ts(
            type = typeA,
            start = 2004 + 1 / 7,
            frequency = 12L
        )
        testthat::expect_error(combine2ts(ts_A, ts_B))
        testthat::expect_error(combine2ts(ts_B, ts_A))

        ts_A <- create_random_ts(type = typeA, start = 2015L, frequency = 4L)
        ts_B <- create_random_ts(
            type = typeA,
            start = 2016 + 1 / 12,
            frequency = 4L
        )
        testthat::expect_error(combine2ts(ts_A, ts_B))
        testthat::expect_error(combine2ts(ts_B, ts_A))
    }
})

# Initialisation ---------------------------------------------------------------

set.seed(2051L)


# Tests de résultat ------------------------------------------------------------

typeA <- "complex"

for (frequenceA in list_frequence) {
    for (startA in list_start) {
        for (lenA in list_len[-1L]) {
            A_content <- create_random_type(type = typeA, len = lenA)
            ts_A <- ts(A_content, start = startA, frequency = frequenceA)
            for (param1 in list_len) {
                for (param2 in list_len) {
                    test_name <- paste0(
                        "expected result with ",
                        "\ntypeA = '",
                        typeA,
                        "'\nfrequenceA = ",
                        frequenceA,
                        "\nstartA = ",
                        deparse(startA),
                        "\nlenA = ",
                        lenA,
                        "\nparam1 = ",
                        param1,
                        "\nparam2 = ",
                        param2
                    )

                    testthat::test_that(desc = test_name, {
                        # Cas 1
                        if (param1 <= lenA & param1 > 0L) {
                            date_from_1 <- end(ts_A) |> as.integer()
                            if (length(date_from_1) == 1L) {
                                date_from_1 <- c(date_from_1, 1L)
                            }
                            date_from_1[2L] <- date_from_1[2L] - (param1 - 1L)
                            date_from_1 <- c(
                                date_from_1[1L] +
                                    (date_from_1[2L] - 1L) %/% frequenceA,
                                (date_from_1[2L] - 1L) %% frequenceA + 1L
                            )

                            date_to_1 <- end(ts_A) |> as.integer()
                            if (length(date_to_1) == 1L) {
                                date_to_1 <- c(date_to_1, 1L)
                            }
                            date_to_1[2L] <- date_to_1[2L] + param2
                            date_to_1 <- c(
                                date_to_1[1L] +
                                    (date_to_1[2L] - 1L) %/% frequenceA,
                                (date_to_1[2L] - 1L) %% frequenceA + 1L
                            )

                            res_theo_1 <- c(
                                A_content[(lenA - param1 + 1L):lenA],
                                create_NA_type(type = typeA, len = param2)
                            )

                            vect_res1 <- get_value_ts(
                                series = ts_A,
                                date_from = date_from_1,
                                date_to = date_to_1
                            )
                            vect_res2 <- get_value_ts(
                                series = ts_A,
                                date_from = date_from_1,
                                n = param1 + param2
                            )
                            vect_res3 <- get_value_ts(
                                series = ts_A,
                                date_to = date_to_1,
                                n = param1 + param2
                            )

                            testthat::expect_identical(res_theo_1, vect_res1)
                            testthat::expect_identical(res_theo_1, vect_res2)
                            testthat::expect_identical(res_theo_1, vect_res3)
                        }

                        # Cas 2
                        if (param2 <= lenA & param2 > 0L) {
                            date_from_2 <- start(ts_A) |> as.integer()
                            if (length(date_from_2) == 1L) {
                                date_from_2 <- c(date_from_2, 1L)
                            }
                            date_from_2[2L] <- date_from_2[2L] - param1
                            date_from_2 <- c(
                                date_from_2[1L] +
                                    (date_from_2[2L] - 1L) %/% frequenceA,
                                (date_from_2[2L] - 1L) %% frequenceA + 1L
                            )

                            date_to_2 <- start(ts_A) |> as.integer()
                            if (length(date_to_2) == 1L) {
                                date_to_2 <- c(date_to_2, 1L)
                            }
                            date_to_2[2L] <- date_to_2[2L] + param2 - 1L
                            date_to_2 <- c(
                                date_to_2[1L] +
                                    (date_to_2[2L] - 1L) %/% frequenceA,
                                (date_to_2[2L] - 1L) %% frequenceA + 1L
                            )

                            res_theo_2 <- c(
                                create_NA_type(type = typeA, len = param1),
                                A_content[1:param2]
                            )

                            vect_res1 <- get_value_ts(
                                ts_A,
                                date_from = date_from_2,
                                date_to = date_to_2
                            )
                            vect_res2 <- get_value_ts(
                                ts_A,
                                date_from = date_from_2,
                                n = param1 + param2
                            )
                            vect_res3 <- get_value_ts(
                                ts_A,
                                date_to = date_to_2,
                                n = param1 + param2
                            )

                            testthat::expect_identical(res_theo_2, vect_res1)
                            testthat::expect_identical(res_theo_2, vect_res2)
                            testthat::expect_identical(res_theo_2, vect_res3)
                        }

                        # Cas 3
                        if (param1 > 0L) {
                            date_from_3 <- start(ts_A) |> as.integer()
                            if (length(date_from_3) == 1L) {
                                date_from_3 <- c(date_from_3, 1L)
                            }
                            date_from_3[2L] <- date_from_3[2L] - param1 - param2
                            date_from_3 <- c(
                                date_from_3[1L] +
                                    (date_from_3[2L] - 1L) %/% frequenceA,
                                (date_from_3[2L] - 1L) %% frequenceA + 1L
                            )

                            date_to_3 <- start(ts_A) |> as.integer()
                            if (length(date_to_3) == 1L) {
                                date_to_3 <- c(date_to_3, 1L)
                            }
                            date_to_3[2L] <- date_to_3[2L] - param2 - 1L
                            date_to_3 <- c(
                                date_to_3[1L] +
                                    (date_to_3[2L] - 1L) %/% frequenceA,
                                (date_to_3[2L] - 1L) %% frequenceA + 1L
                            )

                            res_theo_3 <- create_NA_type(
                                type = typeA,
                                len = param1
                            )

                            vect_res1 <- get_value_ts(
                                ts_A,
                                date_from = date_from_3,
                                date_to = date_to_3
                            )
                            vect_res2 <- get_value_ts(
                                ts_A,
                                date_from = date_from_3,
                                n = param1
                            )
                            vect_res3 <- get_value_ts(
                                ts_A,
                                date_to = date_to_3,
                                n = param1
                            )

                            testthat::expect_identical(res_theo_3, vect_res1)
                            testthat::expect_identical(res_theo_3, vect_res2)
                            testthat::expect_identical(res_theo_3, vect_res3)
                        }

                        # Cas 4
                        if (param2 > 0L) {
                            date_from_4 <- end(ts_A) |> as.integer()
                            if (length(date_from_4) == 1L) {
                                date_from_4 <- c(date_from_4, 1L)
                            }
                            date_from_4[2L] <- date_from_4[2L] + param1 + 1L
                            date_from_4 <- c(
                                date_from_4[1L] +
                                    (date_from_4[2L] - 1L) %/% frequenceA,
                                (date_from_4[2L] - 1L) %% frequenceA + 1L
                            )

                            date_to_4 <- end(ts_A) |> as.integer()
                            if (length(date_to_4) == 1L) {
                                date_to_4 <- c(date_to_4, 1L)
                            }
                            date_to_4[2L] <- date_to_4[2L] + param1 + param2
                            date_to_4 <- c(
                                date_to_4[1L] +
                                    (date_to_4[2L] - 1L) %/% frequenceA,
                                (date_to_4[2L] - 1L) %% frequenceA + 1L
                            )

                            res_theo_4 <- create_NA_type(
                                type = typeA,
                                len = param2
                            )

                            vect_res1 <- get_value_ts(
                                ts_A,
                                date_from = date_from_4,
                                date_to = date_to_4
                            )
                            vect_res2 <- get_value_ts(
                                ts_A,
                                date_from = date_from_4,
                                n = param2
                            )
                            vect_res3 <- get_value_ts(
                                ts_A,
                                date_to = date_to_4,
                                n = param2
                            )

                            testthat::expect_identical(res_theo_4, vect_res1)
                            testthat::expect_identical(res_theo_4, vect_res2)
                            testthat::expect_identical(res_theo_4, vect_res3)
                        }

                        # Cas 5
                        date_from_5 <- start(ts_A) |> as.integer()
                        if (length(date_from_5) == 1L) {
                            date_from_5 <- c(date_from_5, 1L)
                        }
                        date_from_5[2L] <- date_from_5[2L] - param1
                        date_from_5 <- c(
                            date_from_5[1L] +
                                (date_from_5[2L] - 1L) %/% frequenceA,
                            (date_from_5[2L] - 1L) %% frequenceA + 1L
                        )

                        date_to_5 <- end(ts_A) |> as.integer()
                        if (length(date_to_5) == 1L) {
                            date_to_5 <- c(date_to_5, 1L)
                        }
                        date_to_5[2L] <- date_to_5[2L] + param2
                        date_to_5 <- c(
                            date_to_5[1L] + (date_to_5[2L] - 1L) %/% frequenceA,
                            (date_to_5[2L] - 1L) %% frequenceA + 1L
                        )

                        res_theo_5 <- c(
                            create_NA_type(type = typeA, len = param1),
                            A_content,
                            create_NA_type(type = typeA, len = param2)
                        )

                        vect_res1 <- get_value_ts(
                            ts_A,
                            date_from = date_from_5,
                            date_to = date_to_5
                        )
                        vect_res2 <- get_value_ts(
                            ts_A,
                            date_from = date_from_5,
                            n = param2 + lenA + param1
                        )
                        vect_res3 <- get_value_ts(
                            ts_A,
                            date_to = date_to_5,
                            n = param2 + lenA + param1
                        )

                        testthat::expect_identical(res_theo_5, vect_res1)
                        testthat::expect_identical(res_theo_5, vect_res2)
                        testthat::expect_identical(res_theo_5, vect_res3)

                        # Cas 6
                        if (param1 + param2 <= lenA && param2 > 0) {
                            date_from_6 <- start(ts_A) |> as.integer()
                            if (length(date_from_6) == 1L) {
                                date_from_6 <- c(date_from_6, 1L)
                            }
                            date_from_6[2L] <- date_from_6[2L] + param1
                            date_from_6 <- c(
                                date_from_6[1L] +
                                    (date_from_6[2L] - 1L) %/% frequenceA,
                                (date_from_6[2L] - 1L) %% frequenceA + 1L
                            )

                            date_to_6 <- start(ts_A) |> as.integer()
                            if (length(date_to_6) == 1L) {
                                date_to_6 <- c(date_to_6, 1L)
                            }
                            date_to_6[2L] <- date_to_6[2L] +
                                param1 +
                                param2 -
                                1L
                            date_to_6 <- c(
                                date_to_6[1L] +
                                    (date_to_6[2L] - 1L) %/% frequenceA,
                                (date_to_6[2L] - 1L) %% frequenceA + 1L
                            )

                            res_theo_6 <- A_content[
                                (param1 + 1L):(param1 + param2)
                            ]

                            vect_res1 <- get_value_ts(
                                ts_A,
                                date_from = date_from_6,
                                date_to = date_to_6
                            )
                            vect_res2 <- get_value_ts(
                                ts_A,
                                date_from = date_from_6,
                                n = param2
                            )
                            vect_res3 <- get_value_ts(
                                ts_A,
                                date_to = date_to_6,
                                n = param2
                            )

                            testthat::expect_identical(res_theo_6, vect_res1)
                            testthat::expect_identical(res_theo_6, vect_res2)
                            testthat::expect_identical(res_theo_6, vect_res3)
                        }
                    })
                }
            }
        }
    }
}

# Initialisation ---------------------------------------------------------------

set.seed(2025L)


# Tests de résultats positifs --------------------------------------------------

testthat::test_that("good result for integer date", {
    for (month in good_months) {
        for (year in good_years) {
            real_year <- year + (month - 1L) %/% 12L
            testthat::expect_identical(
                libelles_one_date(date_ts = c(year, month), frequency_ts = 12L),
                paste(list_months_name[(month - 1L) %% 12L + 1L], real_year)
            )
        }
    }
})

testthat::test_that("good result for integer date", {
    for (quarter in good_quarters) {
        for (year in good_years) {
            real_year <- year + (quarter - 1L) %/% 4L
            testthat::expect_identical(
                libelles_one_date(
                    date_ts = c(year, quarter),
                    frequency_ts = 4L
                ),
                paste0("Q", (quarter - 1L) %% 4L + 1L, " ", real_year)
            )
        }
    }
})


# Test de résultats positifs avec warning ---------------------------------

testthat::test_that("warning for integer date out of range", {
    for (month in warning_integer_months) {
        for (year in good_years) {
            real_year <- year + (month - 1L) %/% 12L
            testthat::expect_warning(
                {
                    libel1 <- libelles_one_date(
                        date_ts = c(year, month),
                        frequency_ts = 12L,
                        warn = TRUE
                    )
                    testthat::expect_identical(
                        libel1,
                        paste(
                            list_months_name[(month - 1L) %% 12L + 1L],
                            real_year
                        )
                    )
                },
                regexp = "Assertion on 'period' failed: Element 1 is not <= 12.|Assertion on 'period' failed: Element 1 is not >= 1."
            )
            testthat::expect_warning(
                {
                    libel2 <- libelles_one_date(
                        date_ts = c(year, month),
                        frequency_ts = 12L
                    )
                    testthat::expect_identical(
                        libel2,
                        paste(
                            list_months_name[(month - 1L) %% 12L + 1L],
                            real_year
                        )
                    )
                },
                regexp = "Assertion on 'period' failed: Element 1 is not <= 12.|Assertion on 'period' failed: Element 1 is not >= 1."
            )
            testthat::expect_no_warning(
                {
                    libel3 <- libelles_one_date(
                        date_ts = c(year, month),
                        frequency_ts = 12L,
                        warn = FALSE
                    )
                    testthat::expect_identical(
                        libel3,
                        paste(
                            list_months_name[(month - 1L) %% 12L + 1L],
                            real_year
                        )
                    )
                }
            )
        }
    }
})

testthat::test_that("warning for integer date out of range", {
    for (quarter in warning_integer_quarters) {
        for (year in good_years) {
            real_year <- year + (quarter - 1L) %/% 4L
            testthat::expect_warning(
                {
                    libel1 <- libelles_one_date(
                        date_ts = c(year, quarter),
                        frequency_ts = 4L,
                        warn = TRUE
                    )
                },
                regexp = "Assertion on 'period' failed: Element 1 is not <= 4.|Assertion on 'period' failed: Element 1 is not >= 1."
            )
            testthat::expect_warning(
                {
                    libel2 <- libelles_one_date(
                        date_ts = c(year, quarter),
                        frequency_ts = 4L
                    )
                },
                regexp = "Assertion on 'period' failed: Element 1 is not <= 4.|Assertion on 'period' failed: Element 1 is not >= 1."
            )
            testthat::expect_no_warning(
                {
                    libel3 <- libelles_one_date(
                        date_ts = c(year, quarter),
                        frequency_ts = 4L,
                        warn = FALSE
                    )
                }
            )
            testthat::expect_identical(
                libel1,
                paste0("Q", (quarter - 1L) %% 4L + 1L, " ", real_year)
            )
            testthat::expect_identical(
                libel2,
                paste0("Q", (quarter - 1L) %% 4L + 1L, " ", real_year)
            )
            testthat::expect_identical(
                libel3,
                paste0("Q", (quarter - 1L) %% 4L + 1L, " ", real_year)
            )
        }
    }
})

testthat::test_that("warning for double date", {
    for (month in double_months) {
        for (year in good_years) {
            real_year <- year + (month - 1L) %/% 12L
            testthat::expect_warning(
                {
                    libel1 <- libelles_one_date(
                        date_ts = c(year, month),
                        frequency_ts = 12L,
                        warn = TRUE
                    )
                },
                regexp = message_double("date_ts")
            )
            testthat::expect_warning(
                {
                    libel2 <- libelles_one_date(
                        date_ts = c(year, month),
                        frequency_ts = 12L
                    )
                },
                regexp = message_double("date_ts")
            )
            testthat::expect_no_warning(
                {
                    libel3 <- libelles_one_date(
                        date_ts = c(year, month),
                        frequency_ts = 12L,
                        warn = FALSE
                    )
                }
            )
            testthat::expect_identical(
                libel1,
                paste(list_months_name[(month - 1L) %% 12L + 1L], real_year)
            )
            testthat::expect_identical(
                libel2,
                paste(list_months_name[(month - 1L) %% 12L + 1L], real_year)
            )
            testthat::expect_identical(
                libel3,
                paste(list_months_name[(month - 1L) %% 12L + 1L], real_year)
            )
        }
    }

    for (month in good_months) {
        for (year in double_years) {
            real_year <- year + (month - 1L) %/% 12L
            testthat::expect_warning(
                {
                    libel1 <- libelles_one_date(
                        date_ts = c(year, month),
                        frequency_ts = 12L,
                        warn = TRUE
                    )
                },
                regexp = message_double("date_ts")
            )
            testthat::expect_warning(
                {
                    libel2 <- libelles_one_date(
                        date_ts = c(year, month),
                        frequency_ts = 12L
                    )
                },
                regexp = message_double("date_ts")
            )
            testthat::expect_no_warning(
                {
                    libel3 <- libelles_one_date(
                        date_ts = c(year, month),
                        frequency_ts = 12L,
                        warn = FALSE
                    )
                }
            )
            testthat::expect_identical(
                libel1,
                paste(list_months_name[(month - 1L) %% 12L + 1L], real_year)
            )
            testthat::expect_identical(
                libel2,
                paste(list_months_name[(month - 1L) %% 12L + 1L], real_year)
            )
            testthat::expect_identical(
                libel3,
                paste(list_months_name[(month - 1L) %% 12L + 1L], real_year)
            )
        }
    }

    for (quarter in double_quarters) {
        for (year in good_years) {
            real_year <- year + (quarter - 1L) %/% 4L
            testthat::expect_warning(
                {
                    libel1 <- libelles_one_date(
                        date_ts = c(year, quarter),
                        frequency_ts = 4L,
                        warn = TRUE
                    )
                },
                regexp = message_double("date_ts")
            )
            testthat::expect_warning(
                {
                    libel2 <- libelles_one_date(
                        date_ts = c(year, quarter),
                        frequency_ts = 4L
                    )
                },
                regexp = message_double("date_ts")
            )
            testthat::expect_no_warning(
                {
                    libel3 <- libelles_one_date(
                        date_ts = c(year, quarter),
                        frequency_ts = 4L,
                        warn = FALSE
                    )
                }
            )
            testthat::expect_identical(
                libel1,
                paste0("Q", (quarter - 1L) %% 4L + 1L, " ", real_year)
            )
            testthat::expect_identical(
                libel2,
                paste0("Q", (quarter - 1L) %% 4L + 1L, " ", real_year)
            )
            testthat::expect_identical(
                libel3,
                paste0("Q", (quarter - 1L) %% 4L + 1L, " ", real_year)
            )
        }
    }

    for (quarter in good_quarters) {
        for (year in double_years) {
            real_year <- year + (quarter - 1L) %/% 4L
            testthat::expect_warning(
                {
                    libel1 <- libelles_one_date(
                        date_ts = c(year, quarter),
                        frequency_ts = 4L,
                        warn = TRUE
                    )
                },
                regexp = message_double("date_ts")
            )
            testthat::expect_warning(
                {
                    libel2 <- libelles_one_date(
                        date_ts = c(year, quarter),
                        frequency_ts = 4L
                    )
                },
                regexp = message_double("date_ts")
            )
            testthat::expect_no_warning(
                {
                    libel3 <- libelles_one_date(
                        date_ts = c(year, quarter),
                        frequency_ts = 4L,
                        warn = FALSE
                    )
                }
            )
            testthat::expect_identical(
                libel1,
                paste0("Q", (quarter - 1L) %% 4L + 1L, " ", real_year)
            )
            testthat::expect_identical(
                libel2,
                paste0("Q", (quarter - 1L) %% 4L + 1L, " ", real_year)
            )
            testthat::expect_identical(
                libel3,
                paste0("Q", (quarter - 1L) %% 4L + 1L, " ", real_year)
            )
        }
    }
})

testthat::test_that("warning for double date and out of range", {
    for (month in warning_double_months) {
        for (year in good_years) {
            real_year <- year + (month - 1L) %/% 12L
            testthat::expect_warning(
                testthat::expect_warning(
                    object = {
                        libel1 <- libelles_one_date(
                            date_ts = c(year, month),
                            frequency_ts = 12L,
                            warn = TRUE
                        )
                    },
                    regexp = "Assertion on 'period' failed: Element 1 is not <= 12.|Assertion on 'period' failed: Element 1 is not >= 1."
                ),
                regexp = message_double("date_ts")
            )
            testthat::expect_warning(
                testthat::expect_warning(
                    object = {
                        libel2 <- libelles_one_date(
                            date_ts = c(year, month),
                            frequency_ts = 12L
                        )
                    },
                    regexp = "Assertion on 'period' failed: Element 1 is not <= 12.|Assertion on 'period' failed: Element 1 is not >= 1."
                ),
                regexp = message_double("date_ts")
            )
            testthat::expect_no_warning(
                object = {
                    libel3 <- libelles_one_date(
                        date_ts = c(year, month),
                        frequency_ts = 12L,
                        warn = FALSE
                    )
                }
            )
            testthat::expect_identical(
                libel1,
                paste(list_months_name[(month - 1L) %% 12L + 1L], real_year)
            )
            testthat::expect_identical(
                libel2,
                paste(list_months_name[(month - 1L) %% 12L + 1L], real_year)
            )
            testthat::expect_identical(
                libel3,
                paste(list_months_name[(month - 1L) %% 12L + 1L], real_year)
            )
        }
    }

    for (quarter in warning_double_quarters) {
        for (year in good_years) {
            real_year <- year + (quarter - 1L) %/% 4L
            testthat::expect_warning(
                testthat::expect_warning(
                    object = {
                        libel1 <- libelles_one_date(
                            date_ts = c(year, quarter),
                            frequency_ts = 4L,
                            warn = TRUE
                        )
                    },
                    regexp = "Assertion on 'period' failed: Element 1 is not <= 4.|Assertion on 'period' failed: Element 1 is not >= 1."
                ),
                regexp = message_double("date_ts")
            )
            testthat::expect_warning(
                testthat::expect_warning(
                    object = {
                        libel2 <- libelles_one_date(
                            date_ts = c(year, quarter),
                            frequency_ts = 4L
                        )
                    },
                    regexp = "Assertion on 'period' failed: Element 1 is not <= 4.|Assertion on 'period' failed: Element 1 is not >= 1."
                ),
                regexp = message_double("date_ts")
            )
            testthat::expect_no_warning(
                object = {
                    libel3 <- libelles_one_date(
                        date_ts = c(year, quarter),
                        frequency_ts = 4L,
                        warn = FALSE
                    )
                }
            )
            testthat::expect_identical(
                libel1,
                paste0("Q", (quarter - 1L) %% 4L + 1L, " ", real_year)
            )
            testthat::expect_identical(
                libel2,
                paste0("Q", (quarter - 1L) %% 4L + 1L, " ", real_year)
            )
            testthat::expect_identical(
                libel3,
                paste0("Q", (quarter - 1L) %% 4L + 1L, " ", real_year)
            )
        }
    }
})

testthat::test_that("warning for double monthly frequency", {
    for (month in good_months) {
        for (year in good_years) {
            real_year <- year + (month - 1L) %/% 12L
            testthat::expect_warning(
                {
                    libel1 <- libelles_one_date(
                        date_ts = c(year, month),
                        frequency_ts = 12.0,
                        warn = TRUE
                    )
                },
                regexp = message_double("frequency_ts")
            )
            testthat::expect_warning(
                {
                    libel2 <- libelles_one_date(
                        date_ts = c(year, month),
                        frequency_ts = 12.0
                    )
                },
                regexp = message_double("frequency_ts")
            )
            testthat::expect_no_warning(
                {
                    libel3 <- libelles_one_date(
                        date_ts = c(year, month),
                        frequency_ts = 12.0,
                        warn = FALSE
                    )
                }
            )
            testthat::expect_identical(
                libel1,
                paste(list_months_name[(month - 1L) %% 12L + 1L], real_year)
            )
            testthat::expect_identical(
                libel2,
                paste(list_months_name[(month - 1L) %% 12L + 1L], real_year)
            )
            testthat::expect_identical(
                libel3,
                paste(list_months_name[(month - 1L) %% 12L + 1L], real_year)
            )
        }
    }
})

testthat::test_that("warning for double quaterly frequency", {
    for (quarter in good_quarters) {
        for (year in good_years) {
            real_year <- year + (quarter - 1L) %/% 4L
            testthat::expect_warning(
                {
                    libel1 <- libelles_one_date(
                        date_ts = c(year, quarter),
                        frequency_ts = 4.0,
                        warn = TRUE
                    )
                },
                regexp = message_double("frequency_ts")
            )
            testthat::expect_warning(
                {
                    libel2 <- libelles_one_date(
                        date_ts = c(year, quarter),
                        frequency_ts = 4.0
                    )
                },
                regexp = message_double("frequency_ts")
            )
            testthat::expect_no_warning(
                {
                    libel3 <- libelles_one_date(
                        date_ts = c(year, quarter),
                        frequency_ts = 4.0,
                        warn = FALSE
                    )
                }
            )
            testthat::expect_identical(
                libel1,
                paste0("Q", (quarter - 1L) %% 4L + 1L, " ", real_year)
            )
            testthat::expect_identical(
                libel2,
                paste0("Q", (quarter - 1L) %% 4L + 1L, " ", real_year)
            )
            testthat::expect_identical(
                libel3,
                paste0("Q", (quarter - 1L) %% 4L + 1L, " ", real_year)
            )
        }
    }
})


# Tests de résultats négatifs --------------------------------------------------

testthat::test_that("miscellaneous date are not allowed", {
    for (wrong_date in list_wrong_date_ts) {
        testthat::expect_error(libelles_one_date(
            date_ts = wrong_date,
            frequency_ts = 12L,
            warn = FALSE
        ))
        testthat::expect_error(libelles_one_date(
            date_ts = wrong_date,
            frequency_ts = 4L,
            warn = FALSE
        ))
    }
})

testthat::test_that("miscellaneous frequency are not allowed", {
    for (wrong_frequency in c(object_bank_R, weird_frequency)) {
        testthat::expect_error(libelles_one_date(
            date_ts = create_random_date_ts(),
            frequency_ts = wrong_frequency,
            warn = FALSE
        ))
    }
})

# Initialisation ---------------------------------------------------------------

set.seed(2044L)


# Tests de résultat avec start vecteur d'entiers -------------------------------

typeA <- "Date"

for (frequenceA in list_frequence) {
    for (startA in list_start) {
        for (lenA in list_len[-1L]) {
            A_content <- create_random_type(type = typeA, len = lenA)
            for (lenB in list_len[-1L]) {
                B_content <- create_random_type(type = typeA, len = lenB)

                test_name <- paste0(
                    "expected result with ",
                    "\ntypeA = ",
                    deparse(typeA),
                    "\nfrequenceA = ",
                    deparse(frequenceA),
                    "\nstartA = ",
                    deparse(startA),
                    "\nlenA = ",
                    deparse(lenA),
                    "\nlenB = ",
                    deparse(lenB)
                )

                testthat::test_that(test_name, {
                    # Cas 1 : simple
                    ts_A <- ts(
                        A_content,
                        start = startA,
                        frequency = frequenceA
                    )
                    res_theo <- ts(
                        c(A_content, B_content),
                        start = startA,
                        frequency = frequenceA
                    )
                    testthat::expect_warning(
                        {
                            res <- extend_ts(
                                series = ts_A,
                                replacement = B_content,
                                date_ts = NULL,
                                replace_na = FALSE
                            )
                        },
                        regexp = warning_extend
                    )
                    testthat::expect_equal(expected = res_theo, object = res)

                    # Cas 1.5 : simple (by default)
                    ts_A <- ts(
                        A_content,
                        start = startA,
                        frequency = frequenceA
                    )
                    res_theo <- ts(
                        c(A_content, B_content),
                        start = startA,
                        frequency = frequenceA
                    )
                    testthat::expect_warning(
                        {
                            res <- extend_ts(
                                series = ts_A,
                                replacement = B_content
                            )
                        },
                        regexp = warning_extend
                    )
                    testthat::expect_equal(expected = res_theo, object = res)

                    # Cas 2 : with with NA
                    for (param1 in list_len) {
                        ts_A <- ts(
                            c(
                                A_content,
                                create_NA_type(type = typeA, len = param1)
                            ),
                            start = startA,
                            frequency = frequenceA
                        )
                        res_theo <- ts(
                            c(
                                A_content,
                                create_NA_type(type = typeA, len = param1),
                                B_content
                            ),
                            start = startA,
                            frequency = frequenceA
                        )
                        testthat::expect_warning(
                            {
                                res <- extend_ts(
                                    series = ts_A,
                                    replacement = B_content,
                                    date_ts = NULL,
                                    replace_na = FALSE
                                )
                            },
                            regexp = warning_extend
                        )
                        testthat::expect_equal(
                            expected = res_theo,
                            object = res
                        )
                    }

                    # Cas 2.5 : with with NA (by default)
                    for (param1 in list_len) {
                        ts_A <- ts(
                            c(
                                A_content,
                                create_NA_type(type = typeA, len = param1)
                            ),
                            start = startA,
                            frequency = frequenceA
                        )
                        res_theo <- ts(
                            c(
                                A_content,
                                create_NA_type(type = typeA, len = param1),
                                B_content
                            ),
                            start = startA,
                            frequency = frequenceA
                        )
                        testthat::expect_warning(
                            {
                                res <- extend_ts(
                                    series = ts_A,
                                    replacement = B_content,
                                    replace_na = FALSE
                                )
                            },
                            regexp = warning_extend
                        )
                        testthat::expect_equal(
                            expected = res_theo,
                            object = res
                        )
                    }

                    # Cas 3 : with without NA
                    for (param1 in list_len) {
                        ts_A <- ts(
                            c(
                                A_content,
                                create_NA_type(type = typeA, len = param1)
                            ),
                            start = startA,
                            frequency = frequenceA
                        )
                        res_theo <- ts(
                            c(A_content, B_content),
                            start = startA,
                            frequency = frequenceA
                        )

                        testthat::expect_warning(
                            {
                                res <- extend_ts(
                                    series = ts_A,
                                    replacement = B_content,
                                    date_ts = NULL,
                                    replace_na = TRUE
                                )
                            },
                            regexp = warning_extend
                        )

                        testthat::expect_equal(
                            expected = res_theo,
                            object = res
                        )
                    }

                    # Cas 3.5 : with without NA (by default)
                    for (param1 in list_len) {
                        ts_A <- ts(
                            c(
                                A_content,
                                create_NA_type(type = typeA, len = param1)
                            ),
                            start = startA,
                            frequency = frequenceA
                        )
                        res_theo <- ts(
                            c(A_content, B_content),
                            start = startA,
                            frequency = frequenceA
                        )

                        testthat::expect_warning(
                            {
                                res <- extend_ts(
                                    series = ts_A,
                                    replacement = B_content
                                )
                            },
                            regexp = warning_extend
                        )

                        testthat::expect_equal(
                            expected = res_theo,
                            object = res
                        )
                    }

                    # Cas 4 : date_ts
                    ts_A <- ts(
                        A_content,
                        start = startA,
                        frequency = frequenceA
                    )

                    for (param1 in list_len[-1L]) {
                        date_end_replacement <- as.integer(end(ts_A))
                        date_end_replacement[2L] <- date_end_replacement[2L] +
                            lenB * param1

                        date_end_replacement[1L] <- date_end_replacement[1L] +
                            (date_end_replacement[2L] - 1L) %/% frequenceA
                        date_end_replacement[2L] <- (date_end_replacement[2L] -
                            1L) %%
                            frequenceA +
                            1L

                        res_theo <- ts(
                            c(A_content, rep(B_content, param1)),
                            start = startA,
                            frequency = frequenceA
                        )

                        testthat::expect_warning(
                            {
                                res <- extend_ts(
                                    series = ts_A,
                                    replacement = B_content,
                                    date_ts = date_end_replacement,
                                    replace_na = FALSE
                                )
                            },
                            regexp = warning_extend
                        )
                        testthat::expect_equal(
                            expected = res_theo,
                            object = res
                        )
                    }

                    # Cas 5 : date_ts with with NA
                    for (param1 in list_len) {
                        ts_A <- ts(
                            c(
                                A_content,
                                create_NA_type(type = typeA, len = param1)
                            ),
                            start = startA,
                            frequency = frequenceA
                        )

                        for (param2 in list_len[-1L]) {
                            date_end_replacement <- as.integer(end(ts_A))
                            date_end_replacement[2L] <- date_end_replacement[
                                2L
                            ] +
                                lenB * param2

                            date_end_replacement[1L] <- date_end_replacement[
                                1L
                            ] +
                                (date_end_replacement[2L] - 1L) %/% frequenceA
                            date_end_replacement[2L] <- (date_end_replacement[
                                2L
                            ] -
                                1L) %%
                                frequenceA +
                                1L

                            res_theo <- ts(
                                c(
                                    A_content,
                                    create_NA_type(type = typeA, len = param1),
                                    rep(B_content, param2)
                                ),
                                start = startA,
                                frequency = frequenceA
                            )
                            testthat::expect_warning(
                                {
                                    res <- extend_ts(
                                        series = ts_A,
                                        replacement = B_content,
                                        date_ts = date_end_replacement,
                                        replace_na = FALSE
                                    )
                                },
                                regexp = warning_extend
                            )
                            testthat::expect_equal(
                                expected = res_theo,
                                object = res
                            )
                        }
                    }

                    # Cas 6 : date_ts with without NA
                    for (param1 in list_len) {
                        ts_A <- ts(
                            c(
                                A_content,
                                create_NA_type(type = typeA, len = param1)
                            ),
                            start = startA,
                            frequency = frequenceA
                        )

                        for (param2 in list_len[-1L]) {
                            date_end_replacement <- as.integer(end(ts_A))
                            date_end_replacement[2L] <- date_end_replacement[
                                2L
                            ] +
                                lenB * param2 -
                                param1

                            date_end_replacement[1L] <- date_end_replacement[
                                1L
                            ] +
                                (date_end_replacement[2L] - 1L) %/% frequenceA
                            date_end_replacement[2L] <- (date_end_replacement[
                                2L
                            ] -
                                1L) %%
                                frequenceA +
                                1L

                            res_theo <- ts(
                                c(A_content, rep(B_content, param2)),
                                start = startA,
                                frequency = frequenceA
                            )
                            testthat::expect_warning(
                                {
                                    res <- extend_ts(
                                        series = ts_A,
                                        replacement = B_content,
                                        date_ts = date_end_replacement,
                                        replace_na = TRUE
                                    )
                                },
                                regexp = warning_extend
                            )
                            testthat::expect_equal(
                                expected = res_theo,
                                object = res
                            )
                        }
                    }

                    # Cas 6.5 : date_ts with without NA (by default)
                    for (param1 in list_len) {
                        ts_A <- ts(
                            c(
                                A_content,
                                create_NA_type(type = typeA, len = param1)
                            ),
                            start = startA,
                            frequency = frequenceA
                        )

                        for (param2 in list_len[-1L]) {
                            date_end_replacement <- as.integer(end(ts_A))
                            date_end_replacement[2L] <- date_end_replacement[
                                2L
                            ] +
                                lenB * param2 -
                                param1

                            date_end_replacement[1L] <- date_end_replacement[
                                1L
                            ] +
                                (date_end_replacement[2L] - 1L) %/% frequenceA
                            date_end_replacement[2L] <- (date_end_replacement[
                                2L
                            ] -
                                1L) %%
                                frequenceA +
                                1L

                            res_theo <- ts(
                                c(A_content, rep(B_content, param2)),
                                start = startA,
                                frequency = frequenceA
                            )
                            testthat::expect_warning(
                                {
                                    res <- extend_ts(
                                        series = ts_A,
                                        replacement = B_content,
                                        date_ts = date_end_replacement
                                    )
                                },
                                regexp = warning_extend
                            )
                            testthat::expect_equal(
                                expected = res_theo,
                                object = res
                            )
                        }
                    }

                    # stop("A compléter avec des appels de extend_ts avec des arguments par défault (non précisés)")
                })
            }
        }
    }
}

# Initialisation ---------------------------------------------------------------

set.seed(2032L)


# Tests de résultats positifs = TRUE -------------------------------------------

testthat::test_that("No warning nor error with expected with good TS", {
    for (typeA in list_type) {
        for (frequenceA in list_frequence) {
            for (startA in list_start) {
                for (lenA in list_len[-1L]) {
                    ts_A <- create_random_ts(
                        type = typeA,
                        len = lenA,
                        frequency = frequenceA,
                        start = startA
                    )
                    testthat::expect_silent(assert_ts(ts_A))
                }
            }
        }
    }
})


# Tests de résultats négatif = FALSE -------------------------------------------

### Mauvais objet R ------------------------------------------------------------

testthat::test_that("Result FALSE expected with wrong object R", {
    for (wrong_ts in object_bank_R) {
        testthat::expect_error(assert_ts(wrong_ts))
    }
})

### MTS ------------------------------------------------------------------------

testthat::test_that("Result FALSE expected with mts", {
    for (typeA in list_type) {
        for (lenA in list_len[-c(1L, 9L)]) {
            for (frequenceA in list_frequence) {
                for (len2 in list_len[-c(1L, 2L, 9L)]) {
                    A_content <- as.data.frame(lapply(
                        seq_len(len2),
                        function(i) create_random_type(type = typeA, len = lenA)
                    ))
                    startA <- create_random_date_ts()
                    if (typeA == "complex") {
                        mts_A <- do.call(
                            what = cbind,
                            args = lapply(
                                X = A_content,
                                FUN = ts,
                                start = startA,
                                frequency = frequenceA
                            )
                        )
                    } else {
                        mts_A <- ts(
                            A_content,
                            start = startA,
                            frequency = frequenceA
                        )
                    }

                    testthat::expect_error(
                        assert_ts(mts_A),
                        regexp = "Variable 'mts_A': Must be of type 'atomic vector'"
                    )
                }
            }
        }
    }
})


### Mauvaise fréquence ---------------------------------------------------------

testthat::test_that("Result FALSE expected with wrong frequency", {
    for (wrong_freq in weird_frequency) {
        for (typeA in list_type) {
            for (startA in list_start) {
                for (lenA in list_len[-c(1L, 9L)]) {
                    ts_A <- create_random_ts(
                        type = typeA,
                        len = lenA,
                        frequency = wrong_freq,
                        start = startA
                    )
                    testthat::expect_error(
                        object = assert_ts(ts_A),
                        regexp = paste(
                            "Variable 'frequency_ts': Must be element of set",
                            "\\{'4','12'\\}|Variable 'frequency_ts': Must be",
                            "of type 'single integerish value'"
                        )
                    )
                }
            }
        }
    }
})


### Mauvais type ---------------------------------------------------------------

testthat::test_that("Result FALSE expected with wrong type of ts", {
    for (wrong_ts in wrong_type_ts) {
        testthat::expect_error(
            assert_ts(wrong_ts),
            regexp = "Variable 'wrong_ts': Must be of type 'atomic vector'"
        )
    }
})


### Mauvais objet R ------------------------------------------------------------

testthat::test_that("Result FALSE expected with wrong object R", {
    for (wrong_ts in object_bank_R) {
        testthat::expect_error(assert_ts(wrong_ts))
    }
})

# Initialisation ---------------------------------------------------------------

set.seed(2045L)


# Tests de résultats positifs --------------------------------------------------

testthat::test_that("good result with Date", {
    testthat::expect_silent(date2date_ts(date = Sys.Date(), frequency_ts = 4L))
    testthat::expect_silent(date2date_ts(date = Sys.Date(), frequency_ts = 12L))

    for (year in good_years[-1:-2]) {
        for (month in good_months) {
            for (day in seq_len(28L)) {
                dateA <- as.Date(
                    paste0(year, "-", month, "-", day),
                    format = "%Y-%m-%d"
                )

                testthat::expect_identical(
                    object = date2date_ts(date = dateA, frequency_ts = 12L),
                    expected = c(year, month)
                )
                testthat::expect_identical(
                    object = date2date_ts(date = dateA, frequency_ts = 4L),
                    expected = c(year, (month + 2L) %/% 3L)
                )
            }
        }
    }
})


# Test de résultat positif avec warnings ---------------------------------------

## Fréquence -------------------------------------------------------------------

testthat::test_that("warning for double frequency", {
    testthat::expect_warning(
        {
            res <- date2date_ts(date = Sys.Date(), frequency_ts = 12.0)
        },
        regexp = message_double("frequency_ts")
    )

    testthat::expect_warning(
        {
            res <- date2date_ts(date = Sys.Date(), frequency_ts = 4.0)
        },
        regexp = message_double("frequency_ts")
    )

    for (year in good_years[-1:-2]) {
        for (month in good_months) {
            for (day in seq_len(28L)) {
                dateA <- as.Date(
                    paste0(year, "-", month, "-", day),
                    format = "%Y-%m-%d"
                )

                testthat::expect_warning(
                    {
                        res <- date2date_ts(date = dateA, frequency_ts = 12.0)
                    },
                    regexp = message_double("frequency_ts")
                )
                testthat::expect_identical(res, c(year, month))

                testthat::expect_warning(
                    {
                        res <- date2date_ts(date = dateA, frequency_ts = 4.0)
                    },
                    regexp = message_double("frequency_ts")
                )
                testthat::expect_identical(res, c(year, (month + 2L) %/% 3L))
            }
        }
    }
})


# Tests de résultats négatifs --------------------------------------------------

testthat::test_that("miscellaneous date are not allowed", {
    for (wrong_date in object_bank_R[-34L]) {
        testthat::expect_error(date2date_ts(
            date = wrong_date,
            frequency_ts = 12L
        ))
    }
    for (wrong_date in object_bank_R[-34L]) {
        testthat::expect_error(date2date_ts(
            date = wrong_date,
            frequency_ts = 4L
        ))
    }
})

testthat::test_that("miscellaneous frequency are not allowed", {
    for (wrong_frequency in c(object_bank_R, weird_frequency)) {
        testthat::expect_error(date2date_ts(
            date = create_random_type(type = "Date", len = 1L),
            frequency_ts = wrong_frequency
        ))
    }
})

# Initialisation ---------------------------------------------------------------

set.seed(2044L)


# Tests de résultat avec start vecteur d'entiers -------------------------------

for (typeA in list_type) {
    for (frequenceA in list_frequence) {
        for (startA in list_start) {
            for (lenA in list_len[-1L]) {
                A_content <- create_random_type(type = typeA, len = lenA)
                for (lenB in list_len[-1L]) {
                    B_content <- create_random_type(type = typeA, len = lenB)

                    test_name <- paste0(
                        "expected result with ",
                        "\ntypeA = ",
                        deparse(typeA),
                        "\nfrequenceA = ",
                        deparse(frequenceA),
                        "\nstartA = ",
                        deparse(startA),
                        "\nlenA = ",
                        deparse(lenA),
                        "\nlenB = ",
                        deparse(lenB)
                    )

                    testthat::test_that(test_name, {
                        # Cas 1 : simple
                        ts_A <- ts(
                            A_content,
                            start = startA,
                            frequency = frequenceA
                        )
                        res_theo <- ts(
                            c(A_content, B_content),
                            start = startA,
                            frequency = frequenceA
                        )
                        testthat::expect_warning(
                            {
                                res <- extend_ts(
                                    series = ts_A,
                                    replacement = B_content,
                                    date_ts = NULL,
                                    replace_na = FALSE
                                )
                            },
                            regexp = warning_extend
                        )
                        testthat::expect_equal(
                            expected = res_theo,
                            object = res
                        )

                        # Cas 1.5 : simple (by default)
                        ts_A <- ts(
                            A_content,
                            start = startA,
                            frequency = frequenceA
                        )
                        res_theo <- ts(
                            c(A_content, B_content),
                            start = startA,
                            frequency = frequenceA
                        )
                        testthat::expect_warning(
                            {
                                res <- extend_ts(
                                    series = ts_A,
                                    replacement = B_content
                                )
                            },
                            regexp = warning_extend
                        )
                        testthat::expect_equal(
                            expected = res_theo,
                            object = res
                        )

                        # Cas 2 : with with NA
                        for (param1 in list_len) {
                            ts_A <- ts(
                                c(
                                    A_content,
                                    create_NA_type(type = typeA, len = param1)
                                ),
                                start = startA,
                                frequency = frequenceA
                            )
                            res_theo <- ts(
                                c(
                                    A_content,
                                    create_NA_type(type = typeA, len = param1),
                                    B_content
                                ),
                                start = startA,
                                frequency = frequenceA
                            )
                            testthat::expect_warning(
                                {
                                    res <- extend_ts(
                                        series = ts_A,
                                        replacement = B_content,
                                        date_ts = NULL,
                                        replace_na = FALSE
                                    )
                                },
                                regexp = warning_extend
                            )
                            testthat::expect_equal(
                                expected = res_theo,
                                object = res
                            )
                        }

                        # Cas 2.5 : with with NA (by default)
                        for (param1 in list_len) {
                            ts_A <- ts(
                                c(
                                    A_content,
                                    create_NA_type(type = typeA, len = param1)
                                ),
                                start = startA,
                                frequency = frequenceA
                            )
                            res_theo <- ts(
                                c(
                                    A_content,
                                    create_NA_type(type = typeA, len = param1),
                                    B_content
                                ),
                                start = startA,
                                frequency = frequenceA
                            )
                            testthat::expect_warning(
                                {
                                    res <- extend_ts(
                                        series = ts_A,
                                        replacement = B_content,
                                        replace_na = FALSE
                                    )
                                },
                                regexp = warning_extend
                            )
                            testthat::expect_equal(
                                expected = res_theo,
                                object = res
                            )
                        }

                        # Cas 3 : with without NA
                        for (param1 in list_len) {
                            ts_A <- ts(
                                c(
                                    A_content,
                                    create_NA_type(type = typeA, len = param1)
                                ),
                                start = startA,
                                frequency = frequenceA
                            )
                            res_theo <- ts(
                                c(A_content, B_content),
                                start = startA,
                                frequency = frequenceA
                            )

                            testthat::expect_warning(
                                {
                                    res <- extend_ts(
                                        series = ts_A,
                                        replacement = B_content,
                                        date_ts = NULL,
                                        replace_na = TRUE
                                    )
                                },
                                regexp = warning_extend
                            )

                            testthat::expect_equal(
                                expected = res_theo,
                                object = res
                            )
                        }

                        # Cas 3.5 : with without NA (by default)
                        for (param1 in list_len) {
                            ts_A <- ts(
                                c(
                                    A_content,
                                    create_NA_type(type = typeA, len = param1)
                                ),
                                start = startA,
                                frequency = frequenceA
                            )
                            res_theo <- ts(
                                c(A_content, B_content),
                                start = startA,
                                frequency = frequenceA
                            )

                            testthat::expect_warning(
                                {
                                    res <- extend_ts(
                                        series = ts_A,
                                        replacement = B_content
                                    )
                                },
                                regexp = warning_extend
                            )

                            testthat::expect_equal(
                                expected = res_theo,
                                object = res
                            )
                        }

                        # Cas 4 : date_ts
                        ts_A <- ts(
                            A_content,
                            start = startA,
                            frequency = frequenceA
                        )

                        for (param1 in list_len[-1L]) {
                            date_end_replacement <- as.integer(end(ts_A))
                            date_end_replacement[2L] <- date_end_replacement[
                                2L
                            ] +
                                lenB * param1

                            date_end_replacement[1L] <- date_end_replacement[
                                1L
                            ] +
                                (date_end_replacement[2L] - 1L) %/% frequenceA
                            date_end_replacement[2L] <- (date_end_replacement[
                                2L
                            ] -
                                1L) %%
                                frequenceA +
                                1L

                            res_theo <- ts(
                                c(A_content, rep(B_content, param1)),
                                start = startA,
                                frequency = frequenceA
                            )

                            testthat::expect_warning(
                                {
                                    res <- extend_ts(
                                        series = ts_A,
                                        replacement = B_content,
                                        date_ts = date_end_replacement,
                                        replace_na = FALSE
                                    )
                                },
                                regexp = warning_extend
                            )
                            testthat::expect_equal(
                                expected = res_theo,
                                object = res
                            )
                        }

                        # Cas 5 : date_ts with with NA
                        for (param1 in list_len) {
                            ts_A <- ts(
                                c(
                                    A_content,
                                    create_NA_type(type = typeA, len = param1)
                                ),
                                start = startA,
                                frequency = frequenceA
                            )

                            for (param2 in list_len[-1L]) {
                                date_end_replacement <- as.integer(end(ts_A))
                                date_end_replacement[
                                    2L
                                ] <- date_end_replacement[2L] + lenB * param2

                                date_end_replacement[
                                    1L
                                ] <- date_end_replacement[1L] +
                                    (date_end_replacement[2L] - 1L) %/%
                                        frequenceA
                                date_end_replacement[
                                    2L
                                ] <- (date_end_replacement[2L] - 1L) %%
                                    frequenceA +
                                    1L

                                res_theo <- ts(
                                    c(
                                        A_content,
                                        create_NA_type(
                                            type = typeA,
                                            len = param1
                                        ),
                                        rep(B_content, param2)
                                    ),
                                    start = startA,
                                    frequency = frequenceA
                                )
                                testthat::expect_warning(
                                    {
                                        res <- extend_ts(
                                            series = ts_A,
                                            replacement = B_content,
                                            date_ts = date_end_replacement,
                                            replace_na = FALSE
                                        )
                                    },
                                    regexp = warning_extend
                                )
                                testthat::expect_equal(
                                    expected = res_theo,
                                    object = res
                                )
                            }
                        }

                        # Cas 6 : date_ts with without NA
                        for (param1 in list_len) {
                            ts_A <- ts(
                                c(
                                    A_content,
                                    create_NA_type(type = typeA, len = param1)
                                ),
                                start = startA,
                                frequency = frequenceA
                            )

                            for (param2 in list_len[-1L]) {
                                date_end_replacement <- as.integer(end(ts_A))
                                date_end_replacement[
                                    2L
                                ] <- date_end_replacement[2L] +
                                    lenB * param2 -
                                    param1

                                date_end_replacement[
                                    1L
                                ] <- date_end_replacement[1L] +
                                    (date_end_replacement[2L] - 1L) %/%
                                        frequenceA
                                date_end_replacement[
                                    2L
                                ] <- (date_end_replacement[2L] - 1L) %%
                                    frequenceA +
                                    1L

                                res_theo <- ts(
                                    c(A_content, rep(B_content, param2)),
                                    start = startA,
                                    frequency = frequenceA
                                )
                                testthat::expect_warning(
                                    {
                                        res <- extend_ts(
                                            series = ts_A,
                                            replacement = B_content,
                                            date_ts = date_end_replacement,
                                            replace_na = TRUE
                                        )
                                    },
                                    regexp = warning_extend
                                )
                                testthat::expect_equal(
                                    expected = res_theo,
                                    object = res
                                )
                            }
                        }

                        # Cas 6.5 : date_ts with without NA (by default)
                        for (param1 in list_len) {
                            ts_A <- ts(
                                c(
                                    A_content,
                                    create_NA_type(type = typeA, len = param1)
                                ),
                                start = startA,
                                frequency = frequenceA
                            )

                            for (param2 in list_len[-1L]) {
                                date_end_replacement <- as.integer(end(ts_A))
                                date_end_replacement[
                                    2L
                                ] <- date_end_replacement[2L] +
                                    lenB * param2 -
                                    param1

                                date_end_replacement[
                                    1L
                                ] <- date_end_replacement[1L] +
                                    (date_end_replacement[2L] - 1L) %/%
                                        frequenceA
                                date_end_replacement[
                                    2L
                                ] <- (date_end_replacement[2L] - 1L) %%
                                    frequenceA +
                                    1L

                                res_theo <- ts(
                                    c(A_content, rep(B_content, param2)),
                                    start = startA,
                                    frequency = frequenceA
                                )
                                testthat::expect_warning(
                                    {
                                        res <- extend_ts(
                                            series = ts_A,
                                            replacement = B_content,
                                            date_ts = date_end_replacement
                                        )
                                    },
                                    regexp = warning_extend
                                )
                                testthat::expect_equal(
                                    expected = res_theo,
                                    object = res
                                )
                            }
                        }

                        # stop("A compléter avec des appels de extend_ts avec des arguments par défault (non précisés)")
                    })
                }
            }
        }
    }
}


# # Tests sur les erreurs de mts --------------------------------------------

testthat::test_that("Several dimensions are not allowed", {
    for (typeA in list_type) {
        for (frequenceA in list_frequence) {
            for (startA in list_start) {
                for (lenA in list_len[-1L]) {
                    B_content <- as.data.frame(lapply(
                        X = 1L:5L,
                        FUN = function(i) {
                            create_random_type(type = typeA, len = lenA)
                        }
                    ))

                    if (typeA == "complex") {
                        mts_B <- do.call(
                            what = cbind,
                            args = lapply(
                                X = B_content,
                                FUN = ts,
                                start = startA,
                                frequency = frequenceA
                            )
                        )
                    } else {
                        mts_B <- ts(
                            B_content,
                            start = startA,
                            frequency = frequenceA
                        )
                    }

                    testthat::expect_error(
                        extend_ts(
                            series = mts_B,
                            date_ts = create_random_date_ts(),
                            replacement = create_random_type(type = typeA)
                        ),
                        regexp = "Variable 'series': Must be of type 'atomic vector'"
                    )
                }
            }
        }
    }
})

# Tests sur les erreurs d'input ------------------------------------------------

## Test sur le ts --------------------------------------------------------------

testthat::test_that("miscellaneous series are not allowed", {
    for (typeA in list_type) {
        for (obj in object_bank_R) {
            testthat::expect_error(extend_ts(
                series = obj,
                date_ts = create_random_date_ts(),
                replacement = create_random_type(type = typeA)
            ))
        }
    }
})

## Test sur la date ------------------------------------------------------------

testthat::test_that("miscellaneous date are not allowed", {
    for (typeA in list_type) {
        for (wrong_date in list_wrong_date_ts[-46]) {
            testthat::expect_error(extend_ts(
                series = create_random_ts(type = typeA),
                date_ts = wrong_date,
                replacement = create_random_type(type = typeA)
            ))
        }
    }
})

## Test sur le vecteur value ---------------------------------------------------

testthat::test_that("miscellaneous value input are not allowed", {
    list_wrong_value <- c(
        fuzzr::test_df()[-4L],
        NULL,
        character(0L),
        numeric(0L),
        logical(0L),
        integer(0L),
        complex(0L)
    )
    for (typeA in list_type) {
        for (value in list_wrong_value) {
            testthat::expect_error(extend_ts(
                series = create_random_ts(type = typeA),
                date_ts = create_random_date_ts(),
                replacement = value
            ))
            testthat::expect_error(extend_ts(
                series = create_random_ts(type = typeA),
                date_ts = NULL,
                replacement = value
            ))
        }
    }
})

testthat::test_that("value should have same type as series", {
    for (typeA in list_type[-7L]) {
        for (typeB in list_type[-7L]) {
            if (typeA != typeB) {
                testthat::expect_error(
                    extend_ts(
                        series = create_random_ts(type = typeA),
                        date_ts = NULL,
                        replacement = create_random_type(typeB)
                    ),
                    regexp = "Les objets `series` et `replacement` doivent \u00eatre de m\u00eame type."
                )
            }
        }
    }
})

# Tests sur les erreurs de temporalité --------------------------------------------

testthat::test_that("series and date are temporally consistent", {
    for (wrong_timeunits in c(
        object_bank_R[-c(10L, 16L)],
        list_wrong_timeunits
    )) {
        for (typeA in list_type) {
            # Monthly
            testthat::expect_error(extend_ts(
                series = create_random_ts(
                    type = typeA,
                    start = wrong_timeunits,
                    frequency = 12L
                ),
                date_ts = create_random_date_ts(),
                replacement = create_random_type(type = typeA)
            ))
            testthat::expect_error(extend_ts(
                series = create_random_ts(
                    type = typeA,
                    start = wrong_timeunits,
                    frequency = 12L
                ),
                date_ts = NULL,
                replacement = create_random_type(type = typeA)
            ))
            testthat::expect_error(extend_ts(
                series = create_random_ts(
                    type = typeA,
                    start = wrong_timeunits,
                    frequency = 12L
                ),
                replacement = create_random_type(type = typeA)
            ))

            # Quaterly
            testthat::expect_error(extend_ts(
                series = create_random_ts(
                    type = typeA,
                    start = wrong_timeunits,
                    frequency = 4L
                ),
                date_ts = create_random_date_ts(),
                replacement = create_random_type(type = typeA)
            ))
            testthat::expect_error(extend_ts(
                series = create_random_ts(
                    type = typeA,
                    start = wrong_timeunits,
                    frequency = 4L
                ),
                date_ts = NULL,
                replacement = create_random_type(type = typeA)
            ))
            testthat::expect_error(extend_ts(
                series = create_random_ts(
                    type = typeA,
                    start = wrong_timeunits,
                    frequency = 4L
                ),
                replacement = create_random_type(type = typeA)
            ))
        }
    }
})

testthat::test_that("date_ts must be after end_ts", {
    for (typeA in list_type) {
        # Monthly
        testthat::expect_error(extend_ts(
            series = create_random_ts(
                type = typeA,
                start = 2010L,
                frequency = 12L,
                len = 10L
            ),
            date_ts = 2010L,
            replacement = create_random_type(type = typeA)
        ))

        # Quaterly
        testthat::expect_error(extend_ts(
            series = create_random_ts(
                type = typeA,
                start = 2022L,
                frequency = 4L,
                len = 10L
            ),
            date_ts = 2023L,
            replacement = create_random_type(type = typeA)
        ))
    }
})

# Initialisation ---------------------------------------------------------------

set.seed(2041L)


# Tests de résultat avec start vecteur d'entiers -------------------------------

for (typeA in list_type[-6L]) {
    for (lenA in list_len[-1L]) {
        A_content <- create_random_type(type = typeA, len = lenA)
        for (frequenceA in list_frequence) {
            for (startA in list_start) {
                ts_without_na <- ts(
                    A_content,
                    start = startA,
                    frequency = frequenceA
                )
                for (param1 in list_len) {
                    test_name <- paste0(
                        "expected result with ",
                        "\ntypeA = '",
                        typeA,
                        "'\nfrequenceA = ",
                        frequenceA,
                        "\nstartA = ",
                        deparse(startA),
                        "\nlenA = ",
                        lenA,
                        "\nparam1 = ",
                        param1
                    )

                    testthat::test_that(desc = test_name, {
                        # Cas 1
                        ts_with_na <- ts(
                            c(
                                A_content,
                                create_NA_type(type = typeA, len = param1)
                            ),
                            start = startA,
                            frequency = frequenceA
                        )
                        res <- testthat::expect_silent(na_trim(ts_with_na))
                        testthat::expect_equal(
                            object = res,
                            expected = ts_without_na
                        )

                        # Cas 2
                        ts_with_na <- ts(
                            c(
                                create_NA_type(type = typeA, len = param1),
                                A_content
                            ),
                            end = end(ts_without_na),
                            frequency = frequenceA
                        )
                        res <- testthat::expect_silent(na_trim(ts_with_na))
                        testthat::expect_equal(
                            object = res,
                            expected = ts_without_na
                        )

                        # Cas 3
                        for (param2 in list_len[2]) {
                            ts_with_na <- ts(
                                c(
                                    create_NA_type(type = typeA, len = param1),
                                    A_content,
                                    create_NA_type(type = typeA, len = param2)
                                ),
                                start = startA,
                                frequency = frequenceA
                            )

                            if (length(startA) == 1L) {
                                startA <- c(startA, 1L)
                            }
                            startA[2L] <- startA[2L] + param1
                            ts_without_na <- ts(
                                A_content,
                                start = c(startA, param1),
                                frequency = frequenceA
                            )

                            res <- testthat::expect_silent(na_trim(ts_with_na))
                            testthat::expect_equal(
                                object = res,
                                expected = ts_without_na
                            )
                        }
                    })
                }
            }
        }
    }
}


# Tests sur les erreurs de mts --------------------------------------------

testthat::test_that("Several dimensions are not allowed", {
    for (typeA in list_type) {
        for (frequenceA in list_frequence) {
            for (startA in list_start) {
                for (lenA in list_len[-1L]) {
                    B_content <- as.data.frame(lapply(
                        X = 1L:5L,
                        FUN = function(i) {
                            create_random_type(type = typeA, len = lenA)
                        }
                    ))

                    if (typeA == "complex") {
                        mts_B <- do.call(
                            what = cbind,
                            args = lapply(
                                X = B_content,
                                FUN = ts,
                                start = startA,
                                frequency = frequenceA
                            )
                        )
                    } else {
                        mts_B <- ts(
                            B_content,
                            start = startA,
                            frequency = frequenceA
                        )
                    }

                    testthat::expect_error(
                        na_trim(series = mts_B),
                        regexp = "Variable 'series': Must be of type 'atomic vector'"
                    )
                }
            }
        }
    }
})


# Tests sur les erreurs d'input ------------------------------------------------

## Test sur le ts --------------------------------------------------------------

testthat::test_that("miscellaneous series are not allowed", {
    for (obj in object_bank_R) {
        testthat::expect_error(na_trim(series = obj))
    }
})


# Tests sur les erreurs de temporalité --------------------------------------------

testthat::test_that("series and date are temporally consistent", {
    for (typeA in list_type) {
        testthat::expect_error(na_trim(
            series = create_random_ts(
                type = typeA,
                start = 2010 + 1 / 7,
                frequency = 12L
            )
        ))
    }

    for (typeA in list_type) {
        testthat::expect_error(na_trim(
            series = create_random_ts(
                type = typeA,
                start = 2022 + 1 / 5,
                frequency = 4L
            )
        ))
    }
})

# Initialisation ---------------------------------------------------------------

set.seed(2046L)


# Tests de résultats positifs --------------------------------------------------

testthat::test_that("good result with Date", {
    for (n in list_len[-1L]) {
        testthat::expect_silent(substr_year(date = Sys.Date(), n = n))
    }

    for (year in good_years[-1L:-2L]) {
        for (month in good_months) {
            for (day in seq_len(28L)) {
                dateA <- as.Date(
                    paste0(year, "-", month, "-", day),
                    format = "%Y-%m-%d"
                )

                for (n in list_len[-1L]) {
                    if (year - n >= 0) {
                        date_theo <- as.Date(
                            paste0(year - n, "-", month, "-", day),
                            format = "%Y-%m-%d"
                        )
                        testthat::expect_identical(
                            object = substr_year(date = dateA, n = n),
                            expected = date_theo
                        )
                    }
                }
            }
        }
    }
})


# Test de résultat positif avec warnings ---------------------------------------

## n ---------------------------------------------------------------------------

testthat::test_that("good result with warning n", {
    for (n in list_len[-1L]) {
        n <- as.double(n)
        testthat::expect_warning(
            {
                res <- substr_year(date = Sys.Date(), n = n)
            },
            regexp = message_double("n")
        )
    }

    for (year in good_years[-1L:-2L]) {
        for (month in good_months) {
            for (day in seq_len(28L)) {
                dateA <- as.Date(
                    paste0(year, "-", month, "-", day),
                    format = "%Y-%m-%d"
                )

                for (n in list_len[-1L]) {
                    if (year - n >= 0L) {
                        n <- as.double(n)

                        date_theo <- as.Date(
                            paste0(year - n, "-", month, "-", day),
                            format = "%Y-%m-%d"
                        )
                        testthat::expect_warning(
                            {
                                res <- substr_year(date = dateA, n = n)
                            },
                            regexp = message_double("n")
                        )
                        testthat::expect_identical(res, date_theo)
                    }
                }
            }
        }
    }
})

# Tests de résultats négatifs --------------------------------------------------

testthat::test_that("miscellaneous date are not allowed", {
    for (wrong_date in object_bank_R[-34L]) {
        testthat::expect_error(substr_year(date = wrong_date, n = 12L))
    }
    for (wrong_date in object_bank_R[-34L]) {
        testthat::expect_error(substr_year(date = wrong_date, n = 4L))
    }
})

testthat::test_that("miscelanous n are not accepted", {
    list_wrong_n <- c(
        list(0., 0L),
        list_wrong_date_ts,
        object_bank_R[-10L],
        rnorm(10L),
        as.double(
            -abs(c(
                list_lag,
                list_len,
                create_random_type("integer", len = 10L)
            ))
        ),
        -abs(c(list_len, list_lag, create_random_type("integer", len = 10L)))
    )

    index_error_warning_n <- c(
        1L,
        17L:18L,
        35L:36L,
        46L,
        59L,
        62L,
        66L:72L,
        81L,
        88L:97L,
        112L:113L,
        130L:131L,
        141L,
        144L:181L
    )
    index_error_n <- setdiff(seq_along(list_wrong_n), index_error_warning_n)

    # Just errors
    for (wrong_n in list_wrong_n[index_error_n]) {
        testthat::expect_error(substr_year(
            date = create_random_type(type = "Date", len = 1L),
            n = wrong_n
        ))
    }

    # Errors + Warnings
    for (wrong_n in list_wrong_n[index_error_warning_n]) {
        testthat::expect_warning(
            testthat::expect_error(
                substr_year(
                    date = create_random_type(type = "Date", len = 1L),
                    n = wrong_n
                )
            )
        )
    }
})

# Initialisation ---------------------------------------------------------------

set.seed(2040L)


# Tests de résultat ------------------------------------------------------------

for (typeA in list_type[-c(5L, 7L)]) {
    for (frequenceA in list_frequence) {
        for (startA in list_start) {
            for (lenA in list_len[-1L]) {
                A_content <- create_random_type(type = typeA, len = lenA)
                ts_A <- ts(A_content, start = startA, frequency = frequenceA)
                for (param1 in list_len) {
                    for (param2 in list_len) {
                        test_name <- paste0(
                            "expected result with ",
                            "\ntypeA = '",
                            typeA,
                            "'\nfrequenceA = ",
                            frequenceA,
                            "\nstartA = ",
                            deparse(startA),
                            "\nlenA = ",
                            lenA,
                            "\nparam1 = ",
                            param1,
                            "\nparam2 = ",
                            param2
                        )

                        testthat::test_that(desc = test_name, {
                            # Cas 1
                            if (param1 <= lenA & param1 > 0L) {
                                date_from_1 <- end(ts_A) |> as.integer()
                                if (length(date_from_1) == 1L) {
                                    date_from_1 <- c(date_from_1, 1L)
                                }
                                date_from_1[2L] <- date_from_1[2L] -
                                    (param1 - 1L)
                                date_from_1 <- c(
                                    date_from_1[1L] +
                                        (date_from_1[2L] - 1L) %/% frequenceA,
                                    (date_from_1[2L] - 1L) %% frequenceA + 1L
                                )

                                date_to_1 <- end(ts_A) |> as.integer()
                                if (length(date_to_1) == 1L) {
                                    date_to_1 <- c(date_to_1, 1L)
                                }
                                date_to_1[2L] <- date_to_1[2L] + param2
                                date_to_1 <- c(
                                    date_to_1[1L] +
                                        (date_to_1[2L] - 1L) %/% frequenceA,
                                    (date_to_1[2L] - 1L) %% frequenceA + 1L
                                )

                                res_theo_1 <- c(
                                    A_content[(lenA - param1 + 1L):lenA],
                                    create_NA_type(type = typeA, len = param2)
                                )

                                vect_res1 <- get_value_ts(
                                    series = ts_A,
                                    date_from = date_from_1,
                                    date_to = date_to_1
                                )
                                vect_res2 <- get_value_ts(
                                    series = ts_A,
                                    date_from = date_from_1,
                                    n = param1 + param2
                                )
                                vect_res3 <- get_value_ts(
                                    series = ts_A,
                                    date_to = date_to_1,
                                    n = param1 + param2
                                )

                                testthat::expect_identical(
                                    res_theo_1,
                                    vect_res1
                                )
                                testthat::expect_identical(
                                    res_theo_1,
                                    vect_res2
                                )
                                testthat::expect_identical(
                                    res_theo_1,
                                    vect_res3
                                )
                            }

                            # Cas 2
                            if (param2 <= lenA & param2 > 0L) {
                                date_from_2 <- start(ts_A) |> as.integer()
                                if (length(date_from_2) == 1L) {
                                    date_from_2 <- c(date_from_2, 1L)
                                }
                                date_from_2[2L] <- date_from_2[2L] - param1
                                date_from_2 <- c(
                                    date_from_2[1L] +
                                        (date_from_2[2L] - 1L) %/% frequenceA,
                                    (date_from_2[2L] - 1L) %% frequenceA + 1L
                                )

                                date_to_2 <- start(ts_A) |> as.integer()
                                if (length(date_to_2) == 1L) {
                                    date_to_2 <- c(date_to_2, 1L)
                                }
                                date_to_2[2L] <- date_to_2[2L] + param2 - 1L
                                date_to_2 <- c(
                                    date_to_2[1L] +
                                        (date_to_2[2L] - 1L) %/% frequenceA,
                                    (date_to_2[2L] - 1L) %% frequenceA + 1L
                                )

                                res_theo_2 <- c(
                                    create_NA_type(type = typeA, len = param1),
                                    A_content[1:param2]
                                )

                                vect_res1 <- get_value_ts(
                                    ts_A,
                                    date_from = date_from_2,
                                    date_to = date_to_2
                                )
                                vect_res2 <- get_value_ts(
                                    ts_A,
                                    date_from = date_from_2,
                                    n = param1 + param2
                                )
                                vect_res3 <- get_value_ts(
                                    ts_A,
                                    date_to = date_to_2,
                                    n = param1 + param2
                                )

                                testthat::expect_identical(
                                    res_theo_2,
                                    vect_res1
                                )
                                testthat::expect_identical(
                                    res_theo_2,
                                    vect_res2
                                )
                                testthat::expect_identical(
                                    res_theo_2,
                                    vect_res3
                                )
                            }

                            # Cas 3
                            if (param1 > 0L) {
                                date_from_3 <- start(ts_A) |> as.integer()
                                if (length(date_from_3) == 1L) {
                                    date_from_3 <- c(date_from_3, 1L)
                                }
                                date_from_3[2L] <- date_from_3[2L] -
                                    param1 -
                                    param2
                                date_from_3 <- c(
                                    date_from_3[1L] +
                                        (date_from_3[2L] - 1L) %/% frequenceA,
                                    (date_from_3[2L] - 1L) %% frequenceA + 1L
                                )

                                date_to_3 <- start(ts_A) |> as.integer()
                                if (length(date_to_3) == 1L) {
                                    date_to_3 <- c(date_to_3, 1L)
                                }
                                date_to_3[2L] <- date_to_3[2L] - param2 - 1L
                                date_to_3 <- c(
                                    date_to_3[1L] +
                                        (date_to_3[2L] - 1L) %/% frequenceA,
                                    (date_to_3[2L] - 1L) %% frequenceA + 1L
                                )

                                res_theo_3 <- create_NA_type(
                                    type = typeA,
                                    len = param1
                                )

                                vect_res1 <- get_value_ts(
                                    ts_A,
                                    date_from = date_from_3,
                                    date_to = date_to_3
                                )
                                vect_res2 <- get_value_ts(
                                    ts_A,
                                    date_from = date_from_3,
                                    n = param1
                                )
                                vect_res3 <- get_value_ts(
                                    ts_A,
                                    date_to = date_to_3,
                                    n = param1
                                )

                                testthat::expect_identical(
                                    res_theo_3,
                                    vect_res1
                                )
                                testthat::expect_identical(
                                    res_theo_3,
                                    vect_res2
                                )
                                testthat::expect_identical(
                                    res_theo_3,
                                    vect_res3
                                )
                            }

                            # Cas 4
                            if (param2 > 0L) {
                                date_from_4 <- end(ts_A) |> as.integer()
                                if (length(date_from_4) == 1L) {
                                    date_from_4 <- c(date_from_4, 1L)
                                }
                                date_from_4[2L] <- date_from_4[2L] + param1 + 1L
                                date_from_4 <- c(
                                    date_from_4[1L] +
                                        (date_from_4[2L] - 1L) %/% frequenceA,
                                    (date_from_4[2L] - 1L) %% frequenceA + 1L
                                )

                                date_to_4 <- end(ts_A) |> as.integer()
                                if (length(date_to_4) == 1L) {
                                    date_to_4 <- c(date_to_4, 1L)
                                }
                                date_to_4[2L] <- date_to_4[2L] + param1 + param2
                                date_to_4 <- c(
                                    date_to_4[1L] +
                                        (date_to_4[2L] - 1L) %/% frequenceA,
                                    (date_to_4[2L] - 1L) %% frequenceA + 1L
                                )

                                res_theo_4 <- create_NA_type(
                                    type = typeA,
                                    len = param2
                                )

                                vect_res1 <- get_value_ts(
                                    ts_A,
                                    date_from = date_from_4,
                                    date_to = date_to_4
                                )
                                vect_res2 <- get_value_ts(
                                    ts_A,
                                    date_from = date_from_4,
                                    n = param2
                                )
                                vect_res3 <- get_value_ts(
                                    ts_A,
                                    date_to = date_to_4,
                                    n = param2
                                )

                                testthat::expect_identical(
                                    res_theo_4,
                                    vect_res1
                                )
                                testthat::expect_identical(
                                    res_theo_4,
                                    vect_res2
                                )
                                testthat::expect_identical(
                                    res_theo_4,
                                    vect_res3
                                )
                            }

                            # Cas 5
                            date_from_5 <- start(ts_A) |> as.integer()
                            if (length(date_from_5) == 1L) {
                                date_from_5 <- c(date_from_5, 1L)
                            }
                            date_from_5[2L] <- date_from_5[2L] - param1
                            date_from_5 <- c(
                                date_from_5[1L] +
                                    (date_from_5[2L] - 1L) %/% frequenceA,
                                (date_from_5[2L] - 1L) %% frequenceA + 1L
                            )

                            date_to_5 <- end(ts_A) |> as.integer()
                            if (length(date_to_5) == 1L) {
                                date_to_5 <- c(date_to_5, 1L)
                            }
                            date_to_5[2L] <- date_to_5[2L] + param2
                            date_to_5 <- c(
                                date_to_5[1L] +
                                    (date_to_5[2L] - 1L) %/% frequenceA,
                                (date_to_5[2L] - 1L) %% frequenceA + 1L
                            )

                            res_theo_5 <- c(
                                create_NA_type(type = typeA, len = param1),
                                A_content,
                                create_NA_type(type = typeA, len = param2)
                            )

                            vect_res1 <- get_value_ts(
                                ts_A,
                                date_from = date_from_5,
                                date_to = date_to_5
                            )
                            vect_res2 <- get_value_ts(
                                ts_A,
                                date_from = date_from_5,
                                n = param2 + lenA + param1
                            )
                            vect_res3 <- get_value_ts(
                                ts_A,
                                date_to = date_to_5,
                                n = param2 + lenA + param1
                            )

                            testthat::expect_identical(res_theo_5, vect_res1)
                            testthat::expect_identical(res_theo_5, vect_res2)
                            testthat::expect_identical(res_theo_5, vect_res3)

                            # Cas 6
                            if (param1 + param2 <= lenA && param2 > 0) {
                                date_from_6 <- start(ts_A) |> as.integer()
                                if (length(date_from_6) == 1L) {
                                    date_from_6 <- c(date_from_6, 1L)
                                }
                                date_from_6[2L] <- date_from_6[2L] + param1
                                date_from_6 <- c(
                                    date_from_6[1L] +
                                        (date_from_6[2L] - 1L) %/% frequenceA,
                                    (date_from_6[2L] - 1L) %% frequenceA + 1L
                                )

                                date_to_6 <- start(ts_A) |> as.integer()
                                if (length(date_to_6) == 1L) {
                                    date_to_6 <- c(date_to_6, 1L)
                                }
                                date_to_6[2L] <- date_to_6[2L] +
                                    param1 +
                                    param2 -
                                    1L
                                date_to_6 <- c(
                                    date_to_6[1L] +
                                        (date_to_6[2L] - 1L) %/% frequenceA,
                                    (date_to_6[2L] - 1L) %% frequenceA + 1L
                                )

                                res_theo_6 <- A_content[
                                    (param1 + 1L):(param1 + param2)
                                ]

                                vect_res1 <- get_value_ts(
                                    ts_A,
                                    date_from = date_from_6,
                                    date_to = date_to_6
                                )
                                vect_res2 <- get_value_ts(
                                    ts_A,
                                    date_from = date_from_6,
                                    n = param2
                                )
                                vect_res3 <- get_value_ts(
                                    ts_A,
                                    date_to = date_to_6,
                                    n = param2
                                )

                                testthat::expect_identical(
                                    res_theo_6,
                                    vect_res1
                                )
                                testthat::expect_identical(
                                    res_theo_6,
                                    vect_res2
                                )
                                testthat::expect_identical(
                                    res_theo_6,
                                    vect_res3
                                )
                            }
                        })
                    }
                }
            }
        }
    }
}


# Tests sur les erreurs de mts -------------------------------------------------

testthat::test_that("Several dimensions are not allowed", {
    for (typeA in list_type) {
        for (frequenceA in list_frequence) {
            for (startA in list_start) {
                for (lenA in list_len[-1L]) {
                    A_content <- as.data.frame(lapply(
                        1L:5L,
                        function(i) create_random_type(type = typeA, len = 100L)
                    ))
                    startA <- create_random_date_ts()

                    if (typeA == "complex") {
                        mts_A <- do.call(
                            what = cbind,
                            args = lapply(
                                X = A_content,
                                FUN = ts,
                                start = startA,
                                frequency = frequenceA
                            )
                        )
                    } else {
                        mts_A <- ts(
                            A_content,
                            start = startA,
                            frequency = frequenceA
                        )
                    }

                    testthat::expect_error(
                        object = get_value_ts(
                            series = mts_A,
                            date_from = startA,
                            date_to = end(mts_A) |> as.integer()
                        ),
                        regexp = "Variable 'series': Must be of type 'atomic vector'"
                    )
                    testthat::expect_error(
                        object = get_value_ts(
                            series = mts_A,
                            date_from = startA,
                            n = lenA
                        ),
                        regexp = "Variable 'series': Must be of type 'atomic vector'"
                    )
                    testthat::expect_error(
                        object = get_value_ts(
                            series = mts_A,
                            date_to = end(mts_A) |> as.integer(),
                            n = lenA
                        ),
                        regexp = "Variable 'series': Must be of type 'atomic vector'"
                    )
                }
            }
        }
    }
})

# Tests sur les erreurs d'input ------------------------------------------------

testthat::test_that("miscellaneous input are not allowed", {
    date_1 <- 2000L
    date_2 <- 2001L
    len <- 15L

    for (objA in object_bank_R) {
        testthat::expect_error(
            object = get_value_ts(
                series = objA,
                date_from = date_1,
                date_to = date_2
            )
        )
        testthat::expect_error(
            object = get_value_ts(series = objA, date_from = date_1, n = len)
        )
        testthat::expect_error(
            object = get_value_ts(series = objA, date_to = date_2, n = len)
        )
    }
})

testthat::test_that("3 argyuments input are not allowed", {
    for (typeA in list_type[-7]) {
        for (frequenceA in list_frequence) {
            ts_A <- create_random_ts(type = typeA, frequency = frequenceA)
            testthat::expect_error(
                object = get_value_ts(
                    series = ts_A,
                    date_from = create_random_date_ts(),
                    date_to = create_random_date_ts(),
                    n = create_random_type(type = "integer", len = 1L)
                )
            )
        }
    }
})

# Tests sur les erreurs de temporalité -----------------------------------------

testthat::test_that("date_from must be before date_to", {
    for (year in good_years) {
        for (len in list_len[-1L]) {
            ts_A <- create_random_ts(frequency = 12L)

            date_1 <- year
            date_2 <- year + len
            testthat::expect_error(get_value_ts(
                ts_A,
                date_from = date_2,
                date_to = date_1
            ))

            for (month in good_months) {
                date_1 <- c(year, month)
                date_2 <- date_1
                date_2[1L] <- date_2[1L] + len
                date_2 <- c(
                    date_2[1L] + (date_2[2L] - 1L) %/% 12L,
                    (date_2[2L] - 1L) %% 12L + 1L
                )
                testthat::expect_error(get_value_ts(
                    ts_A,
                    date_from = date_2,
                    date_to = date_1
                ))
            }

            ts_A <- create_random_ts(frequency = 4L)

            for (quarter in good_quarters) {
                date_1 <- c(year, quarter)
                date_2 <- date_1
                date_2[1L] <- date_2[1L] + len
                date_2 <- c(
                    date_2[1L] + (date_2[2L] - 1L) %/% 4L,
                    (date_2[2L] - 1L) %% 4L + 1L
                )
                testthat::expect_error(get_value_ts(
                    ts_A,
                    date_from = date_2,
                    date_to = date_1
                ))
            }
        }
    }
})

testthat::test_that("n is non positive", {
    for (len in list_len) {
        ts_A <- create_random_ts()
        testthat::expect_error(
            object = get_value_ts(
                series = ts_A,
                date_from = create_random_date_ts(),
                n = -abs(len)
            )
        )
        testthat::expect_error(
            object = get_value_ts(
                series = ts_A,
                date_to = create_random_date_ts(),
                n = -abs(len)
            )
        )
    }
})

testthat::test_that("miscelanous date are not accepted", {
    for (wrong_date_ts in list_wrong_date_ts) {
        ts_A <- create_random_ts()

        testthat::expect_error(
            object = get_value_ts(
                series = ts_A,
                date_to = wrong_date_ts,
                n = 10L
            )
        )

        testthat::expect_error(
            object = get_value_ts(
                series = ts_A,
                date_from = wrong_date_ts,
                n = 10L
            )
        )

        testthat::expect_error(
            object = get_value_ts(
                series = ts_A,
                date_from = wrong_date_ts,
                date_to = create_random_date_ts(frequency_ts = frequency(ts_A))
            )
        )

        testthat::expect_error(
            object = get_value_ts(
                series = ts_A,
                date_from = create_random_date_ts(
                    frequency_ts = frequency(ts_A)
                ),
                date_to = wrong_date_ts
            )
        )
    }
})

testthat::test_that("miscelanous n are not accepted", {
    for (wrong_n in c(
        list(0., 0L),
        list_wrong_date_ts,
        object_bank_R[-10L],
        rnorm(10L),
        as.double(
            -abs(c(
                list_lag,
                list_len,
                create_random_type("integer", len = 10L)
            ))
        ),
        -abs(c(list_len, list_lag, create_random_type("integer", len = 10L)))
    )) {
        ts_A <- create_random_ts()

        testthat::expect_error(
            object = get_value_ts(
                series = ts_A,
                date_to = create_random_date_ts(frequency_ts = frequency(ts_A)),
                n = wrong_n
            )
        )

        testthat::expect_error(
            object = get_value_ts(
                series = ts_A,
                date_from = create_random_date_ts(
                    frequency_ts = frequency(ts_A)
                ),
                n = wrong_n
            )
        )
    }
})

# Initialisation ---------------------------------------------------------------

set.seed(2042L)


# Tests de résultats positifs --------------------------------------------------

testthat::test_that(desc = "Expect good result with monthly frequency", {
    for (year in good_years) {
        for (lag in list_lag) {
            date_a <- year

            minus_year <- (-lag) %/% 12L
            minus_months <- (-lag) %% 12L + 1L

            date_b <- c(year + minus_year, minus_months)

            testthat::expect_identical(
                object = previous_date_ts(
                    date_a,
                    frequency_ts = 12L,
                    lag = lag
                ),
                expected = date_b
            )

            for (month in good_months) {
                date_a <- c(year, month)

                minus_year <- (month - lag - 1L) %/% 12L
                minus_months <- (month - lag - 1L) %% 12L + 1L

                date_b <- c(year + minus_year, minus_months)

                testthat::expect_identical(
                    object = previous_date_ts(
                        date_a,
                        frequency_ts = 12L,
                        lag = lag
                    ),
                    expected = date_b
                )
            }
        }
    }
})

testthat::test_that(desc = "Expect good result with quaterly frequency", {
    for (year in good_years) {
        for (lag in list_lag) {
            date_a <- year

            minus_year <- (-lag) %/% 4L
            minus_months <- (-lag) %% 4L + 1L

            date_b <- c(year + minus_year, minus_months)

            testthat::expect_identical(
                object = previous_date_ts(date_a, frequency_ts = 4L, lag = lag),
                expected = date_b
            )

            for (quarter in good_quarters) {
                date_a <- c(year, quarter)

                minus_year <- (quarter - lag - 1L) %/% 4L
                minus_quarters <- (quarter - lag - 1L) %% 4L + 1L

                date_b <- c(year + minus_year, minus_quarters)

                testthat::expect_identical(
                    object = previous_date_ts(
                        date_a,
                        frequency_ts = 4L,
                        lag = lag
                    ),
                    expected = date_b
                )
            }
        }
    }
})


# Tests de résultats positif avec warning --------------------------------------

testthat::test_that(desc = "Warning with double years and monthly frequency", {
    for (year in double_years) {
        for (lag in list_lag) {
            date_a <- year

            minus_year <- (-lag) %/% 12L
            minus_months <- (-lag) %% 12L + 1L

            date_b <- c(year + minus_year, minus_months) |> as.integer()

            testthat::expect_warning(
                object = {
                    res <- previous_date_ts(
                        date_a,
                        frequency_ts = 12L,
                        lag = lag
                    )
                },
                regexp = message_double("date_ts")
            )

            testthat::expect_identical(
                object = res,
                expected = date_b
            )

            for (month in good_months) {
                date_a <- c(year, month)

                minus_year <- (month - lag - 1L) %/% 12L
                minus_months <- (month - lag - 1L) %% 12L + 1L

                date_b <- c(year + minus_year, minus_months) |> as.integer()

                testthat::expect_warning(
                    object = {
                        res <- previous_date_ts(
                            date_a,
                            frequency_ts = 12L,
                            lag = lag
                        )
                    },
                    regexp = message_double("date_ts")
                )

                testthat::expect_identical(
                    object = res,
                    expected = date_b
                )
            }
        }
    }
})

testthat::test_that(desc = "Warning with double years and quaterly frequency", {
    for (year in double_years) {
        for (lag in list_lag) {
            date_a <- year

            minus_year <- (-lag) %/% 4L
            minus_months <- (-lag) %% 4L + 1L

            date_b <- c(year + minus_year, minus_months) |> as.integer()

            testthat::expect_warning(
                object = {
                    res <- previous_date_ts(
                        date_a,
                        frequency_ts = 4L,
                        lag = lag
                    )
                },
                regexp = message_double("date_ts")
            )

            testthat::expect_identical(
                object = res,
                expected = date_b
            )

            for (quarter in good_quarters) {
                date_a <- c(year, quarter)

                minus_year <- (quarter - lag - 1L) %/% 4L
                minus_quarters <- (quarter - lag - 1L) %% 4L + 1L

                date_b <- c(year + minus_year, minus_quarters) |> as.integer()

                testthat::expect_warning(
                    object = {
                        res <- previous_date_ts(
                            date_a,
                            frequency_ts = 4L,
                            lag = lag
                        )
                    },
                    regexp = message_double("date_ts")
                )

                testthat::expect_identical(
                    object = res,
                    expected = date_b
                )
            }
        }
    }
})

testthat::test_that(desc = "Warning with frequency and monthly frequency", {
    for (year in good_years) {
        for (lag in list_lag) {
            date_a <- year

            minus_year <- (-lag) %/% 12L
            minus_months <- (-lag) %% 12L + 1L

            date_b <- c(year + minus_year, minus_months) |> as.integer()

            testthat::expect_warning(
                object = {
                    res <- previous_date_ts(
                        date_a,
                        frequency_ts = 12,
                        lag = lag
                    )
                },
                regexp = message_double("frequency_ts")
            )

            testthat::expect_identical(
                object = res,
                expected = date_b
            )

            for (month in good_months) {
                date_a <- c(year, month)

                minus_year <- (month - lag - 1L) %/% 12L
                minus_months <- (month - lag - 1L) %% 12L + 1L

                date_b <- c(year + minus_year, minus_months) |> as.integer()

                testthat::expect_warning(
                    object = {
                        res <- previous_date_ts(
                            date_a,
                            frequency_ts = 12,
                            lag = lag
                        )
                    },
                    regexp = message_double("frequency_ts")
                )

                testthat::expect_identical(
                    object = res,
                    expected = date_b
                )
            }
        }
    }
})

testthat::test_that(desc = "Warning with frequency and quaterly frequency", {
    for (year in good_years) {
        for (lag in list_lag) {
            date_a <- year

            minus_year <- (-lag) %/% 4L
            minus_months <- (-lag) %% 4L + 1L

            date_b <- c(year + minus_year, minus_months) |> as.integer()

            testthat::expect_warning(
                object = {
                    res <- previous_date_ts(date_a, frequency_ts = 4, lag = lag)
                },
                regexp = message_double("frequency_ts")
            )

            testthat::expect_identical(
                object = res,
                expected = date_b
            )

            for (quarter in good_quarters) {
                date_a <- c(year, quarter)

                minus_year <- (quarter - lag - 1L) %/% 4L
                minus_quarters <- (quarter - lag - 1L) %% 4L + 1L

                date_b <- c(year + minus_year, minus_quarters) |> as.integer()

                testthat::expect_warning(
                    object = {
                        res <- previous_date_ts(
                            date_a,
                            frequency_ts = 4,
                            lag = lag
                        )
                    },
                    regexp = message_double("frequency_ts")
                )

                testthat::expect_identical(
                    object = res,
                    expected = date_b
                )
            }
        }
    }
})

# Tests de résultats négatifs (erreur) -----------------------------------------

testthat::test_that("miscellaneous date are not allowed", {
    for (wrong_date in list_wrong_date_ts) {
        testthat::expect_error(previous_date_ts(
            date_ts = wrong_date,
            frequency_ts = 12L,
            lag = 4L
        ))
        testthat::expect_error(previous_date_ts(
            date_ts = wrong_date,
            frequency_ts = 12L,
            lag = 4L
        ))
    }
})

testthat::test_that("miscellaneous frequency are not allowed", {
    for (wrong_frequency in c(object_bank_R, weird_frequency)) {
        testthat::expect_error(previous_date_ts(
            date_ts = create_random_date_ts(),
            frequency_ts = wrong_frequency,
            lag = 4L
        ))
    }
})

testthat::test_that("miscellaneous frequency are not allowed", {
    for (wrong_lag in list_wrong_date_ts) {
        date_ts <- create_random_date_ts(frequency_ts = 12L)
        testthat::expect_error(previous_date_ts(
            date_ts = date_ts,
            frequency_ts = 12L,
            lag = wrong_lag
        ))

        date_ts <- create_random_date_ts(frequency_ts = 4L)
        testthat::expect_error(previous_date_ts(
            date_ts = date_ts,
            frequency_ts = 4L,
            lag = wrong_lag
        ))
    }
})

# Tests à la main

testthat::expect_identical(
    object = previous_date_ts(2020L, frequency_ts = 12L, lag = 2L),
    expected = c(2019L, 11L)
)
testthat::expect_identical(
    object = previous_date_ts(c(2020L, 5L), frequency_ts = 12L, lag = 5L),
    expected = c(2019L, 12L)
)
testthat::expect_identical(
    object = previous_date_ts(2020L, frequency_ts = 12L, lag = 0L),
    expected = c(2020L, 1L)
)

# Initialisation ---------------------------------------------------------------

set.seed(2023L)


# Tests de résultat avec start vecteur d'entiers -------------------------------

for (typeA in list_type) {
    for (frequenceA in list_frequence) {
        for (startA in list_start) {
            for (lenA in list_len[-1L]) {
                A_content <- create_random_type(type = typeA, len = lenA)
                ts_A <- ts(A_content, start = startA, frequency = frequenceA)
                for (lagB in list_lag) {
                    for (lenB in list_len[-1L]) {
                        test_name <- paste0(
                            "expected result with ",
                            "\ntypeA = '",
                            typeA,
                            "'\nfrequenceA = ",
                            frequenceA,
                            "\nstartA = ",
                            deparse(startA),
                            "\nlenA = ",
                            lenA,
                            "\nlagB = ",
                            lagB,
                            "\nlenB = ",
                            lenB
                        )

                        testthat::test_that(test_name, {
                            if (length(startA) == 1L) {
                                startA <- c(startA, 1L)
                            }
                            valueB <- create_random_type(
                                type = typeA,
                                len = lenB
                            )
                            startB <- normalize_date_ts(
                                date_ts = c(startA[1L], startA[2L] + lagB),
                                frequency_ts = frequenceA,
                                test = FALSE
                            )
                            ts_B <- ts(
                                valueB,
                                start = startB,
                                frequency = frequenceA
                            )

                            # Cas où lagB >= 0 (start = startA)
                            if (lagB > 0L) {
                                if (lagB > lenA) {
                                    # Cas 4

                                    if (typeA == "raw") {
                                        ts_ResAB1 <- ts(
                                            c(
                                                A_content,
                                                rep(as.raw(0L), lagB - lenA),
                                                valueB
                                            ),
                                            start = startA,
                                            frequency = frequenceA
                                        )

                                        testthat::expect_warning(
                                            testthat::expect_warning(
                                                {
                                                    resAB <- set_value_ts(
                                                        ts_A,
                                                        date_ts = startB,
                                                        replacement = valueB
                                                    )
                                                },
                                                regexp = warning_extend
                                            ),
                                            regexp = "out-of-range values treated as 0 in coercion to raw"
                                        )
                                        testthat::expect_warning(
                                            testthat::expect_warning(
                                                {
                                                    ts_ResAB2 <- combine2ts(
                                                        ts_A,
                                                        ts_B
                                                    )
                                                },
                                                regexp = warning_extend
                                            ),
                                            regexp = "out-of-range values treated as 0 in coercion to raw"
                                        )
                                    } else {
                                        ts_ResAB1 <- ts(
                                            c(
                                                A_content,
                                                rep(NA, lagB - lenA),
                                                valueB
                                            ),
                                            start = startA,
                                            frequency = frequenceA
                                        )

                                        testthat::expect_warning(
                                            {
                                                resAB <- set_value_ts(
                                                    ts_A,
                                                    date_ts = startB,
                                                    replacement = valueB
                                                )
                                            },
                                            regexp = warning_extend
                                        )
                                        testthat::expect_warning(
                                            {
                                                ts_ResAB2 <- combine2ts(
                                                    ts_A,
                                                    ts_B
                                                )
                                            },
                                            regexp = warning_extend
                                        )
                                    }
                                } else if (lagB + lenB < lenA) {
                                    # Cas 6
                                    ts_ResAB1 <- ts(
                                        c(
                                            A_content[1L:lagB],
                                            valueB,
                                            A_content[(lagB + lenB + 1):lenA]
                                        ),
                                        start = startA,
                                        frequency = frequenceA
                                    )
                                    ts_ResAB2 <- combine2ts(ts_A, ts_B)
                                    resAB <- set_value_ts(
                                        ts_A,
                                        date_ts = startB,
                                        replacement = valueB
                                    )
                                } else if (lagB + lenB == lenA) {
                                    # Autres cas
                                    ts_ResAB1 <- ts(
                                        c(A_content[1L:lagB], valueB),
                                        start = startA,
                                        frequency = frequenceA
                                    )
                                    ts_ResAB2 <- combine2ts(ts_A, ts_B)
                                    resAB <- set_value_ts(
                                        ts_A,
                                        date_ts = startB,
                                        replacement = valueB
                                    )
                                } else {
                                    # Cas1

                                    ts_ResAB1 <- ts(
                                        c(A_content[1L:lagB], valueB),
                                        start = startA,
                                        frequency = frequenceA
                                    )
                                    testthat::expect_warning(
                                        {
                                            ts_ResAB2 <- combine2ts(ts_A, ts_B)
                                        },
                                        regexp = warning_extend
                                    )
                                    testthat::expect_warning(
                                        {
                                            resAB <- set_value_ts(
                                                ts_A,
                                                date_ts = startB,
                                                replacement = valueB
                                            )
                                        },
                                        regexp = warning_extend
                                    )
                                }
                            } else if (lagB < 0L) {
                                # Cas où start = strartB < startA

                                if (lagB + lenB < 0L) {
                                    # Cas 3

                                    if (typeA == "raw") {
                                        ts_ResAB1 <- ts(
                                            c(
                                                valueB,
                                                rep(as.raw(0L), -lagB - lenB),
                                                A_content
                                            ),
                                            start = startB,
                                            frequency = frequenceA
                                        )

                                        testthat::expect_warning(
                                            testthat::expect_warning(
                                                {
                                                    resAB <- set_value_ts(
                                                        ts_A,
                                                        date_ts = startB,
                                                        replacement = valueB
                                                    )
                                                },
                                                regexp = warning_extend
                                            ),
                                            regexp = "out-of-range values treated as 0 in coercion to raw"
                                        )
                                        testthat::expect_warning(
                                            testthat::expect_warning(
                                                {
                                                    ts_ResAB2 <- combine2ts(
                                                        ts_A,
                                                        ts_B
                                                    )
                                                },
                                                regexp = warning_extend
                                            ),
                                            regexp = "out-of-range values treated as 0 in coercion to raw"
                                        )
                                    } else {
                                        ts_ResAB1 <- ts(
                                            c(
                                                valueB,
                                                rep(NA, -lagB - lenB),
                                                A_content
                                            ),
                                            start = startB,
                                            frequency = frequenceA
                                        )
                                        testthat::expect_warning(
                                            {
                                                ts_ResAB2 <- combine2ts(
                                                    ts_A,
                                                    ts_B
                                                )
                                            },
                                            regexp = warning_extend
                                        )
                                        testthat::expect_warning(
                                            {
                                                resAB <- set_value_ts(
                                                    ts_A,
                                                    date_ts = startB,
                                                    replacement = valueB
                                                )
                                            },
                                            regexp = warning_extend
                                        )
                                    }
                                } else if (lagB + lenB >= lenA) {
                                    # Cas 5
                                    ts_ResAB1 <- ts(
                                        valueB,
                                        start = startB,
                                        frequency = frequenceA
                                    )
                                    testthat::expect_warning(
                                        {
                                            ts_ResAB2 <- combine2ts(ts_A, ts_B)
                                        },
                                        regexp = warning_extend
                                    )
                                    testthat::expect_warning(
                                        {
                                            resAB <- set_value_ts(
                                                ts_A,
                                                date_ts = startB,
                                                replacement = valueB
                                            )
                                        },
                                        regexp = warning_extend
                                    )
                                } else {
                                    # Cas 2
                                    ts_ResAB1 <- ts(
                                        c(
                                            valueB,
                                            A_content[(lagB + lenB + 1):lenA]
                                        ),
                                        start = startB,
                                        frequency = frequenceA
                                    )
                                    testthat::expect_warning(
                                        {
                                            ts_ResAB2 <- combine2ts(ts_A, ts_B)
                                        },
                                        regexp = warning_extend
                                    )
                                    testthat::expect_warning(
                                        {
                                            resAB <- set_value_ts(
                                                ts_A,
                                                date_ts = startB,
                                                replacement = valueB
                                            )
                                        },
                                        regexp = warning_extend
                                    )
                                }
                            } else {
                                # Cas où lag = 0
                                if (lenB < lenA) {
                                    ts_ResAB1 <- ts(
                                        c(valueB, A_content[(lenB + 1):lenA]),
                                        start = startA,
                                        frequency = frequenceA
                                    )
                                    ts_ResAB2 <- combine2ts(ts_A, ts_B)
                                    resAB <- set_value_ts(
                                        ts_A,
                                        date_ts = startB,
                                        replacement = valueB
                                    )
                                } else if (lenB == lenA) {
                                    ts_ResAB1 <- ts(
                                        valueB,
                                        start = startA,
                                        frequency = frequenceA
                                    )
                                    ts_ResAB2 <- combine2ts(ts_A, ts_B)
                                    resAB <- set_value_ts(
                                        ts_A,
                                        date_ts = startB,
                                        replacement = valueB
                                    )
                                } else {
                                    ts_ResAB1 <- ts(
                                        valueB,
                                        start = startA,
                                        frequency = frequenceA
                                    )
                                    testthat::expect_warning(
                                        {
                                            ts_ResAB2 <- combine2ts(ts_A, ts_B)
                                        },
                                        regexp = warning_extend
                                    )
                                    testthat::expect_warning(
                                        {
                                            resAB <- set_value_ts(
                                                ts_A,
                                                date_ts = startB,
                                                replacement = valueB
                                            )
                                        },
                                        regexp = warning_extend
                                    )
                                }
                            }

                            if (typeA != "Date") {
                                testthat::expect_type(resAB, typeA)
                            }

                            testthat::expect_equal(resAB, ts_ResAB1)
                            testthat::expect_equal(resAB, ts_ResAB2)
                        })
                    }
                }
            }
        }
    }
}


# Tests sur les erreurs de mts --------------------------------------------

testthat::test_that("Several dimensions are not allowed", {
    for (typeA in list_type) {
        for (frequenceA in list_frequence) {
            for (startA in list_start) {
                for (lenA in list_len[-1L]) {
                    B_content <- as.data.frame(lapply(
                        X = 1L:5L,
                        FUN = function(i) {
                            create_random_type(type = typeA, len = lenA)
                        }
                    ))

                    if (typeA == "complex") {
                        mts_B <- do.call(
                            what = cbind,
                            args = lapply(
                                X = B_content,
                                FUN = ts,
                                start = startA,
                                frequency = frequenceA
                            )
                        )
                    } else {
                        mts_B <- ts(
                            B_content,
                            start = startA,
                            frequency = frequenceA
                        )
                    }

                    testthat::expect_error(
                        set_value_ts(
                            series = mts_B,
                            date_ts = create_random_date_ts(),
                            replacement = create_random_type(type = typeA)
                        ),
                        regexp = "Variable 'series': Must be of type 'atomic vector'"
                    )
                }
            }
        }
    }
})

# Tests sur les erreurs d'input ------------------------------------------------

## Test sur le ts --------------------------------------------------------------

testthat::test_that("miscellaneous series are not allowed", {
    for (typeA in list_type) {
        for (obj in object_bank_R) {
            testthat::expect_error(set_value_ts(
                series = obj,
                date_ts = create_random_date_ts(),
                replacement = create_random_type(type = typeA)
            ))
        }
    }
})

## Test sur la date ------------------------------------------------------------

testthat::test_that("miscellaneous date are not allowed", {
    for (typeA in list_type) {
        for (wrong_date in list_wrong_date_ts) {
            testthat::expect_error(set_value_ts(
                series = create_random_ts(type = typeA),
                date_ts = wrong_date,
                replacement = create_random_type(type = typeA)
            ))
        }
    }
})

## Test sur le vecteur value ---------------------------------------------------

testthat::test_that("miscellaneous value input are not allowed", {
    list_wrong_value <- c(
        fuzzr::test_df()[-4L],
        NULL,
        character(0L),
        numeric(0L),
        logical(0L),
        integer(0L),
        complex(0L)
    )
    for (typeA in list_type) {
        for (value in list_wrong_value) {
            testthat::expect_error(set_value_ts(
                series = create_random_ts(type = typeA),
                date_ts = create_random_date_ts(),
                replacement = value
            ))
        }
    }
})

testthat::test_that("value should have same type as series", {
    for (typeA in list_type[-7L]) {
        for (typeB in list_type[-7L]) {
            if (typeA != typeB) {
                testthat::expect_error(
                    set_value_ts(
                        series = create_random_ts(type = typeA),
                        date_ts = create_random_date_ts(),
                        replacement = create_random_type(typeB)
                    ),
                    regexp = "Les objets `series` et `replacement` doivent \u00eatre de m\u00eame type."
                )
            }
        }
    }
})

testthat::test_that("NA values generate warning", {
    for (typeA in list_type[-6L]) {
        ts_A <- create_random_ts(
            type = typeA,
            start = 2000L,
            len = 80L,
            frequency = 4L
        )
        v1 <- sample(
            c(
                create_random_type(typeA, len = 10L),
                create_NA_type(type = typeA, len = 5L)
            ),
            replace = TRUE
        )
        v2 <- create_NA_type(type = typeA, len = 5L)

        testthat::expect_warning(
            set_value_ts(series = ts_A, date_ts = 2010L, replacement = v1),
            regexp = "Contains missing values"
        )
        testthat::expect_warning(
            set_value_ts(series = ts_A, date_ts = 2010L, replacement = v2),
            regexp = "Contains missing values"
        )
    }
})

# Tests sur les erreurs de temporalité --------------------------------------------

testthat::test_that("series and date are temporally consistent", {
    for (typeA in list_type) {
        testthat::expect_error(set_value_ts(
            series = create_random_ts(
                type = typeA,
                start = 2010 + 1 / 7,
                frequency = 12L
            ),
            date_ts = create_random_date_ts(),
            replacement = create_random_type(type = typeA)
        ))
    }

    for (typeA in list_type) {
        testthat::expect_error(set_value_ts(
            series = create_random_ts(
                type = typeA,
                start = 2022 + 1 / 5,
                frequency = 4L
            ),
            date_ts = create_random_date_ts(),
            replacement = create_random_type(type = typeA)
        ))
    }
})

moy <- function(x, y) {
    return((x + y) / 2)
}


test_that("moy works", {
    expect_identical(moy(2, 2), 2)
    expect_identical(moy(2, 0), 1)
})


test_that("moy doesn't work", {
    expect_error(moy("a", 2))
    expect_error(moy(2))
    expect_error(moy(2, "2"))
})

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/modify.R
\name{extend_ts}
\alias{extend_ts}
\title{Ajoute de nouvelles valeurs à un ts}
\usage{
extend_ts(
  series,
  replacement,
  date_ts_to = NULL,
  replace_na = TRUE,
  times = 1L,
  each = 1L
)
}
\arguments{
\item{series}{un objet ts unidimensionnel conforme aux règles de assert_ts}

\item{replacement}{un vecteur de même type que le ts \code{series}}

\item{date_ts_to}{un vecteur numérique, de préférence \code{integer}, au format
date_ts, c'est-à-dire \code{AAAA}, \code{c(AAAA, MM)} ou \code{c(AAAA, TT)}.}

\item{replace_na}{un booléen.}

\item{times}{un entier qui précise le nombre de fois où \code{replacement}
doit être répété, le vecteur entier.}

\item{each}{un entier qui précise le nombre de fois où \code{replacement}
doit être répété mais élément par élément.}
}
\value{
En sortie, la fonction retourne une copie de l'objet \code{series}
complété avec le vecteur \code{replacement}.
}
\description{
La fonction \code{extend_ts} ajoute de nouvelles valeurs à un ts.
}
\details{
\code{date_ts_to} désigne la date jusqu'à laquelle le remplacement
s'effectue. Par défault, cette valeur vaut \code{NULL}.

Si \code{replace_na} vaut \code{TRUE} alors le remplacement commence dès que
l'objet ne contient que des NA. Dans le cas contraire, le ts est étendu,
qu'il contienne des NA ou non à la fin.
Si le vecteur \code{replacement} est de taille un sous-multiple de la différence
de période entre la date de fin de \code{series} et \code{date_ts_to}, le vecteur
\code{replacement} est répété jusqu'à la date \code{date_ts_to}. Sinon une erreur est
générée.

Les arguments \code{times} et \code{each} en sont utilisé que si
\code{date_ts} est manquant (non fourni par l'utilisateur). Si tel est le
cas, ils se comporte comme si \code{replacement} devenait
\code{rep(replacement, times = times, each = each)}.
}
\examples{

ts1 <- ts(
    data = c(rep(NA_integer_, 3L), 1L:10L, rep(NA_integer_, 3L)),
    start = 2020,
    frequency = 12
)
x <- rep(3L, 2L)

extend_ts(series = ts1, replacement = x)
extend_ts(series = ts1, replacement = x, replace_na = FALSE)
extend_ts(series = ts1, replacement = x,
          date_ts_to = c(2021L, 7L), replace_na = TRUE)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/conversion.R
\name{substr_year}
\alias{substr_year}
\title{Retire une année à une date}
\usage{
substr_year(date, n = 1L)
}
\arguments{
\item{date}{un objet de type Date}

\item{n}{un entier}
}
\value{
En sortie, la fonction retourne un objet de type Date (atomic) de
longueur 1.
}
\description{
La fonction \code{substr_year} retire \code{n} annnée(s) à une date.
}
\examples{

substr_year(as.Date("2000-02-29"), n = 1L)
substr_year(as.Date("2000-02-29"), n = 3L)
substr_year(as.Date("2000-02-29"), n = 4L)
substr_year(as.Date("2000-02-29"), n = 16L)

substr_year(as.Date("2023-01-25"), n = 10L)
substr_year(as.Date("2022-11-01"), n = 3L)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/check.R
\name{check_ts}
\alias{check_ts}
\alias{assert_ts}
\title{Vérifie la conformité d'un objet ts}
\usage{
check_ts(x, .var.name = checkmate::vname(x), allow_mts = FALSE)

assert_ts(x, add = NULL, .var.name = checkmate::vname(x), allow_mts = FALSE)
}
\arguments{
\item{x}{Un objet ts unidimensionnel}

\item{.var.name}{Nom de l'objet à vérifier pour afficher dans les messages}

\item{allow_mts}{Booleen. Est ce que les objects \code{mts} sont acceptés ?}

\item{add}{Collection pour stocker les messages d'erreurs (Default is NULL)}
}
\value{
En sortie la fonction retourne l'objet \code{x} de manière invisible ou
une erreur.
}
\description{
Les fonctions \code{assert_ts} et \code{check_ts} vérifient qu'un objet ts
est bien conforme.
}
\details{
Les fonctions du package date4ts sont faites pour fonctionner
avec des times-series de fréquence mensuelle ou trimestrielle et basées sur
le système des mois, trimestres et années classiques.
On travaille avec des données numériques (integer, double ou logical) mais
les autres types atomic sont acceptés également.
On cherche donc à favoriser l'utilisation de séries temporelles classiques
utilisants des types atomiques.
Lorsque l'objet \code{x} en entrée est au mauvais format, une erreur est généré.

Selon le préfixe de la fonction :
\itemize{
\item si le check réussi :
\itemize{
\item la fonction \code{assert_ts} retourne l'objet \code{x} de manière invisible;
\item la fonction \code{check_ts} retourne le booléen \code{TRUE}.
}
\item si le check échoue :
\itemize{
\item la fonction \code{assert_ts} retourne un message d'erreur;
\item la fonction \code{check_ts} retourne une chaîne de caractère signalant le
problème.
}
}
}
\examples{
ts1 <- ts(1:100, start = 2010L, frequency = 12L)
ts2 <- ts(1:10, start = c(2020L, 4L), frequency = 4L)

assert_ts(ts1)
assert_ts(ts2)

check_ts(ts1)
check_ts(ts2)

# Exemples avec des erreurs

check_ts(1)
check_ts(ts(1:10, start = 2010L, frequency = 2L))
check_ts(1:10)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/check.R
\name{check_scalar_integer}
\alias{check_scalar_integer}
\alias{assert_scalar_integer}
\title{Vérifie la conformité d'un entier scalaire}
\usage{
check_scalar_integer(x, warn = TRUE)

assert_scalar_integer(
  x,
  add = NULL,
  .var.name = checkmate::vname(x),
  warn = TRUE
)
}
\arguments{
\item{x}{un entier relatif (positif, négatif ou nul)}

\item{warn}{un booleen}

\item{add}{Collection pour stocker les messages d'erreurs (Default is NULL)}

\item{.var.name}{Nom de l'objet à vérifier pour afficher dans les messages}
}
\value{
En sortie la fonction retourne l'objet \code{x} de manière invisible ou
une erreur.
}
\description{
Vérifie la conformité d'un entier scalaire
}
\details{
On vérifie que l'objet \code{x} en entrée est bien un entier.
Cette fonction s'appuie essentiellement sur la fonction
\code{checkmate::assert_int}.
Il y a néanmoins une petite subtilité : on vérifie si l'objet \code{x} est de
type double ou integer. Si l'objet est de type double (et non integer), la
fonction retournera aussi un warning.
Dans le premier cas, on affichera un warning et on corrigera l'objet au
format integer pour les traitements ultérieurs. En sortie, \code{x} est retourné
de manière invisible.
Si l'argument \code{warn} vaut \code{FALSE}, alors la fonction ne retournera pas de
warning lors de l'évaluation.

Selon le préfixe de la fonction :
\itemize{
\item si le check réussi :
\itemize{
\item la fonction \code{assert_scalar_integer} retourne l'objet \code{x} de manière
invisible;
\item la fonction \code{check_scalar_integer} retourne le booléen \code{TRUE}.
}
\item si le check échoue :
\itemize{
\item la fonction \code{assert_scalar_integer} retourne un message d'erreur;
\item la fonction \code{check_scalar_integer} retourne une chaîne de caractère
signalant le problème.
}
}
}
\examples{

assert_scalar_integer(1L)
assert_scalar_integer(100L)
assert_scalar_integer(-4L)
assert_scalar_integer(0L)

check_scalar_integer(1L)
check_scalar_integer(100L)
check_scalar_integer(-4L)
check_scalar_integer(0L)

# Avec des erreurs,

check_scalar_integer(Inf)
check_scalar_integer(1:10)
check_scalar_integer(pi)
check_scalar_integer(2.)

}
\seealso{
\code{\link[=check_scalar_natural]{check_scalar_natural()}}, \code{\link[=assert_scalar_natural]{assert_scalar_natural()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/conversion.R
\name{date2date_ts}
\alias{date2date_ts}
\title{Conversion d'une date au format TS}
\usage{
date2date_ts(date, frequency_ts = 12L)
}
\arguments{
\item{date}{un objet de type Date}

\item{frequency_ts}{un entier qui vaut \code{4L} (ou \code{4.0}) pour les séries
trimestrielles et \code{12L} (ou \code{12.0}) pour les séries mensuelles.}
}
\value{
En sortie, la fonction retourne la date au format \code{date_ts}
(\code{c(AAAA, MM)} ou \code{c(AAAA, TT)}) avec le mois ou trimestre en cours selon
l'argument \code{frequency_ts}.
}
\description{
La fonction \code{date2date_ts} prend en argument une date au format
date (integer avec une class Date) et la convertit au format \code{date_ts} :
\code{c(AAAA, MM)} ou \code{c(AAAA, TT)} avec le mois ou trimestre en cours.
}
\examples{

date2date_ts(as.Date("2000-01-01"))
date2date_ts(as.Date("2000-01-01"), frequency_ts = 12L)

date2date_ts(as.Date("2021-10-01"), frequency_ts = 12L)
date2date_ts(as.Date("2021-10-01"), frequency_ts = 4L)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/conversion.R
\name{date_ts2timeunits}
\alias{date_ts2timeunits}
\title{Conversion d'une date du format date_ts au format TimeUnits}
\usage{
date_ts2timeunits(date_ts, frequency_ts)
}
\arguments{
\item{date_ts}{un vecteur numérique, de préférence \code{integer}, au format
\code{AAAA}, \code{c(AAAA, MM)} ou \code{c(AAAA, TT)}}

\item{frequency_ts}{un entier qui vaut \code{4L} (ou \code{4.0}) pour les séries
trimestrielles et \code{12L} (ou \code{12.0}) pour les séries mensuelles.}
}
\value{
En sortie, la fonction retourne la date au format \code{AAAA + TT/4} ou
\code{AAAA + MM/12} (un numeric de longueur 1).
}
\description{
Conversion d'une date du format date_ts au format TimeUnits
}
\details{
\code{AAAA} signifie que l'année est au format numérique avec 4 chiffres
(Exemple : l'année deux mille vingt-deux s'écrit 2022 et non 22)
MM signifie que le mois est au format numérique (Exemple : le mois de mai
s'écrit 5, le moi de décembre s'écrit 12)
TT signifie que le trimestre est au format numérique (Exemple : le troisième
trimestre s'écrit 3)
}
\examples{

# Avril 2020
date_ts2timeunits(date_ts = c(2020L, 4L), frequency_ts = 12L)
# Novembre 2020
date_ts2timeunits(date_ts = c(2022L, 11L), frequency_ts = 12L)

# 4ème trimestre de 2022
date_ts2timeunits(date_ts = c(2022, 4L), frequency_ts = 4L)
# 2ème trimestre de 1995
date_ts2timeunits(date_ts = c(1995L, 2L), frequency_ts = 4L)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/check.R
\name{check_scalar_date}
\alias{check_scalar_date}
\alias{assert_scalar_date}
\title{Vérifie la conformité d'une date scalaire}
\usage{
check_scalar_date(x)

assert_scalar_date(x, add = NULL, .var.name = checkmate::vname(x))
}
\arguments{
\item{x}{un objet de type \code{Date}.}

\item{add}{Collection pour stocker les messages d'erreurs (Default is NULL)}

\item{.var.name}{Nom de l'objet à vérifier pour afficher dans les messages}
}
\value{
En sortie la fonction retourne l'objet \code{x} de manière invisible ou
une erreur.
}
\description{
Vérifie la conformité d'une date scalaire
}
\details{
On vérifie que l'objet \code{x} en entrée est bien au format \code{Date} et qu'il
s'agit d'un scalaire (vecteur de taille 1).
Cette fonction s'appuie essentiellement sur la fonction
\code{checkmate::assert_date}.

Selon le préfixe de la fonction :
\itemize{
\item si le check réussi :
\itemize{
\item la fonction \code{assert_scalar_date} retourne l'objet \code{x} de manière
invisible;
\item la fonction \code{check_scalar_date} retourne le booléen \code{TRUE}.
}
\item si le check échoue :
\itemize{
\item la fonction \code{assert_scalar_date} retourne un message d'erreur;
\item la fonction \code{check_scalar_date} retourne la chaîne de caractère
correspondante à l'erreur du check.
}
}
}
\examples{

assert_scalar_date(as.Date("2018-01-24"))
assert_scalar_date(as.Date("2000-02-29"))
assert_scalar_date(Sys.Date())

check_scalar_date(as.Date("2018-01-24"))
check_scalar_date(as.Date("2000-02-29"))
check_scalar_date(Sys.Date())

# Avec des erreurs

check_scalar_date(2L)
check_scalar_date(seq(from = as.Date("2000-01-01"), to = Sys.Date(), by =
"year"))

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/modify.R
\name{set_value_ts}
\alias{set_value_ts}
\title{Change certaines valeurs d'un ts}
\usage{
set_value_ts(series, date_ts, replacement)
}
\arguments{
\item{series}{un objet ts unidimensionnel conforme aux règles de assert_ts}

\item{date_ts}{un vecteur numérique, de préférence \code{integer}, au format
\code{AAAA}, \code{c(AAAA, MM)} ou \code{c(AAAA, TT)}}

\item{replacement}{un vecteur de même type que le ts \code{series}}
}
\value{
En sortie, la fonction retourne une copie de l'objet \code{series}
modifié avec les valeurs de \code{replacement} imputés à partir de la date
\code{date_ts}.
}
\description{
La fonction \code{set_value_ts} modifie la ou les valeurs d'un objet
ts à une date donnée.
}
\examples{
set_value_ts(
    series = ev_pib,
    date_ts = c(2021L, 2L),
    replacement = c(1, 2, 3)
)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/date_manipulation.R
\name{is_before}
\alias{is_before}
\title{Comparaison de 2 date_ts}
\usage{
is_before(a, b, frequency_ts, strict = FALSE)
}
\arguments{
\item{a}{un objet date_ts, c'est-à-dire un vecteur numérique, de préférence
\code{integer} au format \code{AAAA}, \code{c(AAAA, MM)} ou \code{c(AAAA, TT)}}

\item{b}{un objet date_ts, c'est-à-dire un vecteur numérique, de préférence
\code{integer} au format \code{AAAA}, \code{c(AAAA, MM)} ou \code{c(AAAA, TT)}}

\item{frequency_ts}{un entier qui vaut \code{4L} (ou \code{4.0}) pour les séries
trimestrielles et \code{12L} (ou \code{12.0}) pour les séries mensuelles.}

\item{strict}{un booleen (default \code{FALSE})}
}
\value{
En sortie, la fonction retourne un booleen (de longueur 1) qui
indique si la date \code{a} est antérieure à la date \code{b}.
}
\description{
Comparaison de 2 date_ts
}
\details{
Les dates \code{a} et \code{b} sont au format date_ts. L'argument frequency_ts
est nécessaire pour interpréter les dates.
Ainsi, si je souhaite comparer la date \code{a = c(2023L, 4L)} et la date \code{b = c(2023L, -2L)}. Dans le cas d'une fréquence mensuelle, la date \code{a} est
antérieure à la date \code{b}. Dans le cas d'une fréquence mensuelle, c'est
l'inverse. Si \code{strict} vaut \code{TRUE}, la fonction compare strictement les dates
\code{a} et \code{b} (\code{<}).
}
\examples{

is_before(a = c(2020L, 3L), b = c(2022L, 4L), frequency_ts = 12L)
is_before(a = c(2022L, 3L), b = c(2010L, 1L), frequency_ts = 4L)

is_before(a = c(2022L, 4L), b = c(2022L, 4L), frequency_ts = 12L)
is_before(a = c(2022L, 4L), b = c(2022L, 4L),
    frequency_ts = 12L, strict = TRUE)

# Importance de la fréquence
is_before(a = c(2022L, -3L), b = c(2021L, 8L), frequency_ts = 12L)
is_before(a = c(2022L, -3L), b = c(2021L, 8L), frequency_ts = 4L)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/check.R
\name{check_expression}
\alias{check_expression}
\alias{assert_expression}
\title{Vérifie la conformité d'une expression}
\usage{
check_expression(expr)

assert_expression(expr)
}
\arguments{
\item{expr}{une expression à évaluer}
}
\value{
En sortie la fonction retourne l'objet \code{x} (le résultat de
l'évaluation de l'expression \code{expr}) de manière invisible ou une erreur.
}
\description{
Vérifie la conformité d'une expression
}
\details{
La fonction évalue l'expression \code{expr}. Le check vérifie si la
fonction génère une erreur ou un warning. Si elle ne génère aucun message
particulier, on retourne alors l'objet \code{x} (le résultat de l'évaluation de
l'expression \code{expr}), sans erreur.

Selon le préfixe de la fonction :
\itemize{
\item si le check réussi :
\itemize{
\item la fonction \code{assert_expression} retourne l'objet \code{x} de manière
invisible;
\item la fonction \code{check_expression} retourne le booléen \code{TRUE}.
}
\item si le check échoue :
\itemize{
\item la fonction \code{assert_expression} retourne un message d'erreur;
\item la fonction \code{check_expression} retourne la chaîne de caractère
"Invalid expression".
}
}
}
\examples{

assert_expression(expr = {2 + 2})
assert_expression(expr = {is.integer(1L)})
try(assert_expression(expr = {log("a")}), silent = TRUE)

check_expression(expr = {2 + 2})
check_expression(expr = {is.integer(1L)})
check_expression(expr = {log("a")})

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/check.R
\name{check_timeunits}
\alias{check_timeunits}
\alias{assert_timeunits}
\title{Vérifie la conformité d'un objet TimeUnits}
\usage{
check_timeunits(x, frequency_ts, .var.name = checkmate::vname(x))

assert_timeunits(x, frequency_ts, add = NULL, .var.name = checkmate::vname(x))
}
\arguments{
\item{x}{un numérique qui représente le time units de}

\item{frequency_ts}{un entier qui vaut \code{4L} (ou \code{4.0}) pour les séries
trimestrielles et \code{12L} (ou \code{12.0}) pour les séries mensuelles.}

\item{.var.name}{Nom de l'objet à vérifier pour afficher dans les messages}

\item{add}{Collection pour stocker les messages d'erreurs (Default is NULL)}
}
\value{
En sortie la fonction retourne l'objet \code{x} de manière invisible ou
une erreur.
}
\description{
La fonction \code{assert_timeunits} vérifie qu'un objet est un
TimeUnits.
}
\details{
Un objet de type TimeUnits est un numérique qui désigne l'année et la période
en cours avec ses décimales.
Ainsi pour une série temporelle mensuelle, \code{2020.5} représente la moitié de
l'année donc juillet 2020 et s'écrit \code{c(2020L, 7L)} au format date_ts.

Selon le préfixe de la fonction :
\itemize{
\item si le check réussi :
\itemize{
\item la fonction \code{assert_timeunits} retourne l'objet \code{x} de manière
invisible;
\item la fonction \code{check_timeunits} retourne le booléen \code{TRUE}.
}
\item si le check échoue :
\itemize{
\item la fonction \code{assert_timeunits} retourne un message d'erreur;
\item la fonction \code{check_timeunits} retourne une chaîne de caractère
signalant le problème.
}
}
}
\examples{

assert_timeunits(2020.5, frequency_ts = 12L)
assert_timeunits(2020.5, frequency_ts = 4L)
assert_timeunits(2023., frequency_ts = 12L)

assert_timeunits(2000. + 5. / 12.0, frequency_ts = 12L)
assert_timeunits(2015. + 3. / 4.0, frequency_ts = 4L)

check_timeunits(2020.5, frequency_ts = 12L)
check_timeunits(2015. + 3. / 4.0, frequency_ts = 4L)

# Avec erreur

check_timeunits(list(1.), frequency_ts = 12L)
check_timeunits(2000., frequency_ts = 1L)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/libelles.R
\name{libelles}
\alias{libelles}
\title{Libelés pour une période}
\usage{
libelles(date_ts, frequency_ts, n = 1L, warn = TRUE)
}
\arguments{
\item{date_ts}{un vecteur numérique, de préférence \code{integer}, au format
\code{AAAA}, \code{c(AAAA, MM)} ou \code{c(AAAA, TT)}}

\item{frequency_ts}{un entier qui vaut \code{4L} (ou \code{4.0}) pour les séries
trimestrielles et \code{12L} (ou \code{12.0}) pour les séries mensuelles.}

\item{n}{un entier}

\item{warn}{un booleen}
}
\value{
En sortie, la fonction retourne un vecteur de chaine de caractère de
longueur \code{n} avec les libellés de la période (de la date \code{date_ts} à la date
\verb{date_ts + n périodes}.
}
\description{
La fonction \code{libelles} créé un vecteur de chaines de caractère
contenant les libelés de toutes les dates sur une période
}
\details{
Pour choisir la période, il faut spécifier une date de début
\code{date_ts}, une fréquence \code{frequency_ts} pour le pas entre 2 dates
(trimestrielle ou mensuelle) et un nombre de valeur \code{n} (nombre de période).

Si l'argument \code{warn} est \code{FALSE}, alors la fonction ne retournera pas de
warning lors de l'évaluation.
}
\examples{
libelles(date_ts = c(2019L, 10L), frequency_ts = 12L, n = 9L)
libelles(date_ts = c(2019L, 4L), frequency_ts = 4L, n = 3L)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/conversion.R
\name{date_ts2date}
\alias{date_ts2date}
\title{Conversion d'une date du format TS au format date}
\usage{
date_ts2date(date_ts, frequency_ts)
}
\arguments{
\item{date_ts}{un vecteur numérique, de préférence \code{integer}, au format
\code{AAAA}, \code{c(AAAA, MM)} ou \code{c(AAAA, TT)}}

\item{frequency_ts}{un entier qui vaut \code{4L} (ou \code{4.0}) pour les séries
trimestrielles et \code{12L} (ou \code{12.0}) pour les séries mensuelles.}
}
\value{
En sortie, la fonction retourne un objet de type Date (atomic) de
longueur 1 qui correspond à l'objet \code{date_ts}.
}
\description{
Conversion d'une date du format TS au format date
}
\examples{

date_ts2date(date_ts = c(2020L, 11L), frequency_ts = 12L)
date_ts2date(date_ts = c(1995L, 2L), frequency_ts = 4L)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/modify.R
\name{na_trim}
\alias{na_trim}
\title{Supprime les NA aux bords}
\usage{
na_trim(series, sides = c("both", "left", "right"))
}
\arguments{
\item{series}{un objet ts unidimensionnel conforme aux règles de assert_ts}

\item{sides}{une chaine de caractere qui spécifie quelle NA doivent être
retirés (au début et à la fin ("both"), juste au début ("left") ou juste à
la fin ("right"))}
}
\value{
En sortie, la fonction retourne une copie de l'objet \code{series}
corrigée des NA et début et fin de série.
}
\description{
La fonction \code{na_trim} supprime les NA en début et en fin de
période.
}
\details{
L'objet retourné commence et finis par des valeurs non manquantes.
}
\examples{

ts1 <- ts(c(rep(NA, 3L), 1:10, rep(NA, 3L)), start = 2020, frequency = 12L)
ts2 <- ts(c(1:10, rep(NA, 3L)), start = c(2023, 2), frequency = 4L)
ts3 <- ts(c(rep(NA, 3L), 1:10), start = 2000, frequency = 12L)

na_trim(ts1)
na_trim(ts2)
na_trim(ts3)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{ev_pib}
\alias{ev_pib}
\title{Évolution du PIB français jusqu'au T1 2022}
\format{
Un ts unidimensionnel :
\describe{
\item{start}{le ts commence au T1 1970 mais la série de PIB ne commence qu'au
T2 1980.}
\item{end}{le ts finit au T3 2022 mais la série de PIB finit au T1 2022.}
\item{frequency_ts}{la fréquence est trimestrielle}
}
}
\source{
\url{https://www.insee.fr/fr/statistiques/2830547}
}
\usage{
ev_pib
}
\description{
Ce jeu de données contient une série \code{ts} de l'évolution trimestrielle du
produit intérieur brut français.
Toutes les infos complémentaires sur cette série se trouve sur la page de la
\href{https://www.insee.fr/fr/statistiques/2830547}{publication} sur le site
de l'\href{https://www.insee.fr/fr/accueil}{Insee}.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/conversion.R
\name{trim2mens}
\alias{trim2mens}
\alias{mens2trim}
\title{Conversion entre date mensuelle et trimestrielle}
\usage{
trim2mens(date_ts)

mens2trim(date_ts)
}
\arguments{
\item{date_ts}{un vecteur numérique, de préférence \code{integer}, au format
\code{AAAA}, \code{c(AAAA, MM)} ou \code{c(AAAA, TT)}}
}
\value{
En sortie, la fonction retourne la date toujours au format
\code{date_ts}.
}
\description{
Les fonctions \code{trim2mens} et \code{mens2trim} convertissent une
\code{date_ts} du format mensuelle \code{c(AAAA, MM)} au format trimestrielle \code{c(AAAA, TT)}.
}
\examples{
trim2mens(c(2019L, 4L)) # 4ème trimestre 2019 --> Octobre 2019
trim2mens(c(2020L, 1L)) # 1er trimestre 2020 --> Janvier 2020

mens2trim(c(2019L, 4L)) # Avril 2019 --> 2ème trimestre 2019
mens2trim(c(2020L, 11L)) # Novembre 2020 --> 4ème trimestre 2020

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/modify.R
\name{combine2ts}
\alias{combine2ts}
\title{Combiner 2 ts}
\usage{
combine2ts(a, b)
}
\arguments{
\item{a}{un objet ts unidimensionnel conforme aux règles de assert_ts}

\item{b}{un objet ts unidimensionnel conforme aux règles de assert_ts}
}
\value{
En sortie, la fonction retourne un ts qui contient les valeurs de
\code{a} aux temps de \code{a} et les valeurs de \code{b} aux temps de \code{b}.
}
\description{
La fonction \code{combine2ts} combine (comme c()) 2 time series de
même fréquence (mensuelle ou trimestrielle).
}
\details{
Si \code{a} et \code{b} ont une période en commun, les valeurs de \code{b} écrasent
celles de a sur la période concernée.
Si il existe une période sur laquelle ni \code{a} ni \code{b} ne prennent de valeur
(mais qu'il existe des valeurs à des dates ultérieures et antérieures) alors
le ts en sortie prendra NA sur cette période.
}
\examples{

trim_1 <- stats::ts(rep(1, 4), start = 2021, frequency = 4L)

mens_1 <- stats::ts(rep(1, 4), start = 2020, frequency = 12L)
mens_2 <- stats::ts(rep(2, 4), start = 2022, frequency = 12L)

# La série de PIB est écrasé par trim_1 sur la période temporelle de trim_1
combine2ts(ev_pib, trim_1)

# La période entre les séries temporelles mens_1 et mens_2 est complétée par
# des NA
combine2ts(mens_1, mens_2)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/conversion.R
\name{as_yyyytt}
\alias{as_yyyytt}
\alias{as_yyyymm}
\title{Conversion au format date_ts}
\usage{
as_yyyytt(timeunits)

as_yyyymm(timeunits)
}
\arguments{
\item{timeunits}{une date en année (Par exemple 2015.25 pour le 2ème
trimestre 2015 ou \code{2021.83333333333} pour novembre 2021)}
}
\value{
En sortie, ces fonctions retournent la date au format \code{date_ts}
(c'est-à-dire un vecteur d'entiers de la forme \code{AAAA}, \code{c(AAAA, MM)} ou
\code{c(AAAA, TT)})
}
\description{
Les fonctions \code{as_yyyytt} et \code{as_yyyymm} convertissent une date
du format TimeUnits au format \code{date_ts}.
}
\details{
La fonction \code{as_yyyytt} retourne la date par trimestre et la fonction
\code{as_yyyymm} retourne la date par mois.
}
\examples{
as_yyyytt(2019.75) # 4ème trimestre 2019
as_yyyytt(2020) # 1er trimestre 2020
as_yyyytt(2022 + 1 / 4) # 2ème trimestre 2022

as_yyyymm(2019.75) # Octobre 2019
as_yyyymm(2020) # Janvier 2020
as_yyyymm(2020 + 1 / 12) # Février 2020
as_yyyymm(2020 + 12 / 12) # Janvier 2021

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/check.R
\name{check_scalar_natural}
\alias{check_scalar_natural}
\alias{assert_scalar_natural}
\title{Vérifie la conformité d'un entier naturel}
\usage{
check_scalar_natural(x, warn = TRUE)

assert_scalar_natural(
  x,
  add = NULL,
  .var.name = checkmate::vname(x),
  warn = TRUE
)
}
\arguments{
\item{x}{un entier naturel strictement positif}

\item{warn}{un booleen}

\item{add}{Collection pour stocker les messages d'erreurs (Default is NULL)}

\item{.var.name}{Nom de l'objet à vérifier pour afficher dans les messages}
}
\value{
En sortie la fonction retourne l'objet \code{x} de manière invisible ou
une erreur.
}
\description{
Le but de cett fonction est de tester si une variable x est un nombre naturel
strictement positif.
}
\details{
Cette fonction s'appuie essentiellement sur la fonction
\code{checkmate::assert_count}.
Il y a néanmoins une petite subtilité : on vérifie si l'objet \code{x} est de type
double ou integer.
Dans le premier cas, on affichera un warning et on corrigera l'objet au
format integer pour les traitements ultérieurs. En sortie, \code{x} est retourné
de manière invisible.
Si l'argument \code{warn} est \code{FALSE}, alors la fonction ne retournera pas de
warning lors de l'évaluation.

Selon le préfixe de la fonction :
\itemize{
\item si le check réussi :
\itemize{
\item la fonction \code{assert_scalar_natural} retourne l'objet \code{x} de manière
invisible;
\item la fonction \code{check_scalar_natural} retourne le booléen \code{TRUE}.
}
\item si le check échoue :
\itemize{
\item la fonction \code{assert_scalar_natural} retourne un message d'erreur;
\item la fonction \code{check_scalar_natural} retourne une chaîne de caractère
signalant le problème.
}
}
}
\examples{

# Avec des entier integer
assert_scalar_natural(1L)
assert_scalar_natural(100L)

# Avec des entiers double
assert_scalar_natural(2.)
assert_scalar_natural(457)

}
\seealso{
\code{\link[=check_scalar_integer]{check_scalar_integer()}}, \code{\link[=assert_scalar_integer]{assert_scalar_integer()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/date_manipulation.R
\name{last_date}
\alias{last_date}
\title{Dernière date non NA}
\usage{
last_date(series)
}
\arguments{
\item{series}{un objet ts unidimensionnel conforme aux règles de assert_ts}
}
\value{
En sortie, la fonction retourne un objet au format \code{date_ts}
(\code{AAAA}, \code{c(AAAA, MM)} ou \code{c(AAAA, TT)})
}
\description{
Cette fonction calcule la dernière date pour laquelle l'objet
\code{series} ne vaut pas NA.
}
\details{
La date retournée en output est au format \code{date_ts}. Si l'objet
\code{series} ne contient que des NAs, la fonction retourne une erreur.
}
\examples{

ts1 <- ts(c(NA, NA, NA, 1:10, NA), start = 2000, frequency = 12L)
ts2 <- ts(c(1:10), start = 2020, frequency = 4L)

stats::end(ts1)
last_date(ts1)

stats::end(ts2)
last_date(ts2)

}
\seealso{
\code{first_date}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/check.R
\name{check_frequency}
\alias{check_frequency}
\alias{assert_frequency}
\title{Vérifie la conformité d'une fréquence}
\usage{
check_frequency(x, .var.name = checkmate::vname(x), warn = TRUE)

assert_frequency(x, add = NULL, .var.name = checkmate::vname(x), warn = TRUE)
}
\arguments{
\item{x}{un entier qui vaut \code{4L} (ou \code{4.0}) pour les séries trimestrielles et
\code{12L} (ou \code{12.0}) pour les séries mensuelles.}

\item{.var.name}{Nom de l'objet à vérifier pour afficher dans les messages}

\item{warn}{un booleen}

\item{add}{Collection pour stocker les messages d'erreurs (Default is NULL)}
}
\value{
En sortie la fonction retourne l'objet \code{x} de manière invisible ou
une erreur.
}
\description{
Vérifie la conformité d'une fréquence
}
\details{
La fréquence d'une série temporelle est soit mensuelle (\code{12L} ou
\code{12.0}) soit trimestrielle (\code{4L} ou \code{4.0}). Les autres fréquences ne sont pas
acceptées.
Cette fonction s'appuie essentiellement sur les fonctions
\code{checkmate::check_numeric}, \code{checkmate::check_int} et
\code{checkmate::check_choice}.
Il y a néanmoins une petite subtilité : on vérifie si l'objet \code{x} est de type
double ou integer.
Dans le premier cas, on affichera un warning et on corrigera l'objet au
format integer pour les traitements ultérieurs. En sortie, \code{x} est retourné
de manière invisible.
Si l'argument \code{warn} est \code{FALSE}, alors la fonction ne retournera pas de
warning lors de l'évaluation.

Selon le préfixe de la fonction :
\itemize{
\item si le check réussi :
\itemize{
\item la fonction \code{assert_frequency} retourne l'objet \code{x} de manière
invisible;
\item la fonction \code{check_frequency} retourne le booléen \code{TRUE}.
}
\item si le check échoue :
\itemize{
\item la fonction \code{assert_frequency} retourne un message d'erreur;
\item la fonction \code{check_frequency} retourne une chaîne de caractère
signalant le problème.
}
}
}
\examples{

assert_frequency(4L)
assert_frequency(12L)

check_frequency(4L)
check_frequency(12L)

# Avec des erreurs,

check_frequency(Inf, warn = FALSE)
check_frequency(1:10)
check_frequency(1L)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_data.R
\name{get_value_ts}
\alias{get_value_ts}
\title{Récupère des valeurs d'un ts}
\usage{
get_value_ts(series, date_from, date_to, n)
}
\arguments{
\item{series}{un objet ts unidimensionnel conforme aux règles de assert_ts}

\item{date_from}{un vecteur numérique, de préférence \code{integer} au format
\code{AAAA}, \code{c(AAAA, MM)} ou \code{c(AAAA, TT)}}

\item{date_to}{un vecteur numérique, de préférence \code{integer} au format
\code{AAAA}, \code{c(AAAA, MM)} ou \code{c(AAAA, TT)}}

\item{n}{un entier}
}
\value{
En sortie, la fonction retourne un vecteur (atomic) de même type que
\code{series} avec les valeurs extraites.
}
\description{
La fonction \code{get_value_ts} permet de récupérer des valeurs.
}
\details{
Il faut qu'exactement 2 arguments parmi \code{date_to}, \code{date_to} et \code{n} soient
renseignés. L'argument \code{n} combiné avec \code{date_to} ou \code{date_from} permet de
déterminer combien de période seront retourné à partir de ou jusqu'à la date
renseignée.

Il faudrait parler d'extraction car contrairement à la fonction
\code{window}, ici on retourne un vecteur devaleur et plus un objet ts.
}
\examples{

ts1 <- ts(1:100, start = 2012L, frequency = 12L)
ts2 <- ts(letters, start = 2014L, frequency = 4L)
ts3 <- ts(exp(-(1:50)), start = 2015L, frequency = 12L)

get_value_ts(series = ts1, date_from = c(2015L, 7L), date_to = c(2018L, 6L))
get_value_ts(series = ts2, date_from = c(2018L, 4L), n = 4L)
get_value_ts(series = ts3, date_to = c(2018L, 4L), n = 14L)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/date_manipulation.R
\name{diff_periode}
\alias{diff_periode}
\title{Intervalle entre 2 dates}
\usage{
diff_periode(a, b, frequency_ts)
}
\arguments{
\item{a}{un objet date_ts, c'est-à-dire un vecteur numérique, de préférence
\code{integer} au format \code{AAAA}, \code{c(AAAA, MM)} ou \code{c(AAAA, TT)}}

\item{b}{un objet date_ts, c'est-à-dire un vecteur numérique, de préférence
\code{integer} au format \code{AAAA}, \code{c(AAAA, MM)} ou \code{c(AAAA, TT)}}

\item{frequency_ts}{un entier qui vaut \code{4L} (ou \code{4.0}) pour les séries
trimestrielles et \code{12L} (ou \code{12.0}) pour les séries mensuelles.}
}
\value{
En sortie, la fonction retourne un entier qui désigne le nombre de
période (mois ou trimestres) qui sépare les 2 dates \code{a} et \code{b}.
}
\description{
Intervalle entre 2 dates
}
\details{
On travaille ici avec des dates au format date_ts, c'est-à-dire qui
passe le test de la fonction \code{assert_date_ts}. Lorsqu'on parle d'intervalle
et de nombre de période entre \code{a} et \code{b}, les bornes sont incluses. Ainsi
\code{diff_periode(2020L, 2020L, 12L)} retourne bien 1L et non 2L ou 0L.
}
\examples{
# Une seule période
diff_periode(a = 2020L, b = 2020L, frequency_ts = 4L)

diff_periode(a = c(2000L, 1L), b = c(2020L, 4L), frequency_ts = 4L)

# Ordre chronologique respecté
diff_periode(a = c(2021L, 5L), b = c(2023L, 8L), frequency_ts = 12L)

# Date inversées
diff_periode(a = c(2023L, 8L), b = c(2021L, 5L), frequency_ts = 12L)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/date_manipulation.R
\name{next_date_ts}
\alias{next_date_ts}
\title{Obtenir la date suivante}
\usage{
next_date_ts(date_ts, frequency_ts, lag = 1L)
}
\arguments{
\item{date_ts}{un vecteur numérique, de préférence \code{integer}, au format
\code{AAAA}, \code{c(AAAA, MM)} ou \code{c(AAAA, TT)}}

\item{frequency_ts}{un entier qui vaut \code{4L} (ou \code{4.0}) pour les séries
trimestrielles et \code{12L} (ou \code{12.0}) pour les séries mensuelles.}

\item{lag}{un entier}
}
\value{
En sortie, la fonction retourne un vecteur d'entier qui représente
la date à la période future au format \code{date_ts}.
}
\description{
Obtenir la date suivante
}
\details{
Lorsqu'on parle de date suivante, on parle de date future.
L'argument \code{lag} est entier et désigne le nombre de décalage que l'on affecte
à notre date.
Par exemple pour des lag positif (1L, 2L, 10L) on désigne le décalage de la
période suivante, celle d'après et celle dans 10 périodes.
Cependant, lorsque l'argument \code{lag} vaut zéro, la fonction retourne la
\code{date} inchangée. Aussi lorsque l'argument \code{lag} est négatif, la fonction
se comporte comme la fonction \code{previous_date_ts} et retourne les périodes
passées et non futures.
}
\examples{

next_date_ts(c(2020L, 4L), frequency_ts = 4L, lag = 2L)
next_date_ts(c(2021L, 1L), frequency_ts = 4L, lag = -2L)

next_date_ts(c(2020L, 4L), frequency_ts = 12L, lag = 2L)
next_date_ts(c(2022L, 6L), frequency_ts = 12L, lag = 12L)
}
\seealso{
\code{previous_date_ts}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/formatting.R
\name{normalize_date_ts}
\alias{normalize_date_ts}
\title{Ajuste un objet date_ts dans un format conforme.}
\usage{
normalize_date_ts(date_ts, frequency_ts, test = TRUE)
}
\arguments{
\item{date_ts}{un vecteur numérique, de préférence \code{integer}, au format
\code{AAAA}, \code{c(AAAA, MM)} ou \code{c(AAAA, TT)}}

\item{frequency_ts}{un entier qui vaut \code{4L} (ou \code{4.0}) pour les séries
trimestrielles et \code{12L} (ou \code{12.0}) pour les séries mensuelles.}

\item{test}{un booléen (Default is TRUE)}
}
\value{
En sortie, la fonction retourne une date au même format que l'objet
\code{date_ts} avec la période inclus entre 1 et la fréquence.
}
\description{
Ajuste un objet date_ts dans un format conforme.
}
\details{
Ici le formattage correspond à une réécriture de la date sans
en changer la valeur. Alors que l'objet c(2020L, 12L) désigne le mois de
décembre 2020 et c(2021L, 1L) le mois de janvier 2021, on peut imaginer que
la date_ts c(2021L, 0L) peut aussi représenter le mois de décembre 2020.
Si l'argument \code{test} est mis à FALSE, alors aucun test ne sera effectué sur
les données en entrée.
}
\examples{

# Formattage inchangée
normalize_date_ts(c(2020L, 1L), frequency_ts = 4L) # 1er trimestre de 2020
normalize_date_ts(c(2020L, 8L), frequency_ts = 12L) # Aout 2020

# Retour dans le passé
normalize_date_ts(c(2020L, 0L), frequency_ts = 4L) # 4ème trimestre de 2019
normalize_date_ts(c(2020L, -10L), frequency_ts = 12L) # février 2019

# Avancée dans le futur
normalize_date_ts(c(2020L, 7L), frequency_ts = 4L) # 3ème trimestre de 2021
normalize_date_ts(c(2020L, 13L), frequency_ts = 4L) # janvier 2021

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/conversion.R
\name{ts2df}
\alias{ts2df}
\title{Convertit un objet \code{ts} en \code{data.frame}}
\usage{
ts2df(x)
}
\arguments{
\item{x}{un objet de type \code{ts}.}
}
\value{
En sortie la fonction retourne un \code{data.frame} avec autant de
colonnes que \code{x} et une de plus pour la date.
}
\description{
Convertit un objet \code{ts} en \code{data.frame}
}
\examples{
ts2df(AirPassengers)
ts2df(Seatbelts)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/check.R
\name{check_date_ts}
\alias{check_date_ts}
\alias{assert_date_ts}
\title{Vérifie le format de date}
\usage{
check_date_ts(x, frequency_ts, .var.name = checkmate::vname(x), warn = TRUE)

assert_date_ts(
  x,
  frequency_ts,
  add = NULL,
  .var.name = checkmate::vname(x),
  warn = TRUE
)
}
\arguments{
\item{x}{un vecteur numérique, de préférence \code{integer} au format \code{AAAA},
\code{c(AAAA, MM)} ou \code{c(AAAA, TT)}}

\item{frequency_ts}{un entier qui vaut \code{4L} (ou \code{4.0}) pour les séries
trimestrielles et \code{12L} (ou \code{12.0}) pour les séries mensuelles.}

\item{.var.name}{Nom de l'objet à vérifier pour afficher dans les messages}

\item{warn}{un booleen}

\item{add}{Collection pour stocker les messages d'erreurs (Default is NULL)}
}
\value{
En sortie la fonction retourne l'objet \code{x} de manière invisible ou
une erreur.
}
\description{
La fonction \code{assert_date_ts} vérifie qu'un objet est de type
\code{AAAA}, \code{c(AAAA, MM)} ou \code{c(AAAA, TT)}
}
\details{
Les fonctions du package date4ts sont faites pour fonctionner
avec des times-series de fréquence mensuelle ou trimestrielle et basés sur le
système des mois, trimestres et années classiques.
On cherche donc à favoriser l'utilisation de vecteur \code{c(AAAA, MM)} pour
désigner la date choisie.
Lorsque l'objet \code{x} en entrée est au mauvais format, il est corrigé pendant
la checks et l'objet en sortie est au bon format.
Si l'argument \code{warn} est \code{FALSE}, alors la fonction ne retournera pas de
warning lors de l'évaluation.

Ici, l'argument \code{frequency_ts} est nécessaire car une date sous la forme
c(AAAA, PP), avec PP le nombre de période, ne désigne pas une date absolue.
Par exemple, c(2020L 5L) désigne mai 2020 pour une fréquence mensuelle et le
1er trimestre 2021 pour une fréquence trimestrielle.

Selon le préfixe de la fonction :

\itemize{
\item si le check réussi : \itemize{
\item la fonction \code{assert_date_ts} retourne l'objet \code{x} de manière
invisible;
\item la fonction \code{check_date_ts} retourne le booléen \code{TRUE}.
}
\item si le check échoue : \itemize{
\item la fonction \code{assert_date_ts} retourne un message d'erreur;
\item la fonction \code{check_date_ts} retourne une chaîne de caractère
signalant le problème.
}
}
}
\examples{
# De bons formats de date
assert_date_ts(c(2020L, 8L), frequency_ts = 12L)
assert_date_ts(c(2020L, 2L), frequency_ts = 4L)
check_date_ts(2022L, frequency_ts = 12L)

# Format double --> génération d'un warning
assert_date_ts(c(2020., 4.0), frequency_ts = 4L)
assert_date_ts(2022., frequency_ts = 12L)
check_date_ts(2022., frequency_ts = 12L)

# Fréquence au format double --> génération d'un warning
assert_date_ts(c(2020L, 6L), frequency_ts = 4.0)
assert_date_ts(c(2020L, 42L), frequency_ts = 12.0)

# Dépassement la fréquence --> génération d'un warning
assert_date_ts(c(2020L, 6L), frequency_ts = 4L)
assert_date_ts(c(2020L, 42L), frequency_ts = 12L)
assert_date_ts(c(2020L, -4L), frequency_ts = 12L)

# Avec des erreurs
check_date_ts(1:10, frequency_ts = 12L)

}
